      subroutine dgecom (a,lda,n,ipvt,rcond,z)
      integer lda,n,ipvt(n)
      double precision a(lda,n),z(n)
      double precision rcond
c
c     dgecom factors a double precision matrix by gaussian elimination
c     and estimates the condition of the matrix.
c
c     if rcond is not needed, dgefam is slightly faster.
c     to solve  a*x = b , follow dgecom by dgeslm.
c     to compute  inverse(a)*c , follow dgecom by dgeslm.
c
c     on entry:
c
c        a       double precision(lda,n)
c                the matrix to be factored.
c
c        lda     integer
c                the leading dimension of the array a as declared
c                in the main calling program.
c
c        n       integer
c                the order of the matrix a.
c
c     on return:
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l is a product of permutation and unit lower
c                triangular matrices and u is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c                   ipvt(k) = the index of the k-th pivot row
c                the determinant of a can be obtained on output by
c                   det(a) = s*a(1,1)*a(2,2)* ... *a(n,n)
c                where s = (-1)**(number of times ipvt(k) .ne. k)
c                but this result should be used with caution as it
c                may easily underflow or overflow.
c
c        rcond   double precision
c                an estimate of the reciprocal condition of a.
c                for the system  a*x = b , relative perturbations
c                in a and b of size epsilon may cause relative
c                perturbations in x of size  epsilon/rcond.
c                if rcond is so small that the logical expression
c                             1.0d0 + rcond  .eq.  1.0d0
c                is true, then a may be singular to working
c                precision.  in particular, rcond is zero if
c                exact singularity is detected or the estimate
c                underflows.
c
c        z       double precision(n)
c                a work vector whose contents are usually unimportant.
c                if a is close to a singular matrix, then z is an
c                approximate null vector in the sense that
c                       norm(a*z) = rcond*norm(a)*norm(z).
c
c     this version is adapted from the linpack subroutine dgeco by
c     replacement of calls to the blas with in-line code.  modification
c     done by alan j. laub, university of southern california,
c     april 1980.
c
c     subroutines and functions called:
c
c     dgefam
c
c     fortran functions called:
c
      double precision dabs,dsign
c
c     internal variables:
c
      integer i,info,j,k,kb,km1,kp1,l
      double precision anorm,ek,s,sm,t,wk,wkm,ynorm
c
c     compute 1-norm of a
c
      anorm = 0.0d0
      do 20 j=1,n
         t = 0.0d0
         do 10 i=1,n
            t = t+dabs(a(i,j))
   10    continue
         if (t .gt. anorm) anorm = t
   20 continue
c
c     factor
c
      call dgefam(a,lda,n,ipvt,info)
c
c     rcond = 1/(norm(a)*(estimate of norm(inverse(a)))).
c     estimate = norm(z)/norm(y)  where  a*z = y and  trans(a)*y = e.
c     trans(a)  is the transpose of a.  the components of e are
c     chosen to cause maximum local growth in the elements of w where
c     trans(u)*w = e .  the vectors are frequently rescaled to avoid
c     overflow.
c
c     solve  trans(u)*w = e
c
      ek = 1.0d0
      do 30 i=1,n
         z(i) = 0.0d0
   30 continue
      do 120 k=1,n
         if (z(k) .ne. 0.0d0) ek = dsign(ek,-z(k))
         if (dabs(ek-z(k)) .le. dabs(a(k,k))) go to 50
         s = dabs(a(k,k))/dabs(ek-z(k))
         do 40 i=1,n
            z(i) = s*z(i)
   40    continue
         ek = s*ek
   50    continue
         wk = ek-z(k)
         wkm = -ek-z(k)
         s = dabs(wk)
         sm = dabs(wkm)
         if (a(k,k) .eq. 0.0d0) go to 60
         wk = wk/a(k,k)
         wkm = wkm/a(k,k)
         go to 70
   60    continue
         wk = 1.0d0
         wkm = 1.0d0
   70    continue
         kp1 = k+1
         if (kp1 .gt. n) go to 110
         do 80 j=kp1,n
            sm = sm+dabs(z(j)+wkm*a(k,j))
            z(j) = z(j)+wk*a(k,j)
            s = s+dabs(z(j))
   80    continue
         if (s .ge. sm) go to 100
         t = wkm-wk
         wk = wkm
         do 90 j=kp1,n
            z(j) = z(j)+t*a(k,j)
   90    continue
  100    continue
  110    continue
         z(k) = wk
  120 continue
      t = 0.0d0
      do 130 i=1,n
         t = t+dabs(z(i))
  130 continue
      s = 1.0d0/t
      do 140 i=1,n
         z(i) = s*z(i)
  140 continue
c
c     solve  trans(l)*y = w
c
      do 190 kb=1,n
         k = n+1-kb
         if (k .eq. n) go to 160
         kp1 = k+1
         t = 0.0d0
         do 150 i=kp1,n
            t = t+a(i,k)*z(i)
  150    continue
         z(k) = z(k)-t
  160    continue
         if (dabs(z(k)) .le. 1.0d0) go to 180
         s = 1.0d0/dabs(z(k))
         do 170 i=1,n
            z(i) = s*z(i)
  170    continue
  180    continue
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
  190 continue
      t = 0.0d0
      do 200 i=1,n
         t = t+dabs(z(i))
  200 continue
      s = 1.0d0/t
      do 210 i=1,n
         z(i) = s*z(i)
  210 continue
c
      ynorm = 1.0d0
c
c     solve  l*v = y
c
      do 260 k=1,n
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
         if (k .eq. n) go to 230
         kp1 = k+1
         do 220 i=kp1,n
            z(i) = z(i)-t*a(i,k)
  220    continue
  230    continue
         if (dabs(z(k)) .le. 1.0d0) go to 250
         s = 1.0d0/dabs(z(k))
         do 240 i=1,n
            z(i) = s*z(i)
  240    continue
         ynorm = s*ynorm
  250    continue
  260 continue
      t = 0.0d0
      do 270 i=1,n
         t = t+dabs(z(i))
  270 continue
      s = 1.0d0/t
      do 280 i=1,n
         z(i) = s*z(i)
  280 continue
      ynorm = s*ynorm
c
c     solve  u*z = v
c
      do 330 kb=1,n
         k = n+1-kb
         if (dabs(z(k)) .le. dabs(a(k,k))) go to 300
         s = dabs(a(k,k))/dabs(z(k))
         do 290 i=1,n
            z(i) = s*z(i)
  290    continue
         ynorm = s*ynorm
  300    continue
         if (a(k,k) .ne. 0.0d0) z(k) = z(k)/a(k,k)
         if (a(k,k) .eq. 0.0d0) z(k) = 1.0d0
         t = -z(k)
         if (k .eq. 1) go to 320
         km1 = k-1
         do 310 i=1,km1
            z(i) = z(i)+t*a(i,k)
  310    continue
  320    continue
  330 continue
c
c     make znorm = 1.0d0
c
      t = 0.0d0
      do 340 i=1,n
         t = t+dabs(z(i))
  340 continue
      s = 1.0d0/t
      do 350 i=1,n
         z(i) = s*z(i)
  350 continue
      ynorm = s*ynorm
c
c     determine rcond
c
      if (anorm .ne. 0.0d0) rcond = ynorm/anorm
      if (anorm .eq. 0.0d0) rcond = 0.0d0
      return
      end
