      subroutine eigord(a,z,n,eps,iord,ind,fail,na,nz)
c
c     *****parameters:
      integer n,iord,ind(n),na,nz
      double precision a(na,n),z(nz,n),eps
      logical fail
c
c     *****local variables:
      double precision e1,e2
      integer ifirst,i,ii,iii,is,istep,k,l,ls,ls1,ls2,l1,l2,num
      logical type
c
c     *****functions:
      double precision dabs
      integer isign
c
c     *****subroutines called:
c     exchng
c
c     ---------------------------------------------------------------
c
c     *****purpose:
c     given the upper hessenberg matrix a with 1x1 or 2x2 diagonal
c     blocks, this subroutine reorders the diagonal blocks along with
c     the eigenvalues by unitary similarity transformations.  
c     the transformations are then applied to the matrix z.
c     ref.:  van dooren, p., a generalized eigenvalue approach for
c     solving riccati equations, siam j. sci. stat. comput.,
c     vol. 2, no. 2, june 1981, 121-135.
c
c     *****parameter description:
c     on input:
c       n       integer
c               order of the matrices a,z;
c
c       a       real(na,n)
c               upper hessenberg matrix with 1x1 or 2x2 diagonal
c               blocks.  elements outside the upper hessenberg
c               structure are arbitrary;
c
c       eps     real
c               required absolute accuracy of the results.  normally
c               equal to the machine precision;
c
c       iord    integer
c               control parameter that specifies the regions of the
c               the complex plane by which the eigenvalues are ordered.
c               first below means the upper left block.
c               last means the lower right block.
c               = -2  inside unit circle appears first
c               = -1  left half plane appears first
c               =  0  no ordering is done
c               = +1  right half plane appears first
c               = +2  outside unit circle appears first
c
c       ind     integer(n)
c               working array that is altered by this subroutine.
c
c       na      integer
c               row dimension of the a array as declared 
c               in the calling program dimension statement;
c
c       nz      integer
c               row dimension of the z array as declared 
c               in the calling program dimension statement;
c
c     on output:
c
c       a       real(na,n)
c               upper hessenberg matrix reordered as specified by
c               iord (above);
c
c       z       real(nz,n)
c               this array is overwritten by the product of the
c               contents of the array z (upon entry into this
c               subroutine), and the column transformations zt
c               (calculated by this subroutine);
c
c       fail    logical
c               error flag
c               = .true.   indicates attempted reordering failed
c               = .false.  normal return.
c
c     *****algorithm notes:
c     none.
c
c     *****history:
c     original version that sorted by unit circle region of complex
c     plane written by p. van dooren ("a generalized eigenvalue
c     approach for solving riccati equations", internal report
c     na-80-02, dept. of computer science, stanford university,
c     1980).  modified by w. f. arnold (dept. of electrical 
c     engineering - systems, univ. of southern calif., los angeles
c     , ca  90089) to include the sorting control parameter "type".
c     this version modified by s. g. pratt (honeywell, systems and 
c     research center, minneapolis, mn) to order the ungeneralized
c     eigenvalue problem, and to combine the type and ifirst flags.
c
c     ----------------------------------------------------------------
c
      if (iord .eq. 0) then
	 fail = .false.
	 goto 100
      endif
c
c     compute original flags type and ifirst as determined by iord
c
      type = iord .eq. 1 .or. iord .eq. -1
      ifirst = isign(1,iord)
c
      num = 0
      l = 0
      ls = 1
c
c     determine size and stability of blocks
c
   10 continue
      l = l+ls
      if (l .gt. n) go to 50
      is = -ifirst
      l1 = l+1
      if (l1 .gt. n) go to 20
      if (a(l1,l) .eq. 0.0d0) go to 20
      call split(a,z,n,l,e1,e2,na,nz)
      if (a(l1,l) .eq. 0.0d0) go to 20
c
c     2x2 block
c
      ls = 2
      if (.not. type) go to 15
      if (a(l,l)+a(l1,l1) .lt. 0.0d0) is = ifirst
      go to 30
   15 continue
      if (dabs(a(l,l)*a(l1,l1)-a(l1,l)*a(l,l1)) .lt. 1.0d0)
     +    is = ifirst
      go to 30
c
c     1x1 block
c
   20 continue
      ls = 1
      if (.not. type) go to 25
      if (a(l,l) .lt. 0.0d0) is = ifirst
      go to 30
   25 continue
      if (dabs(a(l,l)) .lt. 1.0d0) is = ifirst
   30 continue
      num = num+1
      ind(num) = ls*is
      go to 10
c
c     reorder blocks
c
   50 continue
      l2 = 1
      i = 0
   60 continue
      i = i+1
      if (ind(i) .gt. 0) go to 70
      l2 = l2-ind(i)
      go to 60
   70 continue
      k = i
   80 continue
      l2 = l2+ind(k)
   85 continue
      k = k+1
      if (k .gt. num) go to 100
      if (ind(k) .gt. 0) go to 80
c
c     interchange block k before block i
c
      istep = k-i
      ls2 = -ind(k)
      l = l2
      do 90 ii = 1,istep
         iii = k-ii
         ls1 = ind(iii)
         l = l-ls1
         call exchng (a,z,n,l,ls1,ls2,eps,fail,na,nz)
         if (fail) goto 100
         ind(iii+1) = ind(iii)
   90 continue
      ind(i) = -ls2
      i = i+1
      l2 = l2+ls2
      go to 85
  100 continue
      return
c
c     last line of eigord
c
      end
