      subroutine hqr3 (a,v,n,nlow,nup,eps,er,ei,itype,na,nv)
c
c     *****parameters:
      integer n,na,nlow,nup,nv,itype(n)
      double precision a(na,n),ei(n),er(n),eps,v(nv,n)
c
c     *****local variables:
      logical fail
      integer i,it,l,mu,nl,nu
      double precision e1,e2,p,q,r,s,t,w,x,y,z
c
c     *****functions:
      double precision dabs
c
c     *****subroutines called:
c     exchng,qrstep,split
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine reduces the upper hessenberg matrix a to quasi-
c     triangular form by unitary similarity transformations. the
c     eigenvalues of a, which are contained in the 1 x 1 and 2 x 2
c     diagonal blocks of the reduced matrix, are ordered in descending
c     order of magnitude along the diagonal. the transformations are
c     accumulated in the array v.
c
c     *****parameter description:
c     on input:
c        na,nv            row dimensions of the arrays containing a and
c                         v, respectively, as declared in the calling
c                         program dimension statement;
c
c        a                n x n array containing the upper hessenberg
c                         matrix to be reduced;
c
c        n                order of the matrices a and v;
c
c        nlow,nup         a(nlow,nlow-1) and a(nup,1+nup) are assumed
c                         to be zero, and only rows nlow through nup
c                         and columns nlow through nup are transformed,
c                         resulting in the calculation of eigenvalues
c                         nlow through nup;
c
c        eps              a convergence criterion used to determine when
c                         a subdiagonal element of a is negligible.
c                         specifically, a(i+1,i) is regarded as
c                         negligible if dabs(a(i+1),i)).le.eps*
c                         (dabs(a(i+1,i+1))). this means that the final
c                         matrix returned by the program will be exactly
c                         similar to a + e where e is of order
c                         eps*norm(a), for any reasonably balanced norm
c                         such as the row-sum norm;
c
c        itype            an integer vector of length n whose
c                         i-th entry is
c                         0  if the i-th eigenvalue is real,
c                         1  if the i-th eigenvalue is complex with
c                            positive imaginary part,
c                         2  if the i-th eigenvalue is complex with
c                            negative imaginary part,
c                         -1 if the i-th eigenvalue was not calculated
c                            successfully.
c
c        on output:
c
c        a                n x n array containing the reduced, quasi-
c                         triangular matrix;
c
c        v                n x n array containing the reducing
c                         transformations to be multiplied;
c
c        er,ei            real scratch vectors of length n which on
c                         return contain the real and imaginary parts,
c                         respectively, of the eigenvalues.
c
c     *****history:
c              adapted from stewart's hqr3 software.
c
c     ------------------------------------------------------------------
c
      do 10 i=nlow,nup
         itype(i)=-1
10    continue
      t=0.0d0
      nu=nup
20    if (nu.lt.nlow) go to 240
      it=0
30    continue
      l=nu
40    continue
      if (l.eq.nlow) go to 50
      if (dabs(a(l,l-1)).le.eps*(dabs(a(l-1,l-1))+dabs(a(l,l))))
     +    go to 50
      l=l-1
      go to 40
50    continue
      x=a(nu,nu)
      if (l.eq.nu) go to 160
      y=a(nu-1,nu-1)
      w=a(nu,nu-1)*a(nu-1,nu)
      if (l.eq.nu-1) go to 100
      if (it.eq.30) go to 240
      if (it.ne.10 .and. it.ne.20) go to 70
      t=t+x
      do 60 i=nlow,nu
         a(i,i)=a(i,i)-x
60    continue
      s=dabs(a(nu,nu-1))+dabs(a(nu-1,nu-2))
      x=0.75d0*s
      y=x
      w=-0.4375d0*s**2
70    continue
      it=it+1
      nl=nu-2
80    continue
      z=a(nl,nl)
      r=x-z
      s=y-z
      p=(r*s-w)/a(nl+1,nl)+a(nl,nl+1)
      q=a(nl+1,nl+1)-z-r-s
      r=a(nl+2,nl+1)
      s=dabs(p)+dabs(q)+dabs(r)
      p=p/s
      q=q/s
      r=r/s
      if (nl.eq.l) go to 90
      if (dabs(a(nl,nl-1))*(dabs(q)+dabs(r)).le.eps*dabs(p)*
     +    (dabs(a(nl-1,nl-1))+dabs(z)+dabs(a(nl+1,nl+1)))) go to 90
      nl=nl-1
      go to 80
90    continue
      call qrstep (a,v,p,q,r,nl,nu,n,na,nv)
      go to 30
100   if (nu.ne.nlow+1) a(nu-1,nu-2)=0.0d0
      a(nu,nu)=a(nu,nu)+t
      a(nu-1,nu-1)=a(nu-1,nu-1)+t
      itype(nu)=0
      itype(nu-1)=0
      mu=nu
110   continue
      nl=mu-1
      call split (a,v,n,nl,e1,e2,na,nv)
      if (a(mu,mu-1).eq.0.0d0) go to 170
      if (mu.eq.nup) go to 230
      if (mu.eq.nup-1) go to 130
      if (a(mu+2,mu+1).eq.0.0d0) go to 130
      if (a(mu-1,mu-1)*a(mu,mu)-a(mu-1,mu)*a(mu,mu-1).ge.a(mu+1,mu+1)*
     +    a(mu+2,mu+2)-a(mu+1,mu+2)*a(mu+2,mu+1)) go to 230
      call exchng (a,v,n,nl,2,2,eps,fail,na,nv)
      if (.not.fail) go to 120
      itype(nl)=-1
      itype(nl+1)=-1
      itype(nl+2)=-1
      itype(nl+3)=-1
      go to 240
120   continue
      mu=mu+2
      go to 150
130   continue
      if (a(mu-1,mu-1)*a(mu,mu)-a(mu-1,mu)*a(mu,mu-1).ge.
     +    a(mu+1,mu+1)**2) go to 230
      call exchng (a,v,n,nl,2,1,eps,fail,na,nv)
      if (.not.fail) go to 140
      itype(nl)=-1
      itype(nl+1)=-1
      itype(nl+2)=-1
      go to 240
140   continue
      mu=mu+1
150   continue
      go to 110
160   nl=0
      a(nu,nu)=a(nu,nu)+t
      if (nu.ne.nlow) a(nu,nu-1)=0.0d0
      itype(nu)=0
      mu=nu
170   continue
180   continue
      if (mu.eq.nup) go to 220
      if (mu.eq.nup-1) go to 200
      if (a(mu+2,mu+1).eq.0.0d0) go to 200
      if (a(mu,mu)**2.ge.a(mu+1,mu+1)*a(mu+2,mu+2)-a(mu+1,mu+2)*
     +    a(mu+2,mu+1)) go to 220
      call exchng (a,v,n,mu,1,2,eps,fail,na,nv)
      if (.not.fail) go to 190
      itype(mu)=-1
      itype(mu+1)=-1
      itype(mu+2)=-1
      go to 240
190   continue
      mu=mu+2
      go to 210
200   continue
      if (dabs(a(mu,mu)).ge.dabs(a(mu+1,mu+1))) go to 220
      call exchng (a,v,n,mu,1,1,eps,fail,na,nv)
      mu=mu+1
210   continue
      go to 180
220   continue
      mu=nl
      nl=0
      if (mu.ne.0) go to 170
230   continue
      nu=l-1
      go to 20
240   if (nu.lt.nlow) go to 260
      do 250 i=nlow,nu
         a(i,i)=a(i,i)+t
250   continue
260   continue
      nu=nup
270   continue
      if (itype(nu).ne.(-1)) go to 280
      nu=nu-1
      go to 310
280   continue
      if (nu.eq.nlow) go to 290
      if (a(nu,nu-1).eq.0.0d0) go to 290
      call split (a,v,n,nu-1,e1,e2,na,nv)
      if (a(nu,nu-1).eq.0.0d0) go to 290
      er(nu)=e1
      ei(nu-1)=e2
      er(nu-1)=er(nu)
      ei(nu)=-ei(nu-1)
      itype(nu-1)=1
      itype(nu)=2
      nu=nu-2
      go to 300
290   continue
      er(nu)=a(nu,nu)
      ei(nu)=0.0d0
      nu=nu-1
300   continue
310   continue
      if (nu.ge.nlow) go to 270
      return
c
c     last line of hqr3
c
      end
