	subroutine lypc (f,u,h,n,er,ei,work,info,nf,nu,nh)
c
c   subroutines used  -  orthes, ortran, hqr4, symslc
c
c   lypc is a fortran subroutine to solve the real matrix
c   equation trans(f)*x + x*f + h = 0, where h is symmetric and
c   trans(f) denotes the transpose of f.  the equation is
c   transformed so that f is in upper real schur form, and the
c   transformed equation is solved by a recursive procedure.
c
c   the parameters in the argument list are
c
c	f	a doubly subscripted array containing the
c		matrix f.  on return, the upper triangle
c		and the first subdiagonal of the array f
c		contain an upper real schur form of f.
c
c	u	a doubly subscripted array that, on
c		return, contains the orthogonal matrix
c		that reduces a to upper real schur form.
c
c	h	a doubly subscripted array containing the
c		matrix h.  on return, h contains the
c		solution matrix x.
c
c	n	the order of the matrix f.
c
c	er	an array of length n that, on return,
c		contains the real part of the eigenvalues
c		of a.
c
c	ei	an array of length n that, on return,
c		contains the imaginary part of the eigenvalues
c		of a.
c
c	work	a scratch vector of length n.
c
c	info	an integer variable that, on entry, contains a flag
c		whose value is zero if the matrix a is already in
c		real schur form.  on return, it contains an error
c		signal whose value is
c		    0  if the solution was found,
c		    1  if the real schur decompostion failed,
c		    2  if at least one of the eigenvalues is unstable.
c
c	nf	the first dimension of the array f.
c
c	nu	the first dimension of the array u.
c
c	nh	the first dimension of the array h.
c
c   when info is zero, the reduction of f to real schur
c   form is skipped and the arrays f and u are assumed to
c   contain the schur form and accompanying orthogonal matrix.
c   this permits the efficient solution of several equations
c   with different h matrices.
c
	real*8 f(nf,*),u(nu,*),h(nh,*),er(*),ei(*),work(*),eps
	integer n,nf,nu,nh,ifail,info,i,j,k,nlow,nup
c
	eps = 1.0
    5	continue
	    eps = 0.5*eps
	if (eps+1.0 .gt. 1.0) goto 5
	eps = 2.0*eps
c
c   if required, reduce a to upper real schur form
c
	if (info .eq. 0) goto 15
	nlow = 1
	nup = n
	call orthes (nf,n,nlow,nup,f,work)
	call ortran (nf,n,nlow,nup,f,work,u)
	call hqr4 (f,u,n,nlow,nup,eps,ifail,nf,nu)
	if (ifail .ne. 0) then
	    info = 1
	    return
	endif
	call eigval (f,u,n,nlow,nup,er,ei,nf,nu)
	info = 0
	do 10 i=1,n
	    if (er(i) .ge. 0.0) then
		info = 2
		goto 15
	    endif
   10	continue
c
c   transform h
c
   15	continue
	do 20 i=1,n
	    h(i,i) = h(i,i)/2.0
   20	continue
	do 45 i=1,n
	    do 35 j=1,n
		work(j) = 0.0
		do 30 k=i,n
		    work(j) = work(j) + h(i,k)*u(k,j)
   30		continue
   35	    continue
	    do 40 j=1,n
		h(i,j) = work(j)
   40	    continue
   45	continue
	do 65 j=1,n
	    do 55 i=1,n
		work(i) = 0.0
		do 50 k=1,n
		    work(i) = work(i) + u(k,i)*h(k,j)
   50		continue
   55	    continue
	    do 60 i=1,n
		h(i,j) = work(i)
   60	    continue
   65	continue
	do 72 i=1,n
	    do 70 j=i,n
		h(i,j) = h(i,j) + h(j,i)
		h(j,i) = h(i,j)
   70	    continue
   72	continue
c
c   move h matrix to right hand side for symslc
c
	do 77 i=1,n
	    do 75 j=1,n
		h(i,j) = -h(i,j)
   75	    continue
   77	continue
c
c   solve the transformed system
c
	call symslc (f,h,n,nf,nh)
c
c   transform h back to the solution
c
	do 80 i=1,n
	    h(i,i) = h(i,i)/2.0
   80	continue
	do 105 i=1,n
	    do 95 j=1,n
		work(j) = 0.0
		do 90 k=i,n
		    work(j) = work(j) + h(i,k)*u(j,k)
   90		continue
   95	    continue
	    do 100 j=1,n
		h(i,j) = work(j)
  100	    continue
  105	continue
	do 125 j=1,n
	    do 115 i=1,n
		work(i) = 0.0
		do 110 k=1,n
		    work(i) = work(i) + u(i,k)*h(k,j)
  110		continue
  115	    continue
	    do 120 i=1,n
		h(i,j) = work(i)
  120	    continue
  125	continue
	do 140 i=1,n
	    do 130 j=i,n
		h(i,j) = h(i,j) + h(j,i)
		h(j,i) = h(i,j)
  130	    continue
  140	continue
	return
	end
