      subroutine lypsol (f,u,h,n,er,ei,work,info,nf,nu,nh)
c
c subroutines used  -  orthes, ortran, hqr3, symslv
c
c lypsol is a fortran subroutine to solve the real matrix
c equation trans(f)*x + x*f + h = 0, where h is symmetric and
c trans(f) denotes the transpose of f.  the equation is
c transformed so that f is in upper real schur form, and the
c transformed equation is solved by a recursive procedure.
c the parameters in the argument list are
c           f      a doubly subscripted array containing the
c                  matrix f.  on return, the upper triangle
c                  and the first subdiagonal of the array f
c                  contain an upper real schur form of f.
c
c           u      a doubly subscripted array that, on
c                  return, contains the orthogonal matrix
c                  that reduces a to upper real schur form.
c
c           h      a doubly subscripted array containing the
c                  matrix h.  on return, h contains the
c                  solution matrix x.
c
c           n      the order of the matrix f.
c
c           er     an array of length n that, on return,
c              	   contains the real part of the eigenvalues
c                  of a.
c
c           ei     an array of length n that, on return,
c              	   contains the imaginary part of the eigenvalues
c                  of a.
c
c           work   a scratch vector of length n.
c
c           info   an integer variable that, on entry,
c                  contains a flag whose value is zero if
c                  the matrix a is already in real schur form.
c                  on return, it contains an error signal whose value is
c                  0  if the solution was found,
c                  1  if the real schur decompostion was not successful,
c                  2  if at least one of the eigenvalues is unstable.
c
c           nf     the first dimension of the array f.
c
c           nu     the first dimension of the array u.
c
c           nh     the first dimension of the array h.
c
c when info is zero, the reduction of f to real schur
c form is skipped and the arrays f and u are assumed to
c contain the schur form and accompanying orthogonal matrix.
c this permits the efficient solution of several equations
c with different h matrices.
c
      real*8 f(nf,*),u(nu,*),h(nh,*),er(*),ei(*),work(*),eps
      integer n,nf,nu,nh,ifail,info,i,j,k,nlow,nup
c
      eps = 1.0d0
    5 continue
      eps = eps/2.0d0
      if (eps+1.0d0 .gt. 1.0d0) goto 5
      eps = 2.0d0*eps
c
c if required, reduce a to upper real schur form.
c
      if (info .eq. 0) goto 15
      nlow = 1
      nup = n
      call orthes (nf,n,nlow,nup,f,work)
      call ortran (nf,n,nlow,nup,f,work,u)
      call hqr4 (f,u,n,nlow,nup,eps,ifail,nf,nu)
      if (ifail .ne. 0) then
	 info = 1
	 goto 140
      endif
      call eigval (f,u,n,nlow,nup,er,ei,nf,nu)
      info = 0
      do 10 i=1,n
      if (er(i) .ge. 0.0) then
	 info = 2
	 goto 15
      endif
   10 continue
c
c transform h.
c
   15 continue
      do 20 i=1,n
        h(i,i) = h(i,i)/2.0d0
   20 continue
      do 40 i=1,n
        do 30 j=1,n
          work(j) = 0.0d0
          do 30 k=i,n
            work(j) = work(j) + h(i,k)*u(k,j)
   30   continue
        do 40 j=1,n
          h(i,j) = work(j)
   40 continue
      do 60 j=1,n
        do 50 i=1,n
          work(i) = 0.0d0
          do 50 k=1,n
            work(i) = work(i) + u(k,i)*h(k,j)
   50   continue
        do 60 i=1,n
          h(i,j) = work(i)
   60 continue
      do 70 i=1,n
        do 70 j=i,n
          h(i,j) = h(i,j) + h(j,i)
          h(j,i) = h(i,j)
   70 continue
c
c move h matrix to right hand side for symslv.
c
      do 75 i=1,n
        do 75 j=1,n
          h(i,j) = -h(i,j)
   75 continue
c
c solve the transformed system.
c
      call symslv(f,h,n,nf,nh)
c
c transform h back to the solution.
c
      do 80 i=1,n
        h(i,i) = h(i,i)/2.0d0
   80 continue
      do 100 i=1,n
        do 90 j=1,n
          work(j) = 0.0d0
          do 90 k=i,n
            work(j) = work(j) + h(i,k)*u(j,k)
   90   continue
        do 100 j=1,n
          h(i,j) = work(j)
  100 continue
      do 120 j=1,n
        do 110 i=1,n
          work(i) = 0.0d0
          do 110 k=1,n
            work(i) = work(i) + u(i,k)*h(k,j)
  110   continue
        do 120 i=1,n
          h(i,j) = work(i)
  120 continue
      do 130 i=1,n
        do 130 j=i,n
          h(i,j) = h(i,j) + h(j,i)
          h(j,i) = h(i,j)
  130 continue
  140 continue
      return
      end
