      subroutine order(a,b,z,nmax,n,eps,ifail,type,ifirst,ind)
c
c     *****parameters:
      integer ifail,ifirst,n,nmax,ind(n)
      double precision eps,a(nmax,n),b(nmax,n),z(nmax,n)
      logical type
c
c     *****local variables:
      integer i,ii,iii,is,istep,k,l,ls,ls1,ls2,l1,l2,num
c
c     *****functions:
      double precision dabs
c
c     *****subroutines called:
c     exchqz
c
c     ---------------------------------------------------------------
c
c     *****purpose:
c     given the upper triangular matrix b and upper hessenberg
c     matrix a with 1x1 or 2x2 diagonal blocks, this subroutine
c     reorders the diagonal blocks along with the generalized
c     eigenvalues corresponding to the regular matrix pencil
c     a - lambda*b  by constructing row and column equivalence
c     transformations qt and zt.  the column transformations are
c     then applied to the matrix z.
c     ref.:  van dooren, p., a generalized eigenvalue approach for
c     solving riccati equations, siam j. sci. stat. comput.,
c     vol. 2, no. 2, june 1981, 121-135.
c
c     *****parameter description:
c     on input:
c       nmax    integer
c               row dimension of the arrays containing a,b,z as
c               declared in the calling program dimension statement;
c
c       n       integer
c               order of the matrices a,b,z;
c
c       a       real(nmax,n)
c               upper hessenberg matrix with 1x1 or 2x2 diagonal
c               blocks.  elements outside the upper hessenberg
c               structure are arbitrary;
c
c       b       real(nmax,n)
c               upper triangular matrix.  elements outside the
c               upper triangular structure are arbitrary;
c
c       eps     real
c               required absolute accuracy of the results.  normally
c               equal to the machine precision;
c
c       type    logical
c               control parameter that specifies the regions of the
c               the complex plane that the generalized eigenvalues
c               are ordered by.  to control the region that appears
c               first, see ifirst below
c               = .true.  generalized eigenvalues are ordered by
c               region inside the complex left half plane or
c               outside this region
c               = .false.  generalized eigenvalues ordered by region
c               inside the unit circle or outside this region;
c
c       ifirst  integer
c               control parameter that specifies which of the regions
c               specified by type(above) appears first(i.e. in the
c               upper left nxn block)
c               = -1  inside region appears first
c               = +1  outside region appears first
c               ifirst is altered by this subroutine;
c
c       ind     integer(n)
c               working array that is altered by this subroutine.
c
c     on output:
c
c       a       real(nmax,n)
c               upper hessenberg matrix reordered as specified by
c               type and ifirst(above);
c
c       b       real(nmax,n)
c               upper triangular matrix reordered as specified by
c               type and ifirst(above);
c
c       z       real(nmax,n)
c               this array is overwritten by the product of the
c               contents of the array z(upon entry into this
c               subroutine), and the column transformations zt
c               (calculated by this subroutine);
c
c       ifail   integer
c               error flag
c               = 1  indicates attempted reordering failed
c               = 0  normal return.
c
c     *****algorithm notes:
c     none.
c
c     *****history:
c     original version that sorted by unit circle region of complex
c     plane written by p. van dooren("a generalized eigenvalue
c     approach for solving riccati equations", internal report
c     na-80-02, dept. of computer science, stanford university,
c     1980).  this version modified by w. f. arnold(dept. of
c     electrical engineering - systems, univ. of southern calif.,
c     los angeles, ca  90089) to include the sorting control parameter
c     "type", sept 1982.
c
c     ----------------------------------------------------------------
c
      ifail = 1
      num = 0
      l = 0
      ls = 1
c
c     determine size and stability of blocks
c
   10 continue
      l = l+ls
      if (l .gt. n) go to 50
      is = -ifirst
      l1 = l+1
      if (l1 .gt. n) go to 20
      if (a(l1,l) .eq. 0.0d0) go to 20
c
c     2x2 block
c
      ls = 2
      if (.not. type) go to 15
      if ((a(l,l)*b(l1,l1)+a(l1,l1)*b(l,l)-a(l1,l)*b(l,l1))/
     x    (b(l,l)*b(l1,l1)) .lt. 0.0d0) is = ifirst
      go to 30
   15 continue
      if (dabs(a(l,l)*a(l1,l1)-a(l1,l)*a(l,l1)) .lt.
     +    dabs(b(l,l)*b(l1,l1))) is= ifirst
      go to 30
c
c     1x1 block
c
   20 continue
      ls = 1
      if (.not. type) go to 25
      if (a(l,l)*b(l,l) .lt. 0.0d0) is = ifirst
      if (a(l,l) .lt. 0.0d0 .and. b(l,l) .eq. 0.0d0) is = ifirst
      go to 30
   25 continue
      if (dabs(a(l,l)) .lt. dabs(b(l,l))) is = ifirst
   30 continue
      num = num+1
      ind(num) = ls*is
      go to 10
c
c     reorder blocks
c
   50 continue
      l2 = 1
      i = 0
   60 continue
      i = i+1
      if (ind(i) .gt. 0) go to 70
      l2 = l2-ind(i)
      go to 60
   70 continue
      k = i
   80 continue
      l2 = l2+ind(k)
   85 continue
      k = k+1
      if (k .gt. num) go to 100
      if (ind(k) .gt. 0) go to 80
c
c     interchange block k before block i
c
      istep = k-i
      ls2 = -ind(k)
      l = l2
      do 90 ii = 1,istep
         iii = k-ii
         ls1 = ind(iii)
         l = l-ls1
         call exchqz (a,b,z,nmax,n,l,ls1,ls2,eps,ifail)
         if (ifail .eq. 1) return
         ind(iii+1) = ind(iii)
   90 continue
      ind(i) = -ls2
      i = i+1
      l2 = l2+ls2
      go to 85
  100 continue
      ifail = 0
      return
c
c     last line of order
c
      end
