      subroutine riccnd (nf,ng,nh,nz,n,nn,f,g,h,z,w,er,ei,work,
     +                   scale,itype,ipvl,ipvs)
c
c     *****parameters:
      integer nf,ng,nh,nz,n,nn,itype(nn),ipvl(nn),ipvs(n)
      double precision f(nf,n),g(ng,n),h(nh,n),z(nz,nn),w(nz,nn),
     +                 er(nn),ei(nn),work(n),scale(nn)
c
c     *****local variables:
      integer i,j,low,igh,nlow,npi,npj,nup
      double precision eps,epsp1,znorm,t,alpha,cond
c
c     *****functions:
      double precision dabs,dsqrt
c
c     *****subroutines called:
c     hqr3,mlineq
c     balanc,balbak,orthes,ortran (eispack)
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine solves the continuous-time algebraic
c     matrix riccati equation
c                      t
c                     f *x + x*f - x*g*x + h = 0
c
c     by laub's variant of the hamiltonian-eigenvector approach.
c     ref.: laub, a.j., a schur method for solving algebraic riccati
c           equations, ieee trans. aut. contr., ac-24(1979), 913-921.
c
c     *****parameter description:
c     on input:
c        nf,ng,nh,nz      row dimensions of the arrays containing
c                         f,g,h,z (and w), respectively, as
c                         declared in the calling program dimension
c                         statement;
c
c        n                order of the matrices f,g,h;
c
c        nn               = 2*n = order of the internally generated
c                         matrices z and w;
c
c        f                n x n (real) matrix;
c
c        g,h              n x n symmetric, nonnegative definite
c                         (real) matrices.
c
c     on output:
c
c        h                n x n array containing the unique positive
c                         (or nonnegative) definite solution of the
c                         riccati equation;
c
c        er,ei            real scratch vectors of length 2*n; on output
c                         (er(i),ei(i)), i=1,n contain the real and
c                         imaginary parts, respectively, of the n
c                         closed loop eigenvalues (i.e., the
c                         spectrum of f - g*x);
c
c        z,w              2*n x 2*n real scratch arrays used for
c                         computations involving the hamiltonian
c                         matrix associated with the riccati equation;
c
c        work,scale       real scratch vectors of lengths n, 2*n,
c                         respectively;  on output, work(1) contains a
c                         condition number estimate for the final nth
c                         order linear matrix equation solved;
c
c        itype,ipvl,ipvs  integer scratch vectors of lengths 2*n, 2*n,
c                         n, respectively.
c
c     ***note:  all scratch arrays must be declared and included
c               in the call.***
c
c     *****algorithm notes:
c     it is assumed that g and h are nonnegative definite and that (f,b)
c                                                      t
c     is stabilizable and (c,f) is detectable where b*b  = g
c                                       t
c     (b of full rank = rank(g)) and c*c = h (c of full
c     rank = rank(h)) in which case the solution (returned in the
c     array h) is unique and nonnegative definite.
c
c     *****history:
c     written by alan j. laub (dep't. of elec. engrg.-systems, univ.
c     of southern calif., los angeles, ca 90007; ph.: (213) 743-5535),
c     sep. 1977.
c     most recent version:  apr. 15, 1981.
c
c     ------------------------------------------------------------------
c
c     eps is an internally generated machine dependent parameter
c     specifying the relative precision of floating point arithmetic.
c     for example, eps = 16.0d0**(-13) for double precision arithmetic
c     on ibm s360/s370.
c
      eps=1.0d0
    5 eps=eps/2.0d0
      epsp1=eps+1.0d0
      if (epsp1.gt.1.0d0) go to 5
      eps=2.0d0*eps
c
c     set up hamiltonian matrix
c
      do 20 j=1,n
         npj=n+j
         do 10 i=1,n
            npi=n+i
            z(i,j)=f(i,j)
            z(npi,j)=-h(i,j)
            z(i,npj)=-g(i,j)
            z(npi,npj)=-f(j,i)
   10    continue
   20 continue
c
c     balance z
c
      call balanc (nz,nn,z,low,igh,scale)
c
c     compute 1-norm of z
c
      znorm=0.0d0
      do 40 j=1,nn
         t=0.0d0
         do 30 i=1,nn
            t=t+dabs(z(i,j))
   30    continue
         if (t.gt.znorm) znorm=t
   40 continue
      alpha=dsqrt(znorm)+1.0d0
c
c                              -1
c     compute w = (alpha*i + z)  *(alpha*i - z), an analytic function
c     of z mapping the left half plane to the exterior of the unit
c     disk.  this permits direct application of hqr3.  this step may
c     be removed if hqr3 is modified appropriately.
c
      do 60 j=1,nn
         do 50 i=1,nn
            w(i,j)=-z(i,j)
   50    continue
         w(j,j)=alpha+w(j,j)
         z(j,j)=alpha+z(j,j)
   60 continue
      call mlineq (nz,nz,nn,nn,z,w,cond,ipvl,er)
c
c     reduce w to real schur form with eigenvalues outside the unit
c     disk in the upper left n x n upper quasi-triangular block
c
      nlow=1
      nup=nn
      call orthes (nz,nn,nlow,nup,w,er)
      call ortran (nz,nn,nlow,nup,w,er,z)
      call hqr3 (w,z,nn,nlow,nup,eps,er,ei,itype,nz,nz)
c
c     compute solution of the riccati equation from the orthogonal
c     matrix now in the array z.  store the result in the array h.
c
      call balbak (nz,nn,low,igh,scale,nn,z)
      k = 0
      do 80 i=1,n
   65    continue
         k = k + 1
         if (er(k)**2+ei(k)**2.lt.1.0d0) goto 65
         er(i) = er(k)
         ei(i) = ei(k)
         do 70 j=1,n
            f(i,j)=z(j,k)
            h(i,j)=z(j+n,k)
   70    continue
   80 continue
      call mlineq (nf,nh,n,n,f,h,cond,ipvs,work)
      work(1)=cond
c
c     transform back to get the closed loop spectrum
c
      do 110 i=1,n
         if (itype(i).ge.0) go to 90
         write (0,*) i,' th eigenvalue not successfully calculated'
         return
   90    if (itype(i).gt.0) go to 100
         er(i)=alpha*(1.0d0-er(i))/(1.0d0+er(i))
         ei(i)=0.0d0
         go to 110
  100    if (itype(i).eq.2) go to 110
         t=alpha/((1.0d0+er(i))**2+ei(i)**2)
         er(i)=(1.0d0-er(i)**2-ei(i)**2)*t
         ei(i)=-2.0d0*ei(i)*t
         er(i+1)=er(i)
         ei(i+1)=-ei(i)
  110 continue
      return
c
c     last line of riccnd
c
      end
