      subroutine ricdsd (nf,ng,nh,nz,n,nn,f,g,h,z,w,er,ei,work,
     +                   scale,itype,ipvt)
c
c     *****parameters:
      integer nf,ng,nh,nz,n,nn,itype(nn),ipvt(n)
      double precision f(nf,n),g(ng,n),h(nh,n),z(nz,nn),w(nz,nn),
     +                 er(nn),ei(nn),work(n),scale(nn)
c
c     *****local variables:
      integer i,j,k,low,igh,nlow,npi,npj,nup
      double precision eps,epsp1,t,cond,condp1
c
c     *****subroutines called:
c     dgecom,dgeslm,hqr3,mlineq,mulwoa,mulwob
c     balanc,balbak,orthes,ortran (eispack)
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine solves the discrete-time algebraic matrix
c     riccati equation
c
c                T        T               T      -1    T
c           X = F *X*F - F *X*G1*((G2 + G1 *X*G1)  )*G1 *X*F + H
c
c     by laub's variant of the hamiltonian-eigenvector approach.
c     ref.: laub, a.j., a schur method for solving algebraic riccati
c           equations, ieee trans. aut. contr., ac-24(1979), 913-921.
c     the matrix f is assumed to be nonsingular and the matrices g1 and
c     g2 are assumed to be combined into the square array g as follows:
c                                    -1   t
c                           g = g1*g2  *g1
c
c     in case f is singular, see: pappas, t., a.j. laub, and n.r.
c       sandell, on the numerical solution of the discrete-time
c       algebraic riccati equation, ieee trans. aut. contr., ac-25(1980),
c       631-641.
c
c     *****parameter description:
c     on input:
c        nf,ng,nh,nz      row dimensions of the arrays containing
c                         f,g,h,z (and w), respectively, as
c                         declared in the calling program dimension
c                         statement;
c
c        n                order of the matrices f,g,h;
c
c        nn               = 2*n = order of the internally generated
c                         matrices z and w;
c
c        f                a nonsingular n x n (real) matrix;
c
c        g,h              n x n symmetric, nonnegative definite
c                         (real) matrices.
c
c     on output:
c
c        h                n x n array containing the unique positive
c                         (or nonnegative) definite solution of the
c                         riccati equation;
c
c        er,ei            real scratch vectors of length 2*n; on output
c                         (er(i),ei(i)), i=1,n  contain the real and
c                         imaginary parts, respectively, of the n
c                         closed loop eigenvalues (i.e., the
c                         spectrum of                 t      -1    t
c                                     f - g1*((g2 + g1 *x*g1)  )*g1 *x*f
c                                            -t
c                                   = f - g*f  *(x - h));
c
c        z,w              2*n x 2*n real scratch arrays used for
c                         computations involving the symplectic
c                         matrix associated with the riccati equation;
c
c        work,scale       real scratch vectors of lengths n, 2*n,
c                         respectively;  on output, work(1) contains a
c                         condition number estimate for the final nth
c                         order linear matrix equation solved;
c
c        itype,ipvt       integer scratch vectors of lengths 2*n, n,
c                         respectively.
c
c     ***note:  all scratch arrays must be declared and included
c               in the call.***
c
c     *****algorithm notes:
c     it is assumed that:
c        (1)  f is nonsingular (can be relaxed; see ref. above)
c        (2)  g and h are nonnegative definite
c        (3)  (f,g1) is stabilizable and (c,f) is detectable where
c              t
c             c *c = h (c of full rank = rank(h)).
c     under these assumptions the solution (returned in the array h) is
c     unique and nonnegative definite.
c
c     *****history:
c     written by alan j. laub (dep't. of elec. engrg. - systems, univ.
c     of southern calif., los angeles, ca 90007; ph.: (213) 743-5535),
c     sep. 1977.
c     most recent version: apr. 15, 1981.
c
c     ------------------------------------------------------------------
c
c     eps is an internally generated machine dependent parameter
c     specifying the relative precision of floating point arithmetic.
c     for example, eps = 16.0d0**(-13) for double precision arithmetic
c     on ibm s360/s370.
c
      eps=1.0d0
    5 eps=eps/2.0d0
      epsp1=eps+1.0d0
      if (epsp1.gt.1.0d0) go to 5
      eps=2.0d0*eps
c
c     set up symplectic matrix z
c
      do 20 j=1,n
         npj=n+j
         do 10 i=1,n
            npi=n+i
            z(npi,npj)=f(j,i)
   10    continue
   20 continue
      call dgecom (f,nf,n,ipvt,cond,work)
      if ((cond+1.0d0) .gt. 1.0d0) go to 30
      write (0,*) 'f matrix is singular to working precision'
      return
   30 continue
      do 60 j=1,n
         do 40 i=1,n
            work(i)=0.0d0
   40    continue
         work(j)=1.0d0
         call dgeslm (f,nf,n,ipvt,work)
         do 50 i=1,n
            z(i,j)=work(i)
   50    continue
   60 continue
      do 80 j=1,n
         do 70 i=1,n
            f(i,j)=z(i,j)
   70    continue
   80 continue
      call mulwoa (nh,nf,n,h,f,work)
      do 120 j=1,n
         npj=n+j
         do 90 i=1,n
            npi=n+i
            z(i,npj)=0.0d0
            z(npi,j)=h(i,j)
   90    continue
         do 110 k=1,n
            do 100 i=1,n
               z(i,npj)=z(i,npj)+f(i,k)*g(k,j)
  100       continue
  110    continue
  120 continue
      call mulwob (nh,ng,n,h,g,work)
      do 140 j=1,n
         npj=n+j
         do 130 i=1,n
            npi=n+i
            z(npi,npj)=z(npi,npj)+g(i,j)
  130    continue
  140 continue
c
c     balance z
c
      call balanc (nz,nn,z,low,igh,scale)
c
c     reduce z to real schur form with eigenvalues outside the unit
c     disk in the upper left n x n upper quasi-triangular block
c
      nlow=1
      nup=nn
      call orthes (nz,nn,nlow,nup,z,er)
      call ortran (nz,nn,nlow,nup,z,er,w)
      call hqr3 (z,w,nn,nlow,nup,eps,er,ei,itype,nz,nz)
c
c     compute solution of the riccati equation from the orthogonal
c     matrix now in the array w.  store the result in the array h.
c
      call balbak (nz,nn,low,igh,scale,nn,w)
      do 160 j=1,n
         npj=n+j
         do 150 i=1,n
            f(i,j)=w(j,i)
            h(i,j)=w(npj,i)
  150    continue
  160 continue
      call mlineq (nf,nh,n,n,f,h,cond,ipvt,work)
      work(1)=cond
c
c     transform to get the closed loop spectrum
c
      do 190 i=1,n
         if (itype(i).ge.0) go to 170
         write (0,*) i,' th eigenvalue not successfully calculated'
         return
  170    if (itype(i).gt.0) go to 180
         er(i)=1.0d0/er(i)
         ei(i)=0.0d0
         go to 190
  180    if (itype(i).eq.2) go to 190
         t=er(i)**2+ei(i)**2
         er(i)=er(i)/t
         ei(i)=ei(i)/t
         er(i+1)=er(i)
         ei(i+1)=-ei(i)
  190 continue
      return
c
c     last line of ricdsd
c
      end
