	subroutine sylc (f1,f2,u1,u2,h,n1,n2,work,info,nf1,nf2,nu1,nu2,nh)
c
c   subroutines used  -	 orthes, ortran, hqr4, shrslc
c
c   sylc is a fortran subroutine to solve the real matrix
c   equation f1*x + x*f2 + h = 0 .  the equation is
c   transformed so that f1 is in lower real schur form and
c   f2 is in upper real schur form, and the transformed equation
c   is solved by a recursive procedure.
c
c   the parameters in the argument list are
c
c	f1	a doubly subscripted array containing the
c		matrix f1.  on return, the lower triangle
c		and the first superdiagonal of the array f1
c		contain an lower real schur form of f1.
c
c	f2	a doubly subscripted array containing the
c		matrix f2.  on return, the upper triangle
c		and the first subdiagonal of the array f2
c		contain an upper real schur form of f2.
c
c	u1	a doubly subscripted array that, on
c		return, contains the orthogonal matrix
c		that reduces f1 to lower real schur form.
c
c	u2	a doubly subscripted array that, on
c		return, contains the orthogonal matrix
c		that reduces f2 to upper real schur form.
c
c	h	a doubly subscripted array containing the
c		matrix h.  on return, h contains the
c		solution matrix x.
c
c	n1	the order of the matrix f1.
c
c	n2	the order of the matrix f2.
c
c	work	a scratch vector of length max(n1,n2).
c
c	info	an integer variable that, on entry, contains a flag
c		whose value is zero if the matrices f1 and f2 are
c		already in real schur form.  on return, it contains
c		an error signal whose value is
c		    0  if the solution was found,
c		    1  if the real schur decompostion of f1 failed,
c		    2  if the real schur decompostion of f2 failed
c
c	nf1	the first dimension of the array f1.
c
c	nf2	the first dimension of the array f2.
c
c	nu1	the first dimension of the array u1.
c
c	nu2	the first dimension of the array u2.
c
c	nh	the first dimension of the array h.
c
c   when info is zero, the reduction of f1 and f2 to real schur
c   form is skipped and the arrays f1, f2, u1, and u2 are assumed to
c   contain the schur forms and accompanying orthogonal matrices.
c   this permits the efficient solution of several equations
c   with different h matrices.
c
	real*8 f1(nf1,*),u1(nu1,*)
	real*8 f2(nf2,*),u2(nu2,*)
	real*8 h(nh,*),work(*),eps,temp
	integer n1,n2,nf1,nf2,nu1,nu2,nh,ifail,info,i,j,k,nlow,nup
c
	eps = 1.0
    1	continue
	eps = 0.5*eps
	if (eps+1.0 .gt. 1.0) goto 1
	eps = 2.0*eps
	if (info .eq. 0) goto 25
c
c   if required, reduce f1 to lower real schur form
c
	do 10 j=1,n1
	    do 5 i=j+1,n1
		temp = f1(i,j)
		f1(i,j) = f1(j,i)
		f1(j,i) = temp
    5	    continue
   10	continue
	nlow = 1
	nup = n1
	call orthes (nf1,n1,nlow,nup,f1,work)
	call ortran (nf1,n1,nlow,nup,f1,work,u1)
	call hqr4 (f1,u1,n1,nlow,nup,eps,ifail,nf1,nu1)
	do 20 j=1,n1
	    do 15 i=j+1,n1
		temp = f1(i,j)
		f1(i,j) = f1(j,i)
		f1(j,i) = temp
   15	    continue
   20	continue
	if (ifail .ne. 0) then
	    info = 1
	    return
	endif
	nlow = 1
	nup = n2
	call orthes (nf2,n2,nlow,nup,f2,work)
	call ortran (nf2,n2,nlow,nup,f2,work,u2)
	call hqr4 (f2,u2,n2,nlow,nup,eps,ifail,nf2,nu2)
	if (ifail .ne. 0) then
	    info = 2
	    return
	endif
	info = 0
c
c   transform h
c
   25	continue
	do 45 i=1,n1
	    do 35 j=1,n2
		work(j) = 0.0
		do 30 k=1,n2
		    work(j) = work(j) + h(i,k)*u2(k,j)
   30		continue
   35	    continue
	    do 40 j=1,n2
		h(i,j) = work(j)
   40	    continue
   45	continue
	do 65 j=1,n2
	    do 55 i=1,n1
		work(i) = 0.0
		do 50 k=1,n1
		    work(i) = work(i) + u1(k,i)*h(k,j)
   50		continue
   55	    continue
	    do 60 i=1,n1
		h(i,j) = work(i)
   60	    continue
   65	continue
c
c   move h matrix to right hand side for shrslc
c
	do 75 i=1,n1
	    do 70 j=1,n2
		h(i,j) = -h(i,j)
   70	    continue
   75	continue
c
c   solve the transformed system
c
	call shrslc (f1,f2,h,n1,n2,nf1,nf2,nh)
c
c   transform h back to the solution
c
	do 105 i=1,n1
	    do 95 j=1,n2
		work(j) = 0.0
		do 90 k=1,n2
		    work(j) = work(j) + h(i,k)*u2(j,k)
   90		continue
   95	    continue
	    do 100 j=1,n2
		h(i,j) = work(j)
  100	    continue
  105	continue
	do 125 j=1,n2
	    do 115 i=1,n1
		work(i) = 0.0
		do 110 k=1,n1
		    work(i) = work(i) + u1(i,k)*h(k,j)
  110		continue
  115	    continue
	    do 120 i=1,n1
		h(i,j) = work(i)
  120	    continue
  125	continue
	return
	end
