      SUBROUTINE DNBFS(ABE,LDA,N,ML,MU,V,ITASK,IND,WORK,IWORK)                  
C***BEGIN PROLOGUE  DNBFS                                                       
C***DATE WRITTEN   800812   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D2A2                                                          
C***KEYWORDS  BAND,DOUBLE PRECISION,LINEAR EQUATIONS,NONSYMMETRIC               
C***AUTHOR  VOORHEES, E., (LANL)                                                
C***PURPOSE  DNBFS solves a general nonsymmetric BANDED                         
C            double precision NXN system of linear equations                    
C***DESCRIPTION                                                                 
C                                                                               
C    Subroutine DNBFS solves a general nonsymmetric banded NxN                  
C    system of double precision real linear equations using                     
C    SLATEC subroutines DNBCO and DNBSL.  These are adaptations                 
C    of the LINPACK subroutines DGBCO and DGBSL which require                   
C    a different format for storing the matrix elements.  If                    
C    A  is an NxN double precision matrix and if  X  and  B  are                
C    double precision N-vectors, then DNBFS solves the equation                 
C                                                                               
C                          A*X=B.                                               
C                                                                               
C    A band matrix is a matrix whose nonzero elements are all                   
C    fairly near the main diagonal, specifically  A(I,J) = 0                    
C    if  I-J is greater than  ML  or  J-I  is greater than                      
C    MU .  The integers ML and MU are called the lower and upper                
C    band widths and  M = ML+MU+1  is the total band width.                     
C    DNBFS uses less time and storage than the corresponding                    
C    program for general matrices (DGEFS) if 2*ML+MU .LT.  N .                  
C                                                                               
C    The matrix A is first factored into upper and lower tri-                   
C    angular matrices U and L using partial pivoting.  These                    
C    factors and the pivoting information are used to find the                  
C    solution vector X.  An approximate condition number is                     
C    calculated to provide a rough estimate of the number of                    
C    digits of accuracy in the computed solution.                               
C                                                                               
C    If the equation A*X=B is to be solved for more than one vector             
C    B, the factoring of A does not need to be performed again and              
C    the option to only solve (ITASK .GT. 1) will be faster for                 
C    the succeeding solutions.  In this case, the contents of A,                
C    LDA, N and IWORK must not have been altered by the user follow-            
C    ing factorization (ITASK=1).  IND will not be changed by DNBFS             
C    in this case.                                                              
C                                                                               
C                                                                               
C    Band Storage                                                               
C                                                                               
C          If  A  is a band matrix, the following program segment               
C          will set up the input.                                               
C                                                                               
C                  ML = (band width below the diagonal)                         
C                  MU = (band width above the diagonal)                         
C                  DO 20 I = 1, N                                               
C                     J1 = MAX0(1, I-ML)                                        
C                     J2 = MIN0(N, I+MU)                                        
C                     DO 10 J = J1, J2                                          
C                        K = J - I + ML + 1                                     
C                        ABE(I,K) = A(I,J)                                      
C               10    CONTINUE                                                  
C               20 CONTINUE                                                     
C                                                                               
C          This uses columns  1  through  ML+MU+1  of ABE .                     
C          Furthermore,  ML  additional columns are needed in                   
C          ABE  starting with column  ML+MU+2  for elements                     
C          generated during the triangularization.  The total                   
C          number of columns needed in  ABE  is  2*ML+MU+1 .                    
C                                                                               
C    Example:  If the original matrix is                                        
C                                                                               
C          11 12 13  0  0  0                                                    
C          21 22 23 24  0  0                                                    
C           0 32 33 34 35  0                                                    
C           0  0 43 44 45 46                                                    
C           0  0  0 54 55 56                                                    
C           0  0  0  0 65 66                                                    
C                                                                               
C     then  N = 6, ML = 1, MU = 2, LDA .GE. 5  and ABE should contain           
C                                                                               
C           * 11 12 13  +     , * = not used                                    
C          21 22 23 24  +     , + = used for pivoting                           
C          32 33 34 35  +                                                       
C          43 44 45 46  +                                                       
C          54 55 56  *  +                                                       
C          65 66  *  *  +                                                       
C                                                                               
C                                                                               
C  Argument Description ***                                                     
C                                                                               
C    ABE    DOUBLE PRECISION(LDA,NC)                                            
C             on entry, contains the matrix in band storage as                  
C               described above.  NC  must not be less than                     
C               2*ML+MU+1 .  The user is cautioned to specify  NC               
C               with care since it is not an argument and cannot                
C               be checked by DNBFS.  The rows of the original                  
C               matrix are stored in the rows of  ABE  and the                  
C               diagonals of the original matrix are stored in                  
C               columns  1  through  ML+MU+1  of  ABE .                         
C             on return, contains an upper triangular matrix U and              
C               the multipliers necessary to construct a matrix L               
C               so that A=L*U.                                                  
C    LDA    INTEGER                                                             
C             the leading dimension of array ABE.  LDA must be great-           
C             er than or equal to N.  (terminal error message IND=-1)           
C    N      INTEGER                                                             
C             the order of the matrix A.  N must be greater                     
C             than or equal to 1 .  (terminal error message IND=-2)             
C    ML     INTEGER                                                             
C             the number of diagonals below the main diagonal.                  
C             ML  must not be less than zero nor greater than or                
C             equal to  N .  (terminal error message IND=-5)                    
C    MU     INTEGER                                                             
C             the number of diagonals above the main diagonal.                  
C             MU  must not be less than zero nor greater than or                
C             equal to  N .  (terminal error message IND=-6)                    
C    V      DOUBLE PRECISION(N)                                                 
C             on entry, the singly subscripted array(vector) of di-             
C               mension N which contains the right hand side B of a             
C               system of simultaneous linear equations A*X=B.                  
C             on return, V contains the solution vector, X .                    
C    ITASK  INTEGER                                                             
C             If ITASK=1, the matrix A is factored and then the                 
C               linear equation is solved.                                      
C             If ITASK .GT. 1, the equation is solved using the existing        
C               factored matrix A and IWORK.                                    
C             If ITASK .LT. 1, then terminal error message IND=-3 is            
C               printed.                                                        
C    IND    INTEGER                                                             
C             GT. 0  IND is a rough estimate of the number of digits            
C                     of accuracy in the solution, X.                           
C             LT. 0  See error message corresponding to IND below.              
C    WORK   DOUBLE PRECISION(N)                                                 
C             a singly subscripted array of dimension at least N.               
C    IWORK  INTEGER(N)                                                          
C             a singly subscripted array of dimension at least N.               
C                                                                               
C  Error Messages Printed ***                                                   
C                                                                               
C    IND=-1  terminal   N is greater than LDA.                                  
C    IND=-2  terminal   N is less than 1.                                       
C    IND=-3  terminal   ITASK is less than 1.                                   
C    IND=-4  terminal   The matrix A is computationally singular.               
C                         A solution has not been computed.                     
C    IND=-5  terminal   ML is less than zero or is greater than                 
C                         or equal to N .                                       
C    IND=-6  terminal   MU is less than zero or is greater than                 
C                         or equal to N .                                       
C    IND=-10 warning    The solution has no apparent significance.              
C                         The solution may be inaccurate or the matrix          
C                         A may be poorly scaled.                               
C                                                                               
C               Note-  The above terminal(*fatal*) error messages are           
C                      designed to be handled by XERRWV in which                
C                      LEVEL=1 (recoverable) and IFLAG=2 .  LEVEL=0             
C                      for warning error messages from XERROR.  Unless          
C                      the user provides otherwise, an error message            
C                      will be printed followed by an abort.                    
C***REFERENCES  SUBROUTINE DNBFS WAS DEVELOPED BY GROUP C-3, LOS ALAMOS         
C                 SCIENTIFIC LABORATORY, LOS ALAMOS, NM 87545.                  
C                 THE SLATEC SUBROUTINES USED BY DNBFS ARE SIMILAR              
C                 (EXCEPT FOR MATRIX STORAGE FORMAT) TO THE CORRESPON-          
C                 DING LINPACK SUBROUTINES (DGBCO AND DGBSL) DESCRIBED          
C                 IN DETAIL IN THE *LINPACK USERS GUIDE* PUBLISHED BY           
C                 THE SOCIETY FOR INDUSTRIAL AND APPLIED MATHEMATICS            
C                 (SIAM) DATED 1979.                                            
C***ROUTINES CALLED  D1MACH,DNBCO,DNBSL,XERROR,XERRWV                           
C***END PROLOGUE  DNBFS                                                         
C                                                                               
      INTEGER LDA,N,ITASK,IND,IWORK(N),ML,MU                                    
      DOUBLE PRECISION ABE(LDA,1),V(N),WORK(N),D1MACH                           
      DOUBLE PRECISION RCOND                                                    
C***FIRST EXECUTABLE STATEMENT  DNBFS                                           
      IF (LDA.LT.N)  GO TO 101                                                  
      IF (N.LE.0)  GO TO 102                                                    
      IF (ITASK.LT.1) GO TO 103                                                 
      IF((ML.LT.0).OR.(ML.GE.N))GO TO 105                                       
      IF((MU.LT.0).OR.(MU.GE.N))GO TO 106                                       
      IF (ITASK.GT.1) GO TO 20                                                  
C                                                                               
C     FACTOR MATRIX A INTO LU                                                   
      CALL DNBCO(ABE,LDA,N,ML,MU,IWORK,RCOND,WORK)                              
C                                                                               
C     CHECK FOR COMPUTATIONALLY SINGULAR MATRIX                                 
      IF (RCOND.EQ.0.0D0)  GO TO 104                                            
C                                                                               
C     COMPUTE IND (ESTIMATE OF NO. OF SIGNIFICANT DIGITS)                       
      IND=-IDINT(DLOG10(D1MACH(4)/RCOND))                                       
C                                                                               
C     CHECK FOR IND GREATER THAN ZERO                                           
      IF (IND.GT.0)  GO TO 20                                                   
      IND=-10                                                                   
      CALL XERROR( 'DNBFS ERROR (IND=-10) -- SOLUTION MAY HAVE NO SIGNIF        
     1ICANCE',58,-10,0)                                                         
C                                                                               
C     SOLVE AFTER FACTORING                                                     
   20 CALL DNBSL(ABE,LDA,N,ML,MU,IWORK,V,0)                                     
      RETURN                                                                    
C                                                                               
C     IF LDA.LT.N, IND=-1, TERMINAL XERRWV MESSAGE                              
  101 IND=-1                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-1) -- LDA=I1 IS LESS THAN N=I2',          
     148,-1,1,2,LDA,N,0,0,0)                                                    
      RETURN                                                                    
C                                                                               
C     IF N.LT.1, IND=-2, TERMINAL XERRWV MESSAGE                                
  102 IND=-2                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-2) -- N=I1 IS LESS THAN 1',               
     143,-2,1,1,N,0,0,0,0)                                                      
      RETURN                                                                    
C                                                                               
C     IF ITASK.LT.1, IND=-3, TERMINAL XERRWV MESSAGE                            
  103 IND=-3                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-3) -- ITASK=I1 IS LESS THAN 1',           
     147,-3,1,1,ITASK,0,0,0,0)                                                  
      RETURN                                                                    
C                                                                               
C     IF SINGULAR MATRIX, IND=-4, TERMINAL XERRWV MESSAGE                       
  104 IND=-4                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-4) -- SINGULAR MATRIX A - NO SOLUT        
     1ION',55,-4,1,0,0,0,0,0,0)                                                 
      RETURN                                                                    
C                                                                               
C     IF IMPROPER ML VALUE, IND=-5, TERMINAL XERRWV MESSAGE                     
  105 IND=-5                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-5) -- ML=I1 IS OUT OF RANGE',45,-5        
     1,1,1,ML,0,0,0,0)                                                          
      RETURN                                                                    
C                                                                               
C     IF IMPROPER MU VALUE, IND=-6, TERMINAL XERRWV MESSAGE                     
  106 IND=-6                                                                    
      CALL XERRWV( 'DNBFS ERROR (IND=-6) -- MU=I1 IS OUT OF RANGE',45,-6        
     1,1,1,MU,0,0,0,0)                                                          
      RETURN                                                                    
C                                                                               
      END                                                                       
      SUBROUTINE DNBCO(ABE,LDA,N,ML,MU,IPVT,RCOND,Z)                            
C***BEGIN PROLOGUE  DNBCO                                                       
C***DATE WRITTEN   800728   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D2A2                                                          
C***KEYWORDS  BAND,DOUBLE PRECISION,LINEAR EQUATIONS,NONSYMMETRIC               
C***AUTHOR  VOORHEES, E., (LANL)                                                
C***PURPOSE  Factors a double precision BAND matrix by Gaussian                 
C            elimination and estimates the condition of the matrix.             
C***DESCRIPTION                                                                 
C                                                                               
C     DNBCO factors a double precision band matrix by Gaussian                  
C     elimination and estimates the condition of the matrix.                    
C                                                                               
C     If RCOND is not needed, DNBFA is slightly faster.                         
C     To solve  A*X = B , follow DNBCO by DNBSL.                                
C     To compute  INVERSE(A)*C , follow DNBCO by DNBSL.                         
C     To compute  DETERMINANT(A) , follow DNBCO by DNBDI.                       
C                                                                               
C     On Entry                                                                  
C                                                                               
C        ABE     DOUBLE PRECISION(LDA, NC)                                      
C                contains the matrix in band storage.  The rows                 
C                of the original matrix are stored in the rows                  
C                of ABE and the diagonals of the original matrix                
C                are stored in columns 1 through ML+MU+1 of ABE.                
C                NC must be .GE. 2*ML+MU+1 .                                    
C                See the comments below for details.                            
C                                                                               
C        LDA     INTEGER                                                        
C                the leading dimension of the array ABE.                        
C                LDA must be .GE.  N .                                          
C                                                                               
C        N       INTEGER                                                        
C                the order of the original matrix.                              
C                                                                               
C        ML      INTEGER                                                        
C                number of diagonals below the main diagonal.                   
C                0 .LE. ML .LT.  N .                                            
C                                                                               
C        MU      INTEGER                                                        
C                number of diagonals above the main diagonal.                   
C                0 .LE. MU .LT.  N .                                            
C                More efficient if ML .LE. MU .                                 
C                                                                               
C     On Return                                                                 
C                                                                               
C        ABE     an upper triangular matrix in band storage                     
C                and the multipliers which were used to obtain it.              
C                The factorization can be written  A = L*U  where               
C                L is a product of permutation and unit lower                   
C                triangular matrices and  U  is upper triangular.               
C                                                                               
C        IPVT    INTEGER(N)                                                     
C                an integer vector of pivot indices.                            
C                                                                               
C        RCOND   DOUBLE PRECISION                                               
C                an estimate of the reciprocal condition of  A .                
C                For the system  A*X = B , relative perturbations               
C                in  A  and  B  of size  EPSILON  may cause                     
C                relative perturbations in  X  of size  EPSILON/RCOND .         
C                If  RCOND  is so small that the logical expression             
C                         1.0 + RCOND .EQ. 1.0                                  
C                is true, then  A  may be singular to working                   
C                precision.  In particular,  RCOND  is zero  if                 
C                exact singularity is detected or the estimate                  
C                underflows.                                                    
C                                                                               
C        Z       DOUBLE PRECISION(N)                                            
C                a work vector whose contents are usually unimportant.          
C                If  A  is close to a singular matrix, then  Z  is              
C                an approximate null vector in the sense that                   
C                NORM(A*Z) = RCOND*NORM(A)*NORM(Z) .                            
C                                                                               
C     Band Storage                                                              
C                                                                               
C           If  A  is a band matrix, the following program segment              
C           will set up the input.                                              
C                                                                               
C                   ML = (band width below the diagonal)                        
C                   MU = (band width above the diagonal)                        
C                   DO 20 I = 1, N                                              
C                      J1 = MAX0(1, I-ML)                                       
C                      J2 = MIN0(N, I+MU)                                       
C                      DO 10 J = J1, J2                                         
C                         K = J - I + ML + 1                                    
C                         ABE(I,K) = A(I,J)                                     
C                10    CONTINUE                                                 
C                20 CONTINUE                                                    
C                                                                               
C           This uses columns  1  through  ML+MU+1  of ABE .                    
C           Furthermore,  ML  additional columns are needed in                  
C           ABE  starting with column  ML+MU+2  for elements                    
C           generated during the triangularization.  The total                  
C           number of columns needed in  ABE  is  2*ML+MU+1 .                   
C                                                                               
C     Example:  If the original matrix is                                       
C                                                                               
C           11 12 13  0  0  0                                                   
C           21 22 23 24  0  0                                                   
C            0 32 33 34 35  0                                                   
C            0  0 43 44 45 46                                                   
C            0  0  0 54 55 56                                                   
C            0  0  0  0 65 66                                                   
C                                                                               
C      then  N = 6, ML = 1, MU = 2, LDA .GE. 5  and ABE should contain          
C                                                                               
C            * 11 12 13  +     , * = not used                                   
C           21 22 23 24  +     , + = used for pivoting                          
C           32 33 34 35  +                                                      
C           43 44 45 46  +                                                      
C           54 55 56  *  +                                                      
C           65 66  *  *  +                                                      
C                                                                               
C     SLATEC.  This version dated 07/28/80 .                                    
C     E. A. Voorhees, Los Alamos Scientific Laboratory                          
C                                                                               
C     Subroutines and Functions                                                 
C                                                                               
C     SLATEC DNBFA                                                              
C      DDOT,DAXPY,DSCAL,DASUM                                                   
C     Fortran  DABS,DMAX1,MAX0,MIN0,DSIGN                                       
C***REFERENCES  SUBROUTINE DNBCO WAS DEVELOPED BY GROUP C-3, LOS ALAMOS         
C                 SCIENTIFIC LABORATORY, LOS ALAMOS, NM 87545.                  
C***ROUTINES CALLED  DASUM,DAXPY,DDOT,DNBFA,DSCAL                               
C***END PROLOGUE  DNBCO                                                         
      INTEGER LDA,N,ML,MU,IPVT(1)                                               
      DOUBLE PRECISION ABE(LDA,1),Z(1)                                          
      DOUBLE PRECISION RCOND                                                    
C                                                                               
      DOUBLE PRECISION DDOT,EK,T,WK,WKM                                         
      DOUBLE PRECISION ANORM,S,DASUM,SM,YNORM                                   
      INTEGER I,INFO,J,JU,K,KB,KP1,L,LDB,LM,LZ,M,ML1,MM,NL,NU                   
C***FIRST EXECUTABLE STATEMENT  DNBCO                                           
      ML1=ML+1                                                                  
      LDB = LDA - 1                                                             
      ANORM = 0.0D0                                                             
      DO 10 J = 1, N                                                            
        NU = MIN0(MU,J-1)                                                       
        NL = MIN0(ML,N-J)                                                       
        L = 1 + NU + NL                                                         
        ANORM = DMAX1(ANORM,DASUM(L,ABE(J+NL,ML1-NL),LDB))                      
   10 CONTINUE                                                                  
C                                                                               
C     FACTOR                                                                    
C                                                                               
      CALL DNBFA(ABE,LDA,N,ML,MU,IPVT,INFO)                                     
C                                                                               
C     RCOND = 1/(NORM(A)*(ESTIMATE OF NORM(INVERSE(A)))) .                      
C     ESTIMATE = NORM(Z)/NORM(Y) WHERE  A*Z = Y  AND TRANS(A)*Y = E .           
C     TRANS(A)  IS THE TRANSPOSE OF A .  THE COMPONENTS OF  E  ARE              
C     CHOSEN TO CAUSE MAXIMUM LOCAL GROWTH IN THE ELEMENTS OF  W WHERE          
C     TRANS(U)*W = E .  THE VECTORS ARE FREQUENTLY RESCALED TO AVOID            
C     OVERFLOW.                                                                 
C                                                                               
C     SOLVE TRANS(U)*W = E                                                      
C                                                                               
      EK = 1.0D0                                                                
      DO 20 J = 1, N                                                            
        Z(J) = 0.0D0                                                            
   20 CONTINUE                                                                  
      M = ML + MU + 1                                                           
      JU = 0                                                                    
      DO 100 K = 1, N                                                           
        IF (Z(K) .NE. 0.0D0) EK = DSIGN(EK,-Z(K))                               
        IF (DABS(EK-Z(K)) .LE. DABS(ABE(K,ML1))) GO TO 30                       
          S = DABS(ABE(K,ML1))/DABS(EK-Z(K))                                    
          CALL DSCAL(N,S,Z,1)                                                   
          EK = S*EK                                                             
   30   CONTINUE                                                                
        WK = EK - Z(K)                                                          
        WKM = -EK - Z(K)                                                        
        S = DABS(WK)                                                            
        SM = DABS(WKM)                                                          
        IF (ABE(K,ML1) .EQ. 0.0D0) GO TO 40                                     
          WK = WK/ABE(K,ML1)                                                    
          WKM = WKM/ABE(K,ML1)                                                  
        GO TO 50                                                                
   40   CONTINUE                                                                
          WK = 1.0D0                                                            
          WKM = 1.0D0                                                           
   50   CONTINUE                                                                
        KP1 = K + 1                                                             
        JU = MIN0(MAX0(JU,MU+IPVT(K)),N)                                        
        MM = ML1                                                                
        IF (KP1 .GT. JU) GO TO 90                                               
          DO 60 I = KP1, JU                                                     
            MM = MM + 1                                                         
            SM = SM + DABS(Z(I)+WKM*ABE(K,MM))                                  
            Z(I) = Z(I) + WK*ABE(K,MM)                                          
            S = S + DABS(Z(I))                                                  
   60     CONTINUE                                                              
          IF (S .GE. SM) GO TO 80                                               
            T = WKM -WK                                                         
            WK = WKM                                                            
            MM = ML1                                                            
            DO 70 I = KP1, JU                                                   
              MM = MM + 1                                                       
              Z(I) = Z(I) + T*ABE(K,MM)                                         
   70       CONTINUE                                                            
   80     CONTINUE                                                              
   90   CONTINUE                                                                
      Z(K) = WK                                                                 
  100 CONTINUE                                                                  
      S = 1.0D0/DASUM(N,Z,1)                                                    
      CALL DSCAL(N,S,Z,1)                                                       
C                                                                               
C     SOLVE TRANS(L)*Y = W                                                      
C                                                                               
      DO 120 KB = 1, N                                                          
        K = N + 1 - KB                                                          
        NL = MIN0(ML,N-K)                                                       
        IF (K .LT. N) Z(K) = Z(K) + DDOT(NL,ABE(K+NL,ML1-NL),-LDB,Z(K+1)        
     1  ,1)                                                                     
        IF (DABS(Z(K)) .LE. 1.0D0) GO TO 110                                    
          S = 1.0D0/DABS(Z(K))                                                  
          CALL DSCAL(N,S,Z,1)                                                   
  110   CONTINUE                                                                
        L = IPVT(K)                                                             
        T = Z(L)                                                                
        Z(L) = Z(K)                                                             
        Z(K) = T                                                                
  120 CONTINUE                                                                  
      S = 1.0D0/DASUM(N,Z,1)                                                    
      CALL DSCAL(N,S,Z,1)                                                       
C                                                                               
      YNORM = 1.0D0                                                             
C                                                                               
C     SOLVE L*V = Y                                                             
C                                                                               
      DO 140 K = 1, N                                                           
        L = IPVT(K)                                                             
        T = Z(L)                                                                
        Z(L) = Z(K)                                                             
        Z(K) = T                                                                
        NL = MIN0(ML,N-K)                                                       
        IF (K .LT. N) CALL DAXPY(NL,T,ABE(K+NL,ML1-NL),-LDB,Z(K+1),1)           
        IF (DABS(Z(K)) .LE. 1.0D0) GO TO 130                                    
          S = 1.0D0/DABS(Z(K))                                                  
          CALL DSCAL(N,S,Z,1)                                                   
          YNORM = S*YNORM                                                       
  130   CONTINUE                                                                
  140 CONTINUE                                                                  
      S = 1.0D0/DASUM(N,Z,1)                                                    
      CALL DSCAL(N,S,Z,1)                                                       
      YNORM = S*YNORM                                                           
C                                                                               
C     SOLVE  U*Z = V                                                            
C                                                                               
      DO 160 KB = 1, N                                                          
        K = N + 1 - KB                                                          
        IF (DABS(Z(K)) .LE. DABS(ABE(K,ML1))) GO TO 150                         
          S = DABS(ABE(K,ML1))/DABS(Z(K))                                       
          CALL DSCAL(N,S,Z,1)                                                   
          YNORM = S*YNORM                                                       
  150   CONTINUE                                                                
        IF (ABE(K,ML1) .NE. 0.0D0) Z(K) = Z(K)/ABE(K,ML1)                       
        IF (ABE(K,ML1) .EQ. 0.0D0) Z(K) = 1.0D0                                 
        LM = MIN0(K,M) - 1                                                      
        LZ = K - LM                                                             
        T = -Z(K)                                                               
        CALL DAXPY(LM,T,ABE(K-1,ML+2),-LDB,Z(LZ),1)                             
  160 CONTINUE                                                                  
C     MAKE ZNORM = 1.0D0                                                        
      S = 1.0D0/DASUM(N,Z,1)                                                    
      CALL DSCAL(N,S,Z,1)                                                       
      YNORM = S*YNORM                                                           
C                                                                               
      IF (ANORM .NE. 0.0D0) RCOND = YNORM/ANORM                                 
      IF (ANORM .EQ. 0.0D0) RCOND = 0.0D0                                       
      RETURN                                                                    
      END                                                                       
      SUBROUTINE DNBSL(ABE,LDA,N,ML,MU,IPVT,B,JOB)                              
C***BEGIN PROLOGUE  DNBSL                                                       
C***DATE WRITTEN   800728   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D2A2                                                          
C***KEYWORDS  BAND,DOUBLE PRECISION,LINEAR EQUATIONS,NONSYMMETRIC               
C***AUTHOR  VOORHEES, E., (LANL)                                                
C***PURPOSE  Solves a double precision BAND system using                        
C            the factors computed by DNBCO or DNBFA.                            
C***DESCRIPTION                                                                 
C                                                                               
C     DNBSL solves the double precision band system                             
C     A * X = B  or  TRANS(A) * X = B                                           
C     using the factors computed by DNBCO or DNBFA.                             
C                                                                               
C     On Entry                                                                  
C                                                                               
C        ABE     DOUBLE PRECISION(LDA, NC)                                      
C                the output from DNBCO or DNBFA.                                
C                NC must be .GE. 2*ML+MU+1 .                                    
C                                                                               
C        LDA     INTEGER                                                        
C                the leading dimension of the array  ABE .                      
C                                                                               
C        N       INTEGER                                                        
C                the order of the original matrix.                              
C                                                                               
C        ML      INTEGER                                                        
C                number of diagonals below the main diagonal.                   
C                                                                               
C        MU      INTEGER                                                        
C                number of diagonals above the main diagonal.                   
C                                                                               
C        IPVT    INTEGER(N)                                                     
C                the pivot vector from DNBCO or DNBFA.                          
C                                                                               
C        B       DOUBLE PRECISION(N)                                            
C                the right hand side vector.                                    
C                                                                               
C        JOB     INTEGER                                                        
C                = 0         to solve  A*X = B .                                
C                = nonzero   to solve  TRANS(A)*X = B , where                   
C                            TRANS(A)  is the transpose.                        
C                                                                               
C     On Return                                                                 
C                                                                               
C        B       the solution vector  X .                                       
C                                                                               
C     Error Condition                                                           
C                                                                               
C        A division by zero will occur if the input factor contains a           
C        zero on the diagonal.  Technically this indicates singularity          
C        but it is often caused by improper arguments or improper               
C        setting of LDA.  It will not occur if the subroutines are              
C        called correctly and if DNBCO has set RCOND .GT. 0.0                   
C        or DNBFA has set INFO .EQ. 0 .                                         
C                                                                               
C     To compute  INVERSE(A) * C  where  C  is a matrix                         
C     with  P  columns                                                          
C           CALL DNBCO(ABE,LDA,N,ML,MU,IPVT,RCOND,Z)                            
C           IF (RCOND is too small) GO TO ...                                   
C           DO 10 J = 1, P                                                      
C             CALL DNBSL(ABE,LDA,N,ML,MU,IPVT,C(1,J),0)                         
C        10 CONTINUE                                                            
C                                                                               
C     SLATEC.  This version dated 07/28/80 .                                    
C     E. A. Voorhees, Los Alamos Scientific Laboratory                          
C                                                                               
C     Subroutines and Functions                                                 
C                                                                               
C      DAXPY,DDOT                                                               
C     Fortran  MIN0                                                             
C***REFERENCES  SUBROUTINE DNBSL WAS DEVELOPED BY GROUP C-3, LOS ALAMOS         
C                 SCIENTIFIC LABORATORY, LOS ALAMOS, NM 87545.                  
C***ROUTINES CALLED  DAXPY,DDOT                                                 
C***END PROLOGUE  DNBSL                                                         
      INTEGER LDA,N,ML,MU,IPVT(1),JOB                                           
      DOUBLE PRECISION ABE(LDA,1),B(1)                                          
C                                                                               
      DOUBLE PRECISION DDOT,T                                                   
      INTEGER K,KB,L,LB,LDB,LM,M,MLM,NM1                                        
C***FIRST EXECUTABLE STATEMENT  DNBSL                                           
      M=MU+ML+1                                                                 
      NM1=N-1                                                                   
      LDB=1-LDA                                                                 
      IF(JOB.NE.0)GO TO 50                                                      
C                                                                               
C       JOB = 0 , SOLVE  A * X = B                                              
C       FIRST SOLVE L*Y = B                                                     
C                                                                               
        IF(ML.EQ.0)GO TO 30                                                     
        IF(NM1.LT.1)GO TO 30                                                    
          DO 20 K=1,NM1                                                         
            LM=MIN0(ML,N-K)                                                     
            L=IPVT(K)                                                           
            T=B(L)                                                              
            IF(L.EQ.K)GO TO 10                                                  
              B(L)=B(K)                                                         
              B(K)=T                                                            
   10       CONTINUE                                                            
            MLM=ML-(LM-1)                                                       
            CALL DAXPY(LM,T,ABE(K+LM,MLM),LDB,B(K+1),1)                         
   20     CONTINUE                                                              
   30   CONTINUE                                                                
C                                                                               
C       NOW SOLVE  U*X = Y                                                      
C                                                                               
        DO 40 KB=1,N                                                            
          K=N+1-KB                                                              
          B(K)=B(K)/ABE(K,ML+1)                                                 
          LM=MIN0(K,M)-1                                                        
          LB=K-LM                                                               
          T=-B(K)                                                               
          CALL DAXPY(LM,T,ABE(K-1,ML+2),LDB,B(LB),1)                            
   40   CONTINUE                                                                
      GO TO 100                                                                 
   50 CONTINUE                                                                  
C                                                                               
C       JOB = NONZERO, SOLVE TRANS(A) * X = B                                   
C       FIRST SOLVE  TRANS(U)*Y = B                                             
C                                                                               
        DO 60 K = 1, N                                                          
          LM = MIN0(K,M) - 1                                                    
          LB = K - LM                                                           
          T = DDOT(LM,ABE(K-1,ML+2),LDB,B(LB),1)                                
          B(K) = (B(K) - T)/ABE(K,ML+1)                                         
   60   CONTINUE                                                                
C                                                                               
C       NOW SOLVE TRANS(L)*X = Y                                                
C                                                                               
        IF (ML .EQ. 0) GO TO 90                                                 
        IF (NM1 .LT. 1) GO TO 90                                                
          DO 80 KB = 1, NM1                                                     
            K = N - KB                                                          
            LM = MIN0(ML,N-K)                                                   
            MLM = ML - (LM - 1)                                                 
            B(K) = B(K) + DDOT(LM,ABE(K+LM,MLM),LDB,B(K+1),1)                   
            L = IPVT(K)                                                         
            IF (L .EQ. K) GO TO 70                                              
              T = B(L)                                                          
              B(L) = B(K)                                                       
              B(K) = T                                                          
   70       CONTINUE                                                            
   80     CONTINUE                                                              
   90   CONTINUE                                                                
  100 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       
      DOUBLE PRECISION FUNCTION DDOT(N,DX,INCX,DY,INCY)
C***BEGIN PROLOGUE  DDOT
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A
C***KEYWORDS  BLAS,VECTOR,DOUBLE PRECISION,INNER PRODUCT,
C             DOT PRODUCT
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C   D.P. INNER PRODUCT OF D.P. VECTORS
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C       DY  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCY  STORAGE SPACING BETWEEN ELEMENTS OF DY
C
C     --OUTPUT--
C     DDOT  DOUBLE PRECISION DOT PRODUCT (ZERO IF N.LE.0)
C
C     RETURNS THE DOT PRODUCT OF DOUBLE PRECISION DX AND DY.
C     DDOT = SUM FOR I = 0 TO N-1 OF  DX(LX+I*INCX) * DY(LY+I*INCY)
C     WHERE LX = 1 IF INCX .GE. 0, ELSE LX = (-INCX)*N, AND LY IS
C     DEFINED IN A SIMILAR WAY USING INCY.
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  DDOT
C
      DOUBLE PRECISION DX(1),DY(1)
C***FIRST EXECUTABLE STATEMENT  DDOT
      DDOT = 0.D0
      IF(N.LE.0)RETURN
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60
    5 CONTINUE
C
C         CODE FOR UNEQUAL OR NONPOSITIVE INCREMENTS.
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
         DDOT = DDOT + DX(IX)*DY(IY)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1.
C
C
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 5.
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
         DDOT = DDOT + DX(I)*DY(I)
   30 CONTINUE
      IF( N .LT. 5 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
         DDOT = DDOT + DX(I)*DY(I) + DX(I+1)*DY(I+1) +
     $   DX(I + 2)*DY(I + 2) + DX(I + 3)*DY(I + 3) + DX(I + 4)*DY(I + 4)
   50 CONTINUE
      RETURN
C
C         CODE FOR POSITIVE EQUAL INCREMENTS .NE.1.
C
   60 CONTINUE
      NS = N*INCX
          DO 70 I=1,NS,INCX
          DDOT = DDOT + DX(I)*DY(I)
   70     CONTINUE
      RETURN
      END
      SUBROUTINE DAXPY(N,DA,DX,INCX,DY,INCY)
C***BEGIN PROLOGUE  DAXPY
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A
C***KEYWORDS  BLAS,VECTOR,DOUBLE PRECISION
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C  D.P COMPUTATION Y = A*X + Y
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DA  DOUBLE PRECISION SCALAR MULTIPLIER
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C       DY  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCY  STORAGE SPACING BETWEEN ELEMENTS OF DY
C
C     --OUTPUT--
C       DY  DOUBLE PRECISION RESULT (UNCHANGED IF N.LE.0)
C
C     OVERWRITE DOUBLE PRECISION DY WITH DOUBLE PRECISION DA*DX + DY.
C     FOR I = 0 TO N-1, REPLACE  DY(LY+I*INCY) WITH DA*DX(LX+I*INCX) +
C       DY(LY+I*INCY), WHERE LX = 1 IF INCX .GE. 0, ELSE LX = (-INCX)*N
C       AND LY IS DEFINED IN A SIMILAR WAY USING INCY.
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED   (NONE)
C***END PROLOGUE  DAXPY
C
      DOUBLE PRECISION DX(1),DY(1),DA
C***FIRST EXECUTABLE STATEMENT  DAXPY
      IF(N.LE.0.OR.DA.EQ.0.D0) RETURN
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60
    5 CONTINUE
C
C        CODE FOR NONEQUAL OR NONPOSITIVE INCREMENTS.
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DY(IY) = DY(IY) + DA*DX(IX)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 4.
C
   20 M = MOD(N,4)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DY(I) = DY(I) + DA*DX(I)
   30 CONTINUE
      IF( N .LT. 4 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,4
        DY(I) = DY(I) + DA*DX(I)
        DY(I + 1) = DY(I + 1) + DA*DX(I + 1)
        DY(I + 2) = DY(I + 2) + DA*DX(I + 2)
        DY(I + 3) = DY(I + 3) + DA*DX(I + 3)
   50 CONTINUE
      RETURN
C
C        CODE FOR EQUAL, POSITIVE, NONUNIT INCREMENTS.
C
   60 CONTINUE
      NS = N*INCX
          DO 70 I=1,NS,INCX
          DY(I) = DA*DX(I) + DY(I)
   70     CONTINUE
      RETURN
      END
      SUBROUTINE DSCAL(N,DA,DX,INCX)
C***BEGIN PROLOGUE  DSCAL
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A, M2
C***KEYWORDS
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C    D.P. VECTOR SCALE X = A*X
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DA  DOUBLE PRECISION SCALE FACTOR
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C
C     --OUTPUT--
C       DX  DOUBLE PRECISION RESULT (UNCHANGED IF N.LE.0)
C
C     REPLACE DOUBLE PRECISION DX BY DOUBLE PRECISION DA*DX.
C     FOR I = 0 TO N-1, REPLACE DX(1+I*INCX) WITH  DA * DX(1+I*INCX)
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  DSCAL
C
      DOUBLE PRECISION DA,DX(1)
C***FIRST EXECUTABLE STATEMENT  DSCAL
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1)GOTO 20
C
C        CODE FOR INCREMENTS NOT EQUAL TO 1.
C
      NS = N*INCX
          DO 10 I = 1,NS,INCX
          DX(I) = DA*DX(I)
   10     CONTINUE
      RETURN
C
C        CODE FOR INCREMENTS EQUAL TO 1.
C
C
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 5.
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DX(I) = DA*DX(I)
   30 CONTINUE
      IF( N .LT. 5 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DX(I) = DA*DX(I)
        DX(I + 1) = DA*DX(I + 1)
        DX(I + 2) = DA*DX(I + 2)
        DX(I + 3) = DA*DX(I + 3)
        DX(I + 4) = DA*DX(I + 4)
   50 CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DASUM(N,DX,INCX)
C***BEGIN PROLOGUE  DASUM
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A
C***KEYWORDS  BLAS,VECTOR,DOUBLE PRECISION,SUM
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C    SUM OF MAGNITUDES OF D.P. VECTOR COMPONENTS
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C
C     --OUTPUT--
C    DASUM  DOUBLE PRECISION RESULT (ZERO IF N.LE.0)
C
C     RETURNS SUM OF MAGNITUDES OF DOUBLE PRECISION DX.
C     DASUM = SUM FROM 0 TO N-1 OF DABS(DX(1+I*INCX))
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  DASUM
C
      DOUBLE PRECISION DX(1)
C***FIRST EXECUTABLE STATEMENT  DASUM
      DASUM = 0.D0
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1)GOTO 20
C
C        CODE FOR INCREMENTS NOT EQUAL TO 1.
C
      NS = N*INCX
          DO 10 I=1,NS,INCX
          DASUM = DASUM + DABS(DX(I))
   10     CONTINUE
      RETURN
C
C        CODE FOR INCREMENTS EQUAL TO 1.
C
C
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 6.
C
   20 M = MOD(N,6)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
         DASUM = DASUM + DABS(DX(I))
   30 CONTINUE
      IF( N .LT. 6 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,6
         DASUM = DASUM + DABS(DX(I)) + DABS(DX(I+1)) + DABS(DX(I+2))
     $   + DABS(DX(I+3)) + DABS(DX(I+4)) + DABS(DX(I+5))
   50 CONTINUE
      RETURN
      END
      SUBROUTINE DNBFA(ABE,LDA,N,ML,MU,IPVT,INFO)
C***BEGIN PROLOGUE  DNBFA
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F4H
C***KEYWORDS           LINEAR EQUATIONS,BAND,NONSYMMETRIC,
C                        DOUBLE PRECISION
C***AUTHOR               VOORHEES E. (LASL)
C***DATE WRITTEN         JULY 28,1980
C***PURPOSE  FACTORS A DOUBLE PRECISION BAND MATRIX BY ELIMINATION.
C***DESCRIPTION
C     DNBFA FACTORS A DOUBLE PRECISION BAND MATRIX BY ELIMINATION.
C
C     DNBFA IS USUALLY CALLED BY DNBCO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF RCOND IS NOT NEEDED.
C
C     ON ENTRY
C
C        ABE     DOUBLE PRECISION(LDA, NC)
C                CONTAINS THE MATRIX IN BAND STORAGE.  THE ROWS
C                OF THE ORIGINAL MATRIX ARE STORED IN THE ROWS
C                OF ABE AND THE DIAGONALS OF THE ORIGINAL MATRIX
C                ARE STORED IN COLUMNS 1 THROUGH ML+MU+1 OF ABE.
C                NC MUST BE .GE. 2*ML+MU+1 .
C                SEE THE COMMENTS BELOW FOR DETAILS.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY ABE.
C                LDA MUST BE .GE. N .
C
C        N       INTEGER
C                THE ORDER OF THE ORIGINAL MATRIX.
C
C        ML      INTEGER
C                NUMBER OF DIAGONALS BELOW THE MAIN DIAGONAL.
C                0 .LE. ML .LT. N .
C
C        MU      INTEGER
C                NUMBER OF DIAGONALS ABOVE THE MAIN DIAGONAL.
C                0 .LE. MU .LT. N .
C                MORE EFFICIENT IF ML .LE. MU .
C
C     ON RETURN
C
C        ABE     AN UPPER TRIANGULAR MATRIX IN BAND STORAGE
C                AND THE MULTIPLIERS WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        INFO    INTEGER
C                =0  NORMAL VALUE
C                =K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
C                CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                INDICATE THAT DNBSL WILL DIVIDE BY ZERO IF
C                CALLED.  USE RCOND IN DNBCO FOR A RELIABLE
C                INDICATION OF SINGULARITY.
C
C     BAND STORAGE
C
C           IF  A  IS A BAND MATRIX, THE FOLLOWING PROGRAM SEGMENT
C           WILL SET UP THE INPUT.
C
C                   ML = (BAND WIDTH BELOW THE DIAGONAL)
C                   MU = (BAND WIDTH ABOVE THE DIAGONAL)
C                   DO 20 I = 1, N
C                      J1 = MAX0(1, I-ML)
C                      J2 = MIN0(N, I+MU)
C                      DO 10 J = J1, J2
C                         K = J - I + ML + 1
C                         ABE(I,K) = A(I,J)
C                10    CONTINUE
C                20 CONTINUE
C
C           THIS USES COLUMNS  1  THROUGH  ML+MU+1  OF ABE .
C           FURTHERMORE,  ML  ADDITIONAL COLUMNS ARE NEEDED IN
C           ABE  STARTING WITH COLUMN  ML+MU+2  FOR ELEMENTS
C           GENERATED DURING THE TRIANGULARIZATION.  THE TOTAL
C           NUMBER OF COLUMNS NEEDED IN  ABE  IS  2*ML+MU+1 .
C
C     EXAMPLE..  IF THE ORIGINAL MATRIX IS
C
C           11 12 13  0  0  0
C           21 22 23 24  0  0
C            0 32 33 34 35  0
C            0  0 43 44 45 46
C            0  0  0 54 55 56
C            0  0  0  0 65 66
C
C      THEN  N = 6, ML = 1, MU = 2, LDA .GE. 5  AND ABE SHOULD CONTAIN
C
C            * 11 12 13  +     , * = NOT USED
C           21 22 23 24  +     , + = USED FOR PIVOTING
C           32 33 34 35  +
C           43 44 45 46  +
C           54 55 56  *  +
C           65 66  *  *  +
C
C     SLATEC.  THIS VERSION DATED 07/28/80 .
C     E. A. VOORHEES, LOS ALAMOS SCIENTIFIC LABORATORY
C
C     SUBROUTINES AND FUNCTIONS
C
C      IDAMAX,DAXPY,DSCAL,DSWAP
C     FORTRAN  MIN0
C
C
C***REFERENCES
C    SUBROUTINE DNBFA WAS DEVELOPED BY GROUP C-3, LOS ALAMOS SCIEN-
C    TIFIC LABORATORY, LOS ALAMOS, NM 87545.
C***ROUTINES CALLED  IDAMAX,DAXPY,DSCAL,DSWAP
C***END PROLOGUE  DNBFA
      INTEGER LDA,N,ML,MU,IPVT(1),INFO
      DOUBLE PRECISION ABE(LDA,1)
C
      INTEGER ML1,MB,M,N1,LDB,I,J,K,L,LM,LM1,LM2,MP,IDAMAX
      DOUBLE PRECISION T
C***FIRST EXECUTABLE STATEMENT  DNBFA
      ML1=ML+1
      MB=ML+MU
      M=ML+MU+1
      N1=N-1
      LDB=LDA-1
      INFO=0
C
C     SET FILL-IN COLUMNS TO ZERO
C
      IF(N.LE.1)GO TO 50
      IF(ML.LE.0)GO TO 7
      DO 6 J=1,ML
        DO 5 I=1,N
          ABE(I,M+J)=0.0D0
    5   CONTINUE
    6 CONTINUE
    7 CONTINUE
C
C     GAUSSIAN ELIMINATION WITH PARTIAL ELIMINATION
C
      DO 40 K=1,N1
        LM=MIN0(N-K,ML)
        LM1=LM+1
        LM2=ML1-LM
C
C     SEARCH FOR PIVOT INDEX
C
        L=-IDAMAX(LM1,ABE(LM+K,LM2),LDB)+LM1+K
        IPVT(K)=L
        MP=MIN0(MB,N-K)
C
C     SWAP ROWS IF NECESSARY
C
        IF(L.NE.K)CALL DSWAP(MP+1,ABE(K,ML1),LDA,ABE(L,LM2),LDA)
C
C     SKIP COLUMN REDUCTION IF PIVOT IS ZERO
C
        IF(ABE(K,ML1).EQ.0.0D0) GO TO 20
C
C     COMPUTE MULTIPLIERS
C
        T=-1.0/ABE(K,ML1)
        CALL DSCAL(LM,T,ABE(LM+K,LM2),LDB)
C
C     ROW ELIMINATION WITH COLUMN INDEXING
C
        DO 10 J=1,MP
          CALL DAXPY(LM,ABE(K,ML1+J),ABE(LM+K,LM2),LDB,ABE(LM+K,LM2+J),L
     1DB)
   10   CONTINUE
        GO TO 30
   20   CONTINUE
        INFO=K
   30   CONTINUE
   40 CONTINUE
   50 CONTINUE
      IPVT(N)=N
      IF(ABE(N,ML1).EQ.0.0D0) INFO=N
      RETURN
      END
      INTEGER FUNCTION IDAMAX(N,DX,INCX)
C***BEGIN PROLOGUE  IDAMAX
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A
C***KEYWORDS  BLAS,VECTOR,DOUBLE PRECISION,LARGEST COMPONENT
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C     FIND LARGEST COMPONENT OF D.P. VECTOR
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C
C     --OUTPUT--
C   IDAMAX  SMALLEST INDEX (ZERO IF N.LE.0)
C
C     FIND SMALLEST INDEX OF MAXIMUM MAGNITUDE OF DOUBLE PRECISION DX.
C     IDAMAX =  FIRST I, I = 1 TO N, TO MINIMIZE  ABS(DX(1-INCX+I*INCX)
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  IDAMAX
C
      DOUBLE PRECISION DX(1),DMAX,XMAG
C***FIRST EXECUTABLE STATEMENT  IDAMAX
      IDAMAX = 0
      IF(N.LE.0) RETURN
      IDAMAX = 1
      IF(N.LE.1)RETURN
      IF(INCX.EQ.1)GOTO 20
C
C        CODE FOR INCREMENTS NOT EQUAL TO 1.
C
      DMAX = DABS(DX(1))
      NS = N*INCX
      II = 1
          DO 10 I = 1,NS,INCX
          XMAG = DABS(DX(I))
          IF(XMAG.LE.DMAX) GO TO 5
          IDAMAX = II
          DMAX = XMAG
    5     II = II + 1
   10     CONTINUE
      RETURN
C
C        CODE FOR INCREMENTS EQUAL TO 1.
C
   20 DMAX = DABS(DX(1))
      DO 30 I = 2,N
          XMAG = DABS(DX(I))
          IF(XMAG.LE.DMAX) GO TO 30
          IDAMAX = I
          DMAX = XMAG
   30 CONTINUE
      RETURN
      END
      SUBROUTINE DSWAP(N,DX,INCX,DY,INCY)
C***BEGIN PROLOGUE  DSWAP
C***REVISION DATE  811015   (YYMMDD)
C***CATEGORY NO.  F1A
C***KEYWORDS  BLAS,VECTOR,DOUBLE PRECISION,SWAP,INTERCHANGE
C***DATE WRITTEN  OCTOBER 1979
C***AUTHOR LAWSON C. (JPL),HANSON R. (SLA),
C                            KINCAID D. (U TEXAS), KROGH F. (JPL)
C***PURPOSE
C     INTERCHANGE D.P. VECTORS
C***DESCRIPTION
C                B L A S  SUBPROGRAM
C    DESCRIPTION OF PARAMETERS
C
C     --INPUT--
C        N  NUMBER OF ELEMENTS IN INPUT VECTOR(S)
C       DX  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCX  STORAGE SPACING BETWEEN ELEMENTS OF DX
C       DY  DOUBLE PRECISION VECTOR WITH N ELEMENTS
C     INCY  STORAGE SPACING BETWEEN ELEMENTS OF DY
C
C     --OUTPUT--
C       DX  INPUT VECTOR DY (UNCHANGED IF N.LE.0)
C       DY  INPUT VECTOR DX (UNCHANGED IF N.LE.0)
C
C     INTERCHANGE DOUBLE PRECISION DX AND DOUBLE PRECISION DY.
C     FOR I = 0 TO N-1, INTERCHANGE  DX(LX+I*INCX) AND DY(LY+I*INCY),
C     WHERE LX = 1 IF INCX .GE. 0, ELSE LX = (-INCX)*N, AND LY IS
C     DEFINED IN A SIMILAR WAY USING INCY.
C
C***REFERENCES
C  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,
C   *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,
C  ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C  VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  DSWAP
C
      DOUBLE PRECISION DX(1),DY(1),DTEMP1,DTEMP2,DTEMP3
C***FIRST EXECUTABLE STATEMENT  DSWAP
      IF(N.LE.0)RETURN
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60
    5 CONTINUE
C
C       CODE FOR UNEQUAL OR NONPOSITIVE INCREMENTS.
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DTEMP1 = DX(IX)
        DX(IX) = DY(IY)
        DY(IY) = DTEMP1
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C       CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C       CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 3.
C
   20 M = MOD(N,3)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DTEMP1 = DX(I)
        DX(I) = DY(I)
        DY(I) = DTEMP1
   30 CONTINUE
      IF( N .LT. 3 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,3
        DTEMP1 = DX(I)
        DTEMP2 = DX(I+1)
        DTEMP3 = DX(I+2)
        DX(I) = DY(I)
        DX(I+1) = DY(I+1)
        DX(I+2) = DY(I+2)
        DY(I) = DTEMP1
        DY(I+1) = DTEMP2
        DY(I+2) = DTEMP3
   50 CONTINUE
      RETURN
   60 CONTINUE
C
C     CODE FOR EQUAL, POSITIVE, NONUNIT INCREMENTS.
C
      NS = N*INCX
        DO 70 I=1,NS,INCX
        DTEMP1 = DX(I)
        DX(I) = DY(I)
        DY(I) = DTEMP1
   70   CONTINUE
      RETURN
      END
