      SUBROUTINE DSVDC(X,LDX,N,P,S,E,U,LDU,V,LDV,WORK,JOB,INFO)                 
C***BEGIN PROLOGUE  DSVDC                                                       
C***DATE WRITTEN   790319   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D6                                                            
C***KEYWORDS  DOUBLE PRECISION,LINEAR ALGEBRA,LINPACK,MATRIX,                   
C             SINGULAR VALUE DECOMPOSITION                                      
C***AUTHOR  STEWART, G. W., (U. OF MARYLAND)                                    
C***PURPOSE  Perform the singular value decomposition of a d.p. NXP             
C            matrix.                                                            
C***DESCRIPTION                                                                 
C                                                                               
C     DSVDC is a subroutine to reduce a double precision NxP matrix X           
C     by orthogonal transformations U and V to diagonal form.  The              
C     diagonal elements S(I) are the singular values of X.  The                 
C     columns of U are the corresponding left singular vectors,                 
C     and the columns of V the right singular vectors.                          
C                                                                               
C     On Entry                                                                  
C                                                                               
C         X         DOUBLE PRECISION(LDX,P), where LDX .GE. N.                  
C                   X contains the matrix whose singular value                  
C                   decomposition is to be computed.  X is                      
C                   destroyed by DSVDC.                                         
C                                                                               
C         LDX       INTEGER.                                                    
C                   LDX is the leading dimension of the array X.                
C                                                                               
C         N         INTEGER.                                                    
C                   N is the number of columns of the matrix X.                 
C                                                                               
C         P         INTEGER.                                                    
C                   P is the number of rows of the matrix X.                    
C                                                                               
C         LDU       INTEGER.                                                    
C                   LDU is the leading dimension of the array U.                
C                   (See below).                                                
C                                                                               
C         LDV       INTEGER.                                                    
C                   LDV is the leading dimension of the array V.                
C                   (See below).                                                
C                                                                               
C         WORK      DOUBLE PRECISION(N).                                        
C                   WORK is a scratch array.                                    
C                                                                               
C         JOB       INTEGER.                                                    
C                   JOB controls the computation of the singular                
C                   vectors.  It has the decimal expansion AB                   
C                   with the following meaning                                  
C                                                                               
C                        A .EQ. 0    do not compute the left singular           
C                                  vectors.                                     
C                        A .EQ. 1    return the N left singular vectors         
C                                  in U.                                        
C                        A .GE. 2    return the first MIN(N,P) singular         
C                                  vectors in U.                                
C                        B .EQ. 0    do not compute the right singular          
C                                  vectors.                                     
C                        B .EQ. 1    return the right singular vectors          
C                                  in V.                                        
C                                                                               
C     On Return                                                                 
C                                                                               
C         S         DOUBLE PRECISION(MM), where MM=MIN(N+1,P).                  
C                   The first MIN(N,P) entries of S contain the                 
C                   singular values of X arranged in descending                 
C                   order of magnitude.                                         
C                                                                               
C         E         DOUBLE PRECISION(P).                                        
C                   E ordinarily contains zeros.  However see the               
C                   discussion of INFO for exceptions.                          
C                                                                               
C         U         DOUBLE PRECISION(LDU,K), where LDU .GE. N.                  
C                   If JOBA .EQ. 1, then K .EQ. N.                              
C                   If JOBA .GE. 2, then K .EQ. MIN(N,P).                       
C                   U contains the matrix of right singular vectors.            
C                   U is not referenced if JOBA .EQ. 0.  If N .LE. P            
C                   or if JOBA .EQ. 2, then U may be identified with X          
C                   in the subroutine call.                                     
C                                                                               
C         V         DOUBLE PRECISION(LDV,P), where LDV .GE. P.                  
C                   V contains the matrix of right singular vectors.            
C                   V is not referenced if JOB .EQ. 0.  If P .LE. N,            
C                   then V may be identified with X in the                      
C                   subroutine call.                                            
C                                                                               
C         INFO      INTEGER.                                                    
C                   The singular values (and their corresponding                
C                   singular vectors) S(INFO+1),S(INFO+2),...,S(M)              
C                   are correct (here M=MIN(N,P)).  Thus if                     
C                   INFO .EQ. 0, all the singular values and their              
C                   vectors are correct.  In any event, the matrix              
C                   B = TRANS(U)*X*V is the bidiagonal matrix                   
C                   with the elements of S on its diagonal and the              
C                   elements of E on its super-diagonal (TRANS(U)               
C                   is the transpose of U).  Thus the singular                  
C                   values of X and B are the same.                             
C                                                                               
C     LINPACK.  This version dated 03/19/79 .                                   
C     G. W. Stewart, University of Maryland, Argonne National Lab.              
C                                                                               
C     DSVDC uses the following functions and subprograms.                       
C                                                                               
C     External DROT                                                             
C     BLAS DAXPY,DDOT,DSCAL,DSWAP,DNRM2,DROTG                                   
C     Fortran DABS,DMAX1,MAX0,MIN0,MOD,DSQRT                                    
C***REFERENCES  DONGARRA J.J., BUNCH J.R., MOLER C.B., STEWART G.W.,            
C                 *LINPACK USERS  GUIDE*, SIAM, 1979.                           
C***ROUTINES CALLED  DAXPY,DDOT,DNRM2,DROT,DROTG,DSCAL,DSWAP                    
C***END PROLOGUE  DSVDC                                                         
      INTEGER LDX,N,P,LDU,LDV,JOB,INFO                                          
      DOUBLE PRECISION X(LDX,1),S(1),E(1),U(LDU,1),V(LDV,1),WORK(1)             
C                                                                               
C                                                                               
      INTEGER I,ITER,J,JOBU,K,KASE,KK,L,LL,LLS,LM1,LP1,LS,LU,M,MAXIT,           
     1        MM,MM1,MP1,NCT,NCTP1,NCU,NRT,NRTP1                                
      DOUBLE PRECISION DDOT,T,R                                                 
      DOUBLE PRECISION B,C,CS,EL,EMM1,F,G,DNRM2,SCALE,SHIFT,SL,SM,SN,           
     1                 SMM1,T1,TEST,ZTEST                                       
      LOGICAL WANTU,WANTV                                                       
C                                                                               
C     SET THE MAXIMUM NUMBER OF ITERATIONS.                                     
C                                                                               
C***FIRST EXECUTABLE STATEMENT  DSVDC                                           
      MAXIT = 30                                                                
C                                                                               
C     DETERMINE WHAT IS TO BE COMPUTED.                                         
C                                                                               
      WANTU = .FALSE.                                                           
      WANTV = .FALSE.                                                           
      JOBU = MOD(JOB,100)/10                                                    
      NCU = N                                                                   
      IF (JOBU .GT. 1) NCU = MIN0(N,P)                                          
      IF (JOBU .NE. 0) WANTU = .TRUE.                                           
      IF (MOD(JOB,10) .NE. 0) WANTV = .TRUE.                                    
C                                                                               
C     REDUCE X TO BIDIAGONAL FORM, STORING THE DIAGONAL ELEMENTS                
C     IN S AND THE SUPER-DIAGONAL ELEMENTS IN E.                                
C                                                                               
      INFO = 0                                                                  
      NCT = MIN0(N-1,P)                                                         
      NRT = MAX0(0,MIN0(P-2,N))                                                 
      LU = MAX0(NCT,NRT)                                                        
      IF (LU .LT. 1) GO TO 170                                                  
      DO 160 L = 1, LU                                                          
         LP1 = L + 1                                                            
         IF (L .GT. NCT) GO TO 20                                               
C                                                                               
C           COMPUTE THE TRANSFORMATION FOR THE L-TH COLUMN AND                  
C           PLACE THE L-TH DIAGONAL IN S(L).                                    
C                                                                               
            S(L) = DNRM2(N-L+1,X(L,L),1)                                        
            IF (S(L) .EQ. 0.0D0) GO TO 10                                       
               IF (X(L,L) .NE. 0.0D0) S(L) = DSIGN(S(L),X(L,L))                 
               CALL DSCAL(N-L+1,1.0D0/S(L),X(L,L),1)                            
               X(L,L) = 1.0D0 + X(L,L)                                          
   10       CONTINUE                                                            
            S(L) = -S(L)                                                        
   20    CONTINUE                                                               
         IF (P .LT. LP1) GO TO 50                                               
         DO 40 J = LP1, P                                                       
            IF (L .GT. NCT) GO TO 30                                            
            IF (S(L) .EQ. 0.0D0) GO TO 30                                       
C                                                                               
C              APPLY THE TRANSFORMATION.                                        
C                                                                               
               T = -DDOT(N-L+1,X(L,L),1,X(L,J),1)/X(L,L)                        
               CALL DAXPY(N-L+1,T,X(L,L),1,X(L,J),1)                            
   30       CONTINUE                                                            
C                                                                               
C           PLACE THE L-TH ROW OF X INTO  E FOR THE                             
C           SUBSEQUENT CALCULATION OF THE ROW TRANSFORMATION.                   
C                                                                               
            E(J) = X(L,J)                                                       
   40    CONTINUE                                                               
   50    CONTINUE                                                               
         IF (.NOT.WANTU .OR. L .GT. NCT) GO TO 70                               
C                                                                               
C           PLACE THE TRANSFORMATION IN U FOR SUBSEQUENT BACK                   
C           MULTIPLICATION.                                                     
C                                                                               
            DO 60 I = L, N                                                      
               U(I,L) = X(I,L)                                                  
   60       CONTINUE                                                            
   70    CONTINUE                                                               
         IF (L .GT. NRT) GO TO 150                                              
C                                                                               
C           COMPUTE THE L-TH ROW TRANSFORMATION AND PLACE THE                   
C           L-TH SUPER-DIAGONAL IN E(L).                                        
C                                                                               
            E(L) = DNRM2(P-L,E(LP1),1)                                          
            IF (E(L) .EQ. 0.0D0) GO TO 80                                       
               IF (E(LP1) .NE. 0.0D0) E(L) = DSIGN(E(L),E(LP1))                 
               CALL DSCAL(P-L,1.0D0/E(L),E(LP1),1)                              
               E(LP1) = 1.0D0 + E(LP1)                                          
   80       CONTINUE                                                            
            E(L) = -E(L)                                                        
            IF (LP1 .GT. N .OR. E(L) .EQ. 0.0D0) GO TO 120                      
C                                                                               
C              APPLY THE TRANSFORMATION.                                        
C                                                                               
               DO 90 I = LP1, N                                                 
                  WORK(I) = 0.0D0                                               
   90          CONTINUE                                                         
               DO 100 J = LP1, P                                                
                  CALL DAXPY(N-L,E(J),X(LP1,J),1,WORK(LP1),1)                   
  100          CONTINUE                                                         
               DO 110 J = LP1, P                                                
                  CALL DAXPY(N-L,-E(J)/E(LP1),WORK(LP1),1,X(LP1,J),1)           
  110          CONTINUE                                                         
  120       CONTINUE                                                            
            IF (.NOT.WANTV) GO TO 140                                           
C                                                                               
C              PLACE THE TRANSFORMATION IN V FOR SUBSEQUENT                     
C              BACK MULTIPLICATION.                                             
C                                                                               
               DO 130 I = LP1, P                                                
                  V(I,L) = E(I)                                                 
  130          CONTINUE                                                         
  140       CONTINUE                                                            
  150    CONTINUE                                                               
  160 CONTINUE                                                                  
  170 CONTINUE                                                                  
C                                                                               
C     SET UP THE FINAL BIDIAGONAL MATRIX OR ORDER M.                            
C                                                                               
      M = MIN0(P,N+1)                                                           
      NCTP1 = NCT + 1                                                           
      NRTP1 = NRT + 1                                                           
      IF (NCT .LT. P) S(NCTP1) = X(NCTP1,NCTP1)                                 
      IF (N .LT. M) S(M) = 0.0D0                                                
      IF (NRTP1 .LT. M) E(NRTP1) = X(NRTP1,M)                                   
      E(M) = 0.0D0                                                              
C                                                                               
C     IF REQUIRED, GENERATE U.                                                  
C                                                                               
      IF (.NOT.WANTU) GO TO 300                                                 
         IF (NCU .LT. NCTP1) GO TO 200                                          
         DO 190 J = NCTP1, NCU                                                  
            DO 180 I = 1, N                                                     
               U(I,J) = 0.0D0                                                   
  180       CONTINUE                                                            
            U(J,J) = 1.0D0                                                      
  190    CONTINUE                                                               
  200    CONTINUE                                                               
         IF (NCT .LT. 1) GO TO 290                                              
         DO 280 LL = 1, NCT                                                     
            L = NCT - LL + 1                                                    
            IF (S(L) .EQ. 0.0D0) GO TO 250                                      
               LP1 = L + 1                                                      
               IF (NCU .LT. LP1) GO TO 220                                      
               DO 210 J = LP1, NCU                                              
                  T = -DDOT(N-L+1,U(L,L),1,U(L,J),1)/U(L,L)                     
                  CALL DAXPY(N-L+1,T,U(L,L),1,U(L,J),1)                         
  210          CONTINUE                                                         
  220          CONTINUE                                                         
               CALL DSCAL(N-L+1,-1.0D0,U(L,L),1)                                
               U(L,L) = 1.0D0 + U(L,L)                                          
               LM1 = L - 1                                                      
               IF (LM1 .LT. 1) GO TO 240                                        
               DO 230 I = 1, LM1                                                
                  U(I,L) = 0.0D0                                                
  230          CONTINUE                                                         
  240          CONTINUE                                                         
            GO TO 270                                                           
  250       CONTINUE                                                            
               DO 260 I = 1, N                                                  
                  U(I,L) = 0.0D0                                                
  260          CONTINUE                                                         
               U(L,L) = 1.0D0                                                   
  270       CONTINUE                                                            
  280    CONTINUE                                                               
  290    CONTINUE                                                               
  300 CONTINUE                                                                  
C                                                                               
C     IF IT IS REQUIRED, GENERATE V.                                            
C                                                                               
      IF (.NOT.WANTV) GO TO 350                                                 
         DO 340 LL = 1, P                                                       
            L = P - LL + 1                                                      
            LP1 = L + 1                                                         
            IF (L .GT. NRT) GO TO 320                                           
            IF (E(L) .EQ. 0.0D0) GO TO 320                                      
               DO 310 J = LP1, P                                                
                  T = -DDOT(P-L,V(LP1,L),1,V(LP1,J),1)/V(LP1,L)                 
                  CALL DAXPY(P-L,T,V(LP1,L),1,V(LP1,J),1)                       
  310          CONTINUE                                                         
  320       CONTINUE                                                            
            DO 330 I = 1, P                                                     
               V(I,L) = 0.0D0                                                   
  330       CONTINUE                                                            
            V(L,L) = 1.0D0                                                      
  340    CONTINUE                                                               
  350 CONTINUE                                                                  
C                                                                               
C     MAIN ITERATION LOOP FOR THE SINGULAR VALUES.                              
C                                                                               
      MM = M                                                                    
      ITER = 0                                                                  
  360 CONTINUE                                                                  
C                                                                               
C        QUIT IF ALL THE SINGULAR VALUES HAVE BEEN FOUND.                       
C                                                                               
C     ...EXIT                                                                   
         IF (M .EQ. 0) GO TO 620                                                
C                                                                               
C        IF TOO MANY ITERATIONS HAVE BEEN PERFORMED, SET                        
C        FLAG AND RETURN.                                                       
C                                                                               
         IF (ITER .LT. MAXIT) GO TO 370                                         
            INFO = M                                                            
C     ......EXIT                                                                
            GO TO 620                                                           
  370    CONTINUE                                                               
C                                                                               
C        THIS SECTION OF THE PROGRAM INSPECTS FOR                               
C        NEGLIGIBLE ELEMENTS IN THE S AND E ARRAYS.  ON                         
C        COMPLETION THE VARIABLES KASE AND L ARE SET AS FOLLOWS.                
C                                                                               
C           KASE = 1     IF S(M) AND E(L-1) ARE NEGLIGIBLE AND L.LT.M           
C           KASE = 2     IF S(L) IS NEGLIGIBLE AND L.LT.M                       
C           KASE = 3     IF E(L-1) IS NEGLIGIBLE, L.LT.M, AND                   
C                        S(L), ..., S(M) ARE NOT NEGLIGIBLE (QR STEP).          
C           KASE = 4     IF E(M-1) IS NEGLIGIBLE (CONVERGENCE).                 
C                                                                               
         DO 390 LL = 1, M                                                       
            L = M - LL                                                          
C        ...EXIT                                                                
            IF (L .EQ. 0) GO TO 400                                             
            TEST = DABS(S(L)) + DABS(S(L+1))                                    
            ZTEST = TEST + DABS(E(L))                                           
            IF (ZTEST .NE. TEST) GO TO 380                                      
               E(L) = 0.0D0                                                     
C        ......EXIT                                                             
               GO TO 400                                                        
  380       CONTINUE                                                            
  390    CONTINUE                                                               
  400    CONTINUE                                                               
         IF (L .NE. M - 1) GO TO 410                                            
            KASE = 4                                                            
         GO TO 480                                                              
  410    CONTINUE                                                               
            LP1 = L + 1                                                         
            MP1 = M + 1                                                         
            DO 430 LLS = LP1, MP1                                               
               LS = M - LLS + LP1                                               
C           ...EXIT                                                             
               IF (LS .EQ. L) GO TO 440                                         
               TEST = 0.0D0                                                     
               IF (LS .NE. M) TEST = TEST + DABS(E(LS))                         
               IF (LS .NE. L + 1) TEST = TEST + DABS(E(LS-1))                   
               ZTEST = TEST + DABS(S(LS))                                       
               IF (ZTEST .NE. TEST) GO TO 420                                   
                  S(LS) = 0.0D0                                                 
C           ......EXIT                                                          
                  GO TO 440                                                     
  420          CONTINUE                                                         
  430       CONTINUE                                                            
  440       CONTINUE                                                            
            IF (LS .NE. L) GO TO 450                                            
               KASE = 3                                                         
            GO TO 470                                                           
  450       CONTINUE                                                            
            IF (LS .NE. M) GO TO 460                                            
               KASE = 1                                                         
            GO TO 470                                                           
  460       CONTINUE                                                            
               KASE = 2                                                         
               L = LS                                                           
  470       CONTINUE                                                            
  480    CONTINUE                                                               
         L = L + 1                                                              
C                                                                               
C        PERFORM THE TASK INDICATED BY KASE.                                    
C                                                                               
         GO TO (490,520,540,570), KASE                                          
C                                                                               
C        DEFLATE NEGLIGIBLE S(M).                                               
C                                                                               
  490    CONTINUE                                                               
            MM1 = M - 1                                                         
            F = E(M-1)                                                          
            E(M-1) = 0.0D0                                                      
            DO 510 KK = L, MM1                                                  
               K = MM1 - KK + L                                                 
               T1 = S(K)                                                        
               CALL DROTG(T1,F,CS,SN)                                           
               S(K) = T1                                                        
               IF (K .EQ. L) GO TO 500                                          
                  F = -SN*E(K-1)                                                
                  E(K-1) = CS*E(K-1)                                            
  500          CONTINUE                                                         
               IF (WANTV) CALL DROT(P,V(1,K),1,V(1,M),1,CS,SN)                  
  510       CONTINUE                                                            
         GO TO 610                                                              
C                                                                               
C        SPLIT AT NEGLIGIBLE S(L).                                              
C                                                                               
  520    CONTINUE                                                               
            F = E(L-1)                                                          
            E(L-1) = 0.0D0                                                      
            DO 530 K = L, M                                                     
               T1 = S(K)                                                        
               CALL DROTG(T1,F,CS,SN)                                           
               S(K) = T1                                                        
               F = -SN*E(K)                                                     
               E(K) = CS*E(K)                                                   
               IF (WANTU) CALL DROT(N,U(1,K),1,U(1,L-1),1,CS,SN)                
  530       CONTINUE                                                            
         GO TO 610                                                              
C                                                                               
C        PERFORM ONE QR STEP.                                                   
C                                                                               
  540    CONTINUE                                                               
C                                                                               
C           CALCULATE THE SHIFT.                                                
C                                                                               
            SCALE = DMAX1(DABS(S(M)),DABS(S(M-1)),DABS(E(M-1)),                 
     1                    DABS(S(L)),DABS(E(L)))                                
            SM = S(M)/SCALE                                                     
            SMM1 = S(M-1)/SCALE                                                 
            EMM1 = E(M-1)/SCALE                                                 
            SL = S(L)/SCALE                                                     
            EL = E(L)/SCALE                                                     
            B = ((SMM1 + SM)*(SMM1 - SM) + EMM1**2)/2.0D0                       
            C = (SM*EMM1)**2                                                    
            SHIFT = 0.0D0                                                       
            IF (B .EQ. 0.0D0 .AND. C .EQ. 0.0D0) GO TO 550                      
               SHIFT = DSQRT(B**2+C)                                            
               IF (B .LT. 0.0D0) SHIFT = -SHIFT                                 
               SHIFT = C/(B + SHIFT)                                            
  550       CONTINUE                                                            
            F = (SL + SM)*(SL - SM) - SHIFT                                     
            G = SL*EL                                                           
C                                                                               
C           CHASE ZEROS.                                                        
C                                                                               
            MM1 = M - 1                                                         
            DO 560 K = L, MM1                                                   
               CALL DROTG(F,G,CS,SN)                                            
               IF (K .NE. L) E(K-1) = F                                         
               F = CS*S(K) + SN*E(K)                                            
               E(K) = CS*E(K) - SN*S(K)                                         
               G = SN*S(K+1)                                                    
               S(K+1) = CS*S(K+1)                                               
               IF (WANTV) CALL DROT(P,V(1,K),1,V(1,K+1),1,CS,SN)                
               CALL DROTG(F,G,CS,SN)                                            
               S(K) = F                                                         
               F = CS*E(K) + SN*S(K+1)                                          
               S(K+1) = -SN*E(K) + CS*S(K+1)                                    
               G = SN*E(K+1)                                                    
               E(K+1) = CS*E(K+1)                                               
               IF (WANTU .AND. K .LT. N)                                        
     1            CALL DROT(N,U(1,K),1,U(1,K+1),1,CS,SN)                        
  560       CONTINUE                                                            
            E(M-1) = F                                                          
            ITER = ITER + 1                                                     
         GO TO 610                                                              
C                                                                               
C        CONVERGENCE.                                                           
C                                                                               
  570    CONTINUE                                                               
C                                                                               
C           MAKE THE SINGULAR VALUE  POSITIVE.                                  
C                                                                               
            IF (S(L) .GE. 0.0D0) GO TO 580                                      
               S(L) = -S(L)                                                     
               IF (WANTV) CALL DSCAL(P,-1.0D0,V(1,L),1)                         
  580       CONTINUE                                                            
C                                                                               
C           ORDER THE SINGULAR VALUE.                                           
C                                                                               
  590       IF (L .EQ. MM) GO TO 600                                            
C           ...EXIT                                                             
               IF (S(L) .GE. S(L+1)) GO TO 600                                  
               T = S(L)                                                         
               S(L) = S(L+1)                                                    
               S(L+1) = T                                                       
               IF (WANTV .AND. L .LT. P)                                        
     1            CALL DSWAP(P,V(1,L),1,V(1,L+1),1)                             
               IF (WANTU .AND. L .LT. N)                                        
     1            CALL DSWAP(N,U(1,L),1,U(1,L+1),1)                             
               L = L + 1                                                        
            GO TO 590                                                           
  600       CONTINUE                                                            
            ITER = 0                                                            
            M = M - 1                                                           
  610    CONTINUE                                                               
      GO TO 360                                                                 
  620 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       

      SUBROUTINE DAXPY(N,DA,DX,INCX,DY,INCY)                                    
C***BEGIN PROLOGUE  DAXPY                                                       
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A7                                                          
C***KEYWORDS  BLAS,DOUBLE PRECISION,LINEAR ALGEBRA,TRIAD,VECTOR                 
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  D.P computation y = a*x + y                                        
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DA  double precision scalar multiplier                                  
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C       DY  double precision vector with N elements                             
C     INCY  storage spacing between elements of DY                              
C                                                                               
C     --Output--                                                                
C       DY  double precision result (unchanged if N .LE. 0)                     
C                                                                               
C     Overwrite double precision DY with double precision DA*DX + DY.           
C     For I = 0 to N-1, replace  DY(LY+I*INCY) with DA*DX(LX+I*INCX) +          
C       DY(LY+I*INCY), where LX = 1 if INCX .GE. 0, else LX = (-INCX)*N         
C       and LY is defined in a similar way using INCY.                          
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DAXPY                                                         
C                                                                               
      DOUBLE PRECISION DX(1),DY(1),DA                                           
C***FIRST EXECUTABLE STATEMENT  DAXPY                                           
      IF(N.LE.0.OR.DA.EQ.0.D0) RETURN                                           
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60                                       
    5 CONTINUE                                                                  
C                                                                               
C        CODE FOR NONEQUAL OR NONPOSITIVE INCREMENTS.                           
C                                                                               
      IX = 1                                                                    
      IY = 1                                                                    
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1                                         
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1                                         
      DO 10 I = 1,N                                                             
        DY(IY) = DY(IY) + DA*DX(IX)                                             
        IX = IX + INCX                                                          
        IY = IY + INCY                                                          
   10 CONTINUE                                                                  
      RETURN                                                                    
C                                                                               
C        CODE FOR BOTH INCREMENTS EQUAL TO 1                                    
C                                                                               
C                                                                               
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 4.           
C                                                                               
   20 M = MOD(N,4)                                                              
      IF( M .EQ. 0 ) GO TO 40                                                   
      DO 30 I = 1,M                                                             
        DY(I) = DY(I) + DA*DX(I)                                                
   30 CONTINUE                                                                  
      IF( N .LT. 4 ) RETURN                                                     
   40 MP1 = M + 1                                                               
      DO 50 I = MP1,N,4                                                         
        DY(I) = DY(I) + DA*DX(I)                                                
        DY(I + 1) = DY(I + 1) + DA*DX(I + 1)                                    
        DY(I + 2) = DY(I + 2) + DA*DX(I + 2)                                    
        DY(I + 3) = DY(I + 3) + DA*DX(I + 3)                                    
   50 CONTINUE                                                                  
      RETURN                                                                    
C                                                                               
C        CODE FOR EQUAL, POSITIVE, NONUNIT INCREMENTS.                          
C                                                                               
   60 CONTINUE                                                                  
      NS = N*INCX                                                               
          DO 70 I=1,NS,INCX                                                     
          DY(I) = DA*DX(I) + DY(I)                                              
   70     CONTINUE                                                              
      RETURN                                                                    
      END                                                                       

      DOUBLE PRECISION FUNCTION DDOT(N,DX,INCX,DY,INCY)                         
C***BEGIN PROLOGUE  DDOT                                                        
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A4                                                          
C***KEYWORDS  BLAS,DOUBLE PRECISION,INNER PRODUCT,LINEAR ALGEBRA,VECTOR         
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  D.P. inner product of d.p. vectors                                 
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C       DY  double precision vector with N elements                             
C     INCY  storage spacing between elements of DY                              
C                                                                               
C     --Output--                                                                
C     DDOT  double precision dot product (zero if N .LE. 0)                     
C                                                                               
C     Returns the dot product of double precision DX and DY.                    
C     DDOT = sum for I = 0 to N-1 of  DX(LX+I*INCX) * DY(LY+I*INCY)             
C     where LX = 1 if INCX .GE. 0, else LX = (-INCX)*N, and LY is               
C     defined in a similar way using INCY.                                      
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DDOT                                                          
C                                                                               
      DOUBLE PRECISION DX(1),DY(1)                                              
C***FIRST EXECUTABLE STATEMENT  DDOT                                            
      DDOT = 0.D0                                                               
      IF(N.LE.0)RETURN                                                          
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60                                       
    5 CONTINUE                                                                  
C                                                                               
C         CODE FOR UNEQUAL OR NONPOSITIVE INCREMENTS.                           
C                                                                               
      IX = 1                                                                    
      IY = 1                                                                    
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1                                         
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1                                         
      DO 10 I = 1,N                                                             
         DDOT = DDOT + DX(IX)*DY(IY)                                            
        IX = IX + INCX                                                          
        IY = IY + INCY                                                          
   10 CONTINUE                                                                  
      RETURN                                                                    
C                                                                               
C        CODE FOR BOTH INCREMENTS EQUAL TO 1.                                   
C                                                                               
C                                                                               
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 5.           
C                                                                               
   20 M = MOD(N,5)                                                              
      IF( M .EQ. 0 ) GO TO 40                                                   
      DO 30 I = 1,M                                                             
         DDOT = DDOT + DX(I)*DY(I)                                              
   30 CONTINUE                                                                  
      IF( N .LT. 5 ) RETURN                                                     
   40 MP1 = M + 1                                                               
      DO 50 I = MP1,N,5                                                         
         DDOT = DDOT + DX(I)*DY(I) + DX(I+1)*DY(I+1) +                          
     1   DX(I + 2)*DY(I + 2) + DX(I + 3)*DY(I + 3) + DX(I + 4)*DY(I + 4)        
   50 CONTINUE                                                                  
      RETURN                                                                    
C                                                                               
C         CODE FOR POSITIVE EQUAL INCREMENTS .NE.1.                             
C                                                                               
   60 CONTINUE                                                                  
      NS = N*INCX                                                               
          DO 70 I=1,NS,INCX                                                     
          DDOT = DDOT + DX(I)*DY(I)                                             
   70     CONTINUE                                                              
      RETURN                                                                    
      END                                                                       

      DOUBLE PRECISION FUNCTION DNRM2(N,DX,INCX)                                
C***BEGIN PROLOGUE  DNRM2                                                       
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A3B                                                         
C***KEYWORDS  BLAS,DOUBLE PRECISION,EUCLIDEAN,L2,LENGTH,LINEAR ALGEBRA,         
C             NORM,VECTOR                                                       
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  Euclidean length (L2 norm) of d.p. vector                          
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C                                                                               
C     --Output--                                                                
C    DNRM2  double precision result (zero if N .LE. 0)                          
C                                                                               
C     Euclidean norm of the N-vector stored in DX() with storage                
C     increment INCX .                                                          
C     If    N .LE. 0 return with result = 0.                                    
C     If N .GE. 1 then INCX must be .GE. 1                                      
C                                                                               
C           C.L. Lawson, 1978 Jan 08                                            
C                                                                               
C     Four phase method     using two built-in constants that are               
C     hopefully applicable to all machines.                                     
C         CUTLO = maximum of  DSQRT(U/EPS)  over all known machines.            
C         CUTHI = minimum of  DSQRT(V)      over all known machines.            
C     where                                                                     
C         EPS = smallest no. such that EPS + 1. .GT. 1.                         
C         U   = smallest positive no.   (underflow limit)                       
C         V   = largest  no.            (overflow  limit)                       
C                                                                               
C     Brief outline of algorithm..                                              
C                                                                               
C     Phase 1    scans zero components.                                         
C     move to phase 2 when a component is nonzero and .LE. CUTLO                
C     move to phase 3 when a component is .GT. CUTLO                            
C     move to phase 4 when a component is .GE. CUTHI/M                          
C     where M = N for X() real and M = 2*N for complex.                         
C                                                                               
C     Values for CUTLO and CUTHI..                                              
C     From the environmental parameters listed in the IMSL converter            
C     document the limiting values are as followS..                             
C     CUTLO, S.P.   U/EPS = 2**(-102) for  Honeywell.  Close seconds are        
C                   Univac and DEC at 2**(-103)                                 
C                   Thus CUTLO = 2**(-51) = 4.44089E-16                         
C     CUTHI, S.P.   V = 2**127 for Univac, Honeywell, and DEC.                  
C                   Thus CUTHI = 2**(63.5) = 1.30438E19                         
C     CUTLO, D.P.   U/EPS = 2**(-67) for Honeywell and DEC.                     
C                   Thus CUTLO = 2**(-33.5) = 8.23181D-11                       
C     CUTHI, D.P.   same as S.P.  CUTHI = 1.30438D19                            
C     DATA CUTLO, CUTHI / 8.232D-11,  1.304D19 /                                
C     DATA CUTLO, CUTHI / 4.441E-16,  1.304E19 /                                
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DNRM2                                                         
      INTEGER          NEXT                                                     
      DOUBLE PRECISION   DX(1), CUTLO, CUTHI, HITEST, SUM, XMAX,ZERO,ONE        
      DATA   ZERO, ONE /0.0D0, 1.0D0/                                           
C                                                                               
      DATA CUTLO, CUTHI / 8.232D-11,  1.304D19 /                                
C***FIRST EXECUTABLE STATEMENT  DNRM2                                           
      IF(N .GT. 0) GO TO 10                                                     
         DNRM2  = ZERO                                                          
         GO TO 300                                                              
C                                                                               
   10 ASSIGN 30 TO NEXT                                                         
      SUM = ZERO                                                                
      NN = N * INCX                                                             
C                                                 BEGIN MAIN LOOP               
      I = 1                                                                     
   20    GO TO NEXT,(30, 50, 70, 110)                                           
   30 IF( DABS(DX(I)) .GT. CUTLO) GO TO 85                                      
      ASSIGN 50 TO NEXT                                                         
      XMAX = ZERO                                                               
C                                                                               
C                        PHASE 1.  SUM IS ZERO                                  
C                                                                               
   50 IF( DX(I) .EQ. ZERO) GO TO 200                                            
      IF( DABS(DX(I)) .GT. CUTLO) GO TO 85                                      
C                                                                               
C                                PREPARE FOR PHASE 2.                           
      ASSIGN 70 TO NEXT                                                         
      GO TO 105                                                                 
C                                                                               
C                                PREPARE FOR PHASE 4.                           
C                                                                               
  100 I = J                                                                     
      ASSIGN 110 TO NEXT                                                        
      SUM = (SUM / DX(I)) / DX(I)                                               
  105 XMAX = DABS(DX(I))                                                        
      GO TO 115                                                                 
C                                                                               
C                   PHASE 2.  SUM IS SMALL.                                     
C                             SCALE TO AVOID DESTRUCTIVE UNDERFLOW.             
C                                                                               
   70 IF( DABS(DX(I)) .GT. CUTLO ) GO TO 75                                     
C                                                                               
C                     COMMON CODE FOR PHASES 2 AND 4.                           
C                     IN PHASE 4 SUM IS LARGE.  SCALE TO AVOID OVERFLOW.        
C                                                                               
  110 IF( DABS(DX(I)) .LE. XMAX ) GO TO 115                                     
         SUM = ONE + SUM * (XMAX / DX(I))**2                                    
         XMAX = DABS(DX(I))                                                     
         GO TO 200                                                              
C                                                                               
  115 SUM = SUM + (DX(I)/XMAX)**2                                               
      GO TO 200                                                                 
C                                                                               
C                                                                               
C                  PREPARE FOR PHASE 3.                                         
C                                                                               
   75 SUM = (SUM * XMAX) * XMAX                                                 
C                                                                               
C                                                                               
C     FOR REAL OR D.P. SET HITEST = CUTHI/N                                     
C     FOR COMPLEX      SET HITEST = CUTHI/(2*N)                                 
C                                                                               
   85 HITEST = CUTHI/FLOAT( N )                                                 
C                                                                               
C                   PHASE 3.  SUM IS MID-RANGE.  NO SCALING.                    
C                                                                               
      DO 95 J =I,NN,INCX                                                        
      IF(DABS(DX(J)) .GE. HITEST) GO TO 100                                     
   95    SUM = SUM + DX(J)**2                                                   
      DNRM2 = DSQRT( SUM )                                                      
      GO TO 300                                                                 
C                                                                               
  200 CONTINUE                                                                  
      I = I + INCX                                                              
      IF ( I .LE. NN ) GO TO 20                                                 
C                                                                               
C              END OF MAIN LOOP.                                                
C                                                                               
C              COMPUTE SQUARE ROOT AND ADJUST FOR SCALING.                      
C                                                                               
      DNRM2 = XMAX * DSQRT(SUM)                                                 
  300 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       

      SUBROUTINE DROT(N,DX,INCX,DY,INCY,DC,DS)                                  
C***BEGIN PROLOGUE  DROT                                                        
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A8                                                          
C***KEYWORDS  BLAS,GIVENS ROTATION,LINEAR ALGEBRA,VECTOR                        
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  APPLY d.p. Givens rotation                                         
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C       DY  double precision vector with N elements                             
C     INCY  storage spacing between elements of DY                              
C       DC  D.P. element of rotation matrix                                     
C       DS  D.P. element of rotation matrix                                     
C                                                                               
C     --Output--                                                                
C       DX  rotated vector (unchanged if N .LE. 0)                              
C       DY  rotated vector (unchanged if N .LE. 0)                              
C                                                                               
C     Multiply the 2 X 2 matrix  ( DC DS) times the 2 X N matrix (DX**T)        
C                                (-DS DC)                        (DY**T)        
C     where **T indicates transpose.  The elements of DX are in                 
C     DX(LX+I*INCX), I = 0 to N-1, where LX = 1 if INCX .GE. 0, else            
C     LX = (-INCX)*N, and similarly for DY using LY and INCY.                   
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DROT                                                          
C                                                                               
      DOUBLE PRECISION DX,DY,DC,DS,ZERO,ONE,W,Z                                 
      DIMENSION DX(1),DY(1)                                                     
      DATA ZERO,ONE/0.D0,1.D0/                                                  
C***FIRST EXECUTABLE STATEMENT  DROT                                            
      IF(N .LE. 0 .OR. (DS .EQ. ZERO .AND. DC .EQ. ONE)) GO TO 40               
      IF(.NOT. (INCX .EQ. INCY .AND. INCX .GT. 0)) GO TO 20                     
C                                                                               
           NSTEPS=INCX*N                                                        
           DO 10 I=1,NSTEPS,INCX                                                
                W=DX(I)                                                         
                Z=DY(I)                                                         
                DX(I)=DC*W+DS*Z                                                 
                DY(I)=-DS*W+DC*Z                                                
   10           CONTINUE                                                        
           GO TO 40                                                             
C                                                                               
   20 CONTINUE                                                                  
           KX=1                                                                 
           KY=1                                                                 
C                                                                               
           IF(INCX .LT. 0) KX=1-(N-1)*INCX                                      
           IF(INCY .LT. 0) KY=1-(N-1)*INCY                                      
C                                                                               
           DO 30 I=1,N                                                          
                W=DX(KX)                                                        
                Z=DY(KY)                                                        
                DX(KX)=DC*W+DS*Z                                                
                DY(KY)=-DS*W+DC*Z                                               
                KX=KX+INCX                                                      
                KY=KY+INCY                                                      
   30           CONTINUE                                                        
   40 CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END                                                                       

      SUBROUTINE DROTG(DA,DB,DC,DS)                                             
C***BEGIN PROLOGUE  DROTG                                                       
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1B10                                                         
C***KEYWORDS  BLAS,GIVENS ROTATION,LINEAR ALGEBRA,VECTOR                        
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  Construct d.p. plane Givens rotation                               
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C       DA  double precision scalar                                             
C       DB  double precision scalar                                             
C                                                                               
C     --Output--                                                                
C       DA  double precision result R                                           
C       DB  double precision result Z                                           
C       DC  double precision result                                             
C       DS  double precision result                                             
C                                                                               
C     Designed by C. L. Lawson, JPL, 1977 Sept 08                               
C                                                                               
C                                                                               
C     Construct the Givens transformation                                       
C                                                                               
C         ( DC  DS )                                                            
C     G = (        ) ,    DC**2 + DS**2 = 1 ,                                   
C         (-DS  DC )                                                            
C                                                                               
C     which zeros the second entry of the 2-vector  (DA,DB)**T .                
C                                                                               
C     The quantity R = (+/-)DSQRT(DA**2 + DB**2) overwrites DA in               
C     storage.  The value of DB is overwritten by a value Z which               
C     allows DC and DS to be recovered by the following algorithm.              
C                                                                               
C           If Z=1  set  DC=0.D0  and  DS=1.D0                                  
C           If DABS(Z) .LT. 1  set  DC=DSQRT(1-Z**2)  and  DS=Z                 
C           If DABS(Z) .GT. 1  set  DC=1/Z  and  DS=DSQRT(1-DC**2)              
C                                                                               
C     Normally, the subprogram DROT(N,DX,INCX,DY,INCY,DC,DS) will               
C     next be called to apply the transformation to a 2 by N matrix.            
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DROTG                                                         
C                                                                               
      DOUBLE PRECISION  DA, DB, DC, DS, U, V, R                                 
C***FIRST EXECUTABLE STATEMENT  DROTG                                           
      IF (DABS(DA) .LE. DABS(DB)) GO TO 10                                      
C                                                                               
C *** HERE DABS(DA) .GT. DABS(DB) ***                                           
C                                                                               
      U = DA + DA                                                               
      V = DB / U                                                                
C                                                                               
C     NOTE THAT U AND R HAVE THE SIGN OF DA                                     
C                                                                               
      R = DSQRT(.25D0 + V**2) * U                                               
C                                                                               
C     NOTE THAT DC IS POSITIVE                                                  
C                                                                               
      DC = DA / R                                                               
      DS = V * (DC + DC)                                                        
      DB = DS                                                                   
      DA = R                                                                    
      RETURN                                                                    
C                                                                               
C *** HERE DABS(DA) .LE. DABS(DB) ***                                           
C                                                                               
   10 IF (DB .EQ. 0.D0) GO TO 20                                                
      U = DB + DB                                                               
      V = DA / U                                                                
C                                                                               
C     NOTE THAT U AND R HAVE THE SIGN OF DB                                     
C     (R IS IMMEDIATELY STORED IN DA)                                           
C                                                                               
      DA = DSQRT(.25D0 + V**2) * U                                              
C                                                                               
C     NOTE THAT DS IS POSITIVE                                                  
C                                                                               
      DS = DB / DA                                                              
      DC = V * (DS + DS)                                                        
      IF (DC .EQ. 0.D0) GO TO 15                                                
      DB = 1.D0 / DC                                                            
      RETURN                                                                    
   15 DB = 1.D0                                                                 
      RETURN                                                                    
C                                                                               
C *** HERE DA = DB = 0.D0 ***                                                   
C                                                                               
   20 DC = 1.D0                                                                 
      DS = 0.D0                                                                 
      RETURN                                                                    
C                                                                               
      END                                                                       

      SUBROUTINE DSCAL(N,DA,DX,INCX)                                            
C***BEGIN PROLOGUE  DSCAL                                                       
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A6                                                          
C***KEYWORDS  BLAS,LINEAR ALGEBRA,SCALE,VECTOR                                  
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  D.P. vector scale x = a*x                                          
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DA  double precision scale factor                                       
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C                                                                               
C     --Output--                                                                
C       DX  double precision result (unchanged if N.LE.0)                       
C                                                                               
C     Replace double precision DX by double precision DA*DX.                    
C     For I = 0 to N-1, replace DX(1+I*INCX) with  DA * DX(1+I*INCX)            
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DSCAL                                                         
C                                                                               
      DOUBLE PRECISION DA,DX(1)                                                 
C***FIRST EXECUTABLE STATEMENT  DSCAL                                           
      IF(N.LE.0)RETURN                                                          
      IF(INCX.EQ.1)GOTO 20                                                      
C                                                                               
C        CODE FOR INCREMENTS NOT EQUAL TO 1.                                    
C                                                                               
      NS = N*INCX                                                               
          DO 10 I = 1,NS,INCX                                                   
          DX(I) = DA*DX(I)                                                      
   10     CONTINUE                                                              
      RETURN                                                                    
C                                                                               
C        CODE FOR INCREMENTS EQUAL TO 1.                                        
C                                                                               
C                                                                               
C        CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 5.           
C                                                                               
   20 M = MOD(N,5)                                                              
      IF( M .EQ. 0 ) GO TO 40                                                   
      DO 30 I = 1,M                                                             
        DX(I) = DA*DX(I)                                                        
   30 CONTINUE                                                                  
      IF( N .LT. 5 ) RETURN                                                     
   40 MP1 = M + 1                                                               
      DO 50 I = MP1,N,5                                                         
        DX(I) = DA*DX(I)                                                        
        DX(I + 1) = DA*DX(I + 1)                                                
        DX(I + 2) = DA*DX(I + 2)                                                
        DX(I + 3) = DA*DX(I + 3)                                                
        DX(I + 4) = DA*DX(I + 4)                                                
   50 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       

      SUBROUTINE DSWAP(N,DX,INCX,DY,INCY)                                       
C***BEGIN PROLOGUE  DSWAP                                                       
C***DATE WRITTEN   791001   (YYMMDD)                                            
C***REVISION DATE  820801   (YYMMDD)                                            
C***CATEGORY NO.  D1A5                                                          
C***KEYWORDS  BLAS,DOUBLE PRECISION,INTERCHANGE,LINEAR ALGEBRA,VECTOR           
C***AUTHOR  LAWSON, C. L., (JPL)                                                
C           HANSON, R. J., (SNLA)                                               
C           KINCAID, D. R., (U. OF TEXAS)                                       
C           KROGH, F. T., (JPL)                                                 
C***PURPOSE  Interchange d.p. vectors                                           
C***DESCRIPTION                                                                 
C                                                                               
C                B L A S  Subprogram                                            
C    Description of Parameters                                                  
C                                                                               
C     --Input--                                                                 
C        N  number of elements in input vector(s)                               
C       DX  double precision vector with N elements                             
C     INCX  storage spacing between elements of DX                              
C       DY  double precision vector with N elements                             
C     INCY  storage spacing between elements of DY                              
C                                                                               
C     --Output--                                                                
C       DX  input vector DY (unchanged if N .LE. 0)                             
C       DY  input vector DX (unchanged if N .LE. 0)                             
C                                                                               
C     Interchange double precision DX and double precision DY.                  
C     For I = 0 to N-1, interchange  DX(LX+I*INCX) and DY(LY+I*INCY),           
C     where LX = 1 if INCX .GE. 0, else LX = (-INCX)*N, and LY is               
C     defined in a similar way using INCY.                                      
C***REFERENCES  LAWSON C.L., HANSON R.J., KINCAID D.R., KROGH F.T.,             
C                 *BASIC LINEAR ALGEBRA SUBPROGRAMS FOR FORTRAN USAGE*,         
C                 ALGORITHM NO. 539, TRANSACTIONS ON MATHEMATICAL               
C                 SOFTWARE, VOLUME 5, NUMBER 3, SEPTEMBER 1979, 308-323         
C***ROUTINES CALLED  (NONE)                                                     
C***END PROLOGUE  DSWAP                                                         
C                                                                               
      DOUBLE PRECISION DX(1),DY(1),DTEMP1,DTEMP2,DTEMP3                         
C***FIRST EXECUTABLE STATEMENT  DSWAP                                           
      IF(N.LE.0)RETURN                                                          
      IF(INCX.EQ.INCY) IF(INCX-1) 5,20,60                                       
    5 CONTINUE                                                                  
C                                                                               
C       CODE FOR UNEQUAL OR NONPOSITIVE INCREMENTS.                             
C                                                                               
      IX = 1                                                                    
      IY = 1                                                                    
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1                                         
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1                                         
      DO 10 I = 1,N                                                             
        DTEMP1 = DX(IX)                                                         
        DX(IX) = DY(IY)                                                         
        DY(IY) = DTEMP1                                                         
        IX = IX + INCX                                                          
        IY = IY + INCY                                                          
   10 CONTINUE                                                                  
      RETURN                                                                    
C                                                                               
C       CODE FOR BOTH INCREMENTS EQUAL TO 1                                     
C                                                                               
C                                                                               
C       CLEAN-UP LOOP SO REMAINING VECTOR LENGTH IS A MULTIPLE OF 3.            
C                                                                               
   20 M = MOD(N,3)                                                              
      IF( M .EQ. 0 ) GO TO 40                                                   
      DO 30 I = 1,M                                                             
        DTEMP1 = DX(I)                                                          
        DX(I) = DY(I)                                                           
        DY(I) = DTEMP1                                                          
   30 CONTINUE                                                                  
      IF( N .LT. 3 ) RETURN                                                     
   40 MP1 = M + 1                                                               
      DO 50 I = MP1,N,3                                                         
        DTEMP1 = DX(I)                                                          
        DTEMP2 = DX(I+1)                                                        
        DTEMP3 = DX(I+2)                                                        
        DX(I) = DY(I)                                                           
        DX(I+1) = DY(I+1)                                                       
        DX(I+2) = DY(I+2)                                                       
        DY(I) = DTEMP1                                                          
        DY(I+1) = DTEMP2                                                        
        DY(I+2) = DTEMP3                                                        
   50 CONTINUE                                                                  
      RETURN                                                                    
   60 CONTINUE                                                                  
C                                                                               
C     CODE FOR EQUAL, POSITIVE, NONUNIT INCREMENTS.                             
C                                                                               
      NS = N*INCX                                                               
        DO 70 I=1,NS,INCX                                                       
        DTEMP1 = DX(I)                                                          
        DX(I) = DY(I)                                                           
        DY(I) = DTEMP1                                                          
   70   CONTINUE                                                                
      RETURN                                                                    
      END                                                                       
