C   SKEEL DISCRETISATION MODULE
C   ***************************
C   THIS MODULE DISCRETISES MIXED SYSTEMS OF PARTIAL DIFFERENTIAL
C   EQUATIONS IN ONE SPACE VARIABLE . THE CLASS OF EQUATIONS THAT CAN BE
C   HANDLED IS GIVEN BY
C
C    NPDE                     .   .                        .
C   SUM  C   (X,T, U, U  , V, V ) U  + F (X,T, U, U  , V , V )
C  J =1   I,J      -  - X  -  -    J      I      -  - X  -   -
C
C                                    -M   M
C                                =  X   (X  R (X,T, U, U  , V ))
C                                            I      -  -X   -   X
C     WHERE                     T
C           U = ( U  , ... , U )   ,  I = 1,... , NPDE.
C           -       1         NPDE
C         .
C     AND U  IS THE PARTIAL DERIV W.R.T. TIME OF THE JTH COMPONENT OF U.
C          J
C
C     THE P.D.E. FLUX FUNCTION R(.......) IS ASSUMED TO BE CONTINUOUS
C     W.R.T. THE SPACE VARIABLE R BUT THE FUNCTIONS F(.... ) AND C ARE
C     ALLOWED TO BE ONLY PIECEWISE CONTINUOUS PROVIDED THAT THE
C     DISCONTINUITIES ARE PLACED AT SPATIAL MESH POINTS. IN THIS CASE
C     THE REMESH OPTION MUST NOT BE USED .
C     THE OTHER VECTORS ARE DEFINED IN THE SAME WAY AS U EXCEPT THAT
C                                T     .               -
C           V = ( V  , ... , V )   AND V  IS SIMILARLY DEFD.
C           -      1          NV       -
C
C                 .
C     WHERE V AND V    ARE THE SOLUTION OF A COUPLED SYSTEM OF
C           -     -            ORDINARY  DIFFERENTIAL EQUATIONS OF
C                              DIMENSION NV.
C     IN THE CASE WHEN NV > 0 THIS SYSTEM OF DIFFERENTIAL EQUATIONS IS
C     ASSUMED TO HAVE THE FORM.
C                .
C        FV ( V, V , XI, UI, UI , RI , UI , UI  , ) = 0
C        --   -  -   -   -   - X  -    - T  - XT
C
C                                             .
C     WHERE ALL THE VECTORS APART FROM V  AND V  ARE OF LENGTH NPDE*NXI.
C                                      -      -
C     AND CONTAIN THE VALUES OF THE P.D.E. VARIABLES U , U  , R, U ,U
C                                                         X       T  XT
C     AT THE SPATIAL O.D.E. /P.D.E. COUPLING POINTS DEINED BY
C     THE VECTOR XI(NXI) .
C
C   THE SPATIAL MESH IS BOUNDED BY A < X < B.
C   THE BOUNDARY CONDITIONS HAVE THE FORM
C                                                      .
C          B (T) R(X,T, U, U , V )  = G (T, U , U , V, V)  AT X = A
C           A    -      -  -X  -      -A    -   -X  -  -
C  AND
C                                                     .
C          B (T) R(X,T, U, U , V)  = G (T, U , U , V, V)  AT X = B.
C           B    -      -  -X  -     -B    -   -X  -  -
C
C  WHERE   NOT ALL OF THE FUNCTIONS B AND G ARE SET TO ZERO.
C
C  THE INITIAL CONDITIONS ARE GIVEN BY
C          U (X, 0)  = K (X)  AND V(O) = K
C          -           -          -      -
C  THE DISCRETISATION METHOD USED BY THIS MODULE IS BASED ON A LUMPED
C  GALERKIN METHOD AND EVALUATES THE P.D.E. FUNCTIONS MIDWAY BETWEEN
C  THE USER SUPPLIED MESH POINTS . ANY DISCONTINUITIES IN THE P.D.E.
C  DEFINING FUNCTIONS C AND Q MUST THEREFORE BE AT THE USER SUPPLIED
C  MESH POINTS.
C  REFERENCE SKEEL R. D. AND BERZINS M . DEPT OF COMPUTER STUDIES
C                  THE UNIVERSITY , LEEDS REPORT NO 217 .
C
C----------------------------------------------------------------------
C
C   HOW TO USE THIS MODULE
C   **********************
C (1) SET NPDE = NUMBER OF P.D.E.S TO BE SOLVED
C     SET NPTS = NUMBER OF MESH POINTS TO BE USED
C     DEFINE A MESH OF INCREASING MESH POINTS IN THE ARRAY X(NPTS).
C       (USING THE REMESH OPTION A START MESH IS CONSTRUCTED BY INITSK)
C     SET M FOR SPACE CO-ORDINATE TYPE
C         = 0 FOR CARTESIAN,  = 1 FOR CYLINDRICAL, = 2 FOR SPHERICAL
C     SET NV = NUMBER OF O.D.E.S COUPLED TO THE P.D.E.S
C     SET NXI = NUMBER OF SPACE POINTS WHERE THE O.D.E.S ARE COUPLED
C     SET XI(NXI) TO THE VALUES OF THE COUPLING POINTS
C     SET REMESH = TRUE IF REMESHING IS REQUIRED (ELSE FALSE) - SEE (3)
C     SET MAXNPT = MAX. NO. OF MESH POINTS TO BE USED
C         (MAXNPT=NPTS IF REMESH IS FALSE)
C     SPECIFY A WORKSPACE OF SIZE AT LEAST  NPDE*(NPTS*2 + NXI*6 + 17 )
C                             +NXI+NV+ 2*NPDE(MAXNPT-NPTS) + 4*MAXNPT
C     FOR USE BY THE ROUTINE RES WHICH DEFINES THE O.D.E. SYSTEM BEING
C     SOLVED BY THE INTEGRATOR.
C
C     CALL THE INITIALISATION ROUTINE INITSK, SEE THE DOCUMENTATION AT
C     THE HEAD OF THIS ROUTINE FOR THE PRECISE DETAILS OF THE CALL.
C     SET TS AND TOUT FOR START AND END INTEGRATION TIMES
C     SET INFORM, WORK ARRAYS AS REQUIRED FOR TIME INTEGRATION
C                - SEE SPRINT DOCUMENTATION
C     CALL THE SPRINT ROUTINE WITH THE RESIDUAL ROUTINE SKLRES
C
C (2) PROVIDE A SET OF ROUTINES WHICH DESCRIBE THE PRECISE FORM OF THE
C     P.D.E. TO BE SOLVED. FIVE ROUTINES MUST BE PROVIDED AND THE NAMES
C     OF THESE ROUTINES ARE FIXED. THESE ROUTINES ARE:
C
C       SPDEF1 : FORMS THE FUNCTIONS C,F AND R IN THE P.D.E. DESCRIBED
C                ABOVE. THIS ROUTINE FORMS THE VALUES OF THE FUNCTIONS
C                C , F AND R OVER SEVERAL MESHPOINTS SIMULTANEOUSLY.
C                IN FACT AT THE X(NPTL) POINTS. THESE POINTS ARE
C                GENERATED BY THE DISCRETISATION ROUTINE AND ARE MIDWAY
C                BETWEEN THE USERS MESHPOINTS.
C        BNDR  : FORMS THE FUNCTIONS B AND G ASSOCIATED WITH THE
C                BOUNDARY CONDITIONS FOR THE P.D.E. ABOVE.
C       UINIT  : SUPPLIES THE INITIAL VALUES OF THE P.D.E. PART.
C       SVINIT : SUPPLIES THE INITIAL VALUES OF THE O.D.E. PART.
C       SODEFN : SUPPLIES THE ODE RESIDUAL AS DEFINED BY THE FUNCTION
C                FV ABOVE.
C           NOTE: THE P.D.E. SOLUTION VALUES AT THE COUPLING POINTS
C                 PASSED INTO SODEFN ARE DEFINED BY LINEAR INTERPOLATION
C                 ON THE VALUES AT THE P.D.E. SPATIAL MESH POINTS.
C                 THIS MAY CAUSE A PROBLEM WHEN REMESHING IS USED AND IF
C                 THE P.D.E. MESH POINTS DRIFT AWAY FROM THE P.D.E/
C                 O.D.E. COUPLING POINTS.
C
C (3) REMESHING PARAMETERS:
C -------------------------
C     THE REMESHING ALGORITHM USED IS DESCRIBED BY
C     FURZELAND R.M. THE CONSTRUCTION OF ADAPTIVE SPACE MESHES FOR THE
C                    SOLUTION OF ORDINARY AND PARTIAL DIFFERENTIAL EQNS.
C                    REPT. TNER 85.022, SHELL RESEARCH, PO BOX 1,CHESTER
C
C     NOTE : REMESHING SHOULD NOT BE USED IF ANY OF THE FUNCTIONS
C            C OR F ARE ONLY PIECEWISE CONTINUOUS OR IF A COUPLED
C            O.D.E./P.D.E. PROBLEM IS SOLVED.
C     DEFINE A MESH OF INCREASING MESH POINTS IN THE ARRAY X(NPTS).
C      (THIS MESH IS USED BY INITSK AT TIME TS TO CONSTRUCT AN ADAPTIVE
C       MESH ACCORDING TO THE FORM OF THE REMESH MONITOR FUNCTION EVNRMF
C      SET UP THE FOLLOWING COMMON BLOCKS
C     COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOP
C CONST - INPUT BOUND ON INTEGRAL OF MONITOR OVER EACH STEP. SIZE
C         DEPENDS ON NPTS AND MONITOR (TYPICAL VALUE 5.0D-2).
C         CONTROLS THE LOCAL TRUNCATION ERROR IF THE MONITOR IS CHOSEN
C         ACCORDINGLY
C STPRAT - INPUT BOUND ON ADJACENT MESH RATIO (>1 TYPICALLY 1.5-3)
C          THIS ALSO CONTROLS LOCAL TRUNCATION ERRORS.
C
C IPMINF - CONTROLS PRINT OUT OF MESH INFORMATION. IPMINF=0 NO PRINT,
C          =1 SUMMARY PRINT OF MESH CHARACTERISITICS E.G. MESH RATIOS &
C          MAX. INTEGRAL OF MONITOR PER INTERVAL (APPROX.=INPUT CONST)
C          =2 PRINT OF MONITORS, OLD & NEW MESHES AND MESH SIZES
C SET NOP=0 IF VARIABLE NO. OUTPUT PTS REQUIRED ELSE SET NOP=NPTS
C
C COMMON MESHTI USED FOR REMESHING IN TIME INFO
C     COMMON /MESHTI/ TOUT, REMESH, NRMESH, ICOUNT
C
C    SET TOUT - SEE BELOW
C    SET REMESH TRUE OR FALSE DEPENDING ON WHETHER REMESHING IS REQUIRED
C      (NOTE REMESH MAY CHANGE FROM TRUE TO FALSE DURING COURSE OF TIME)
C    SET NRMESH TO DICTATE HOW OFTEN TO REMESH
C     =1 THEN REMESHES EVERY TIME STEP THE SPRINT MONITOR CALLED
C     =2        "        "   2ND TIME STEP, ETC....
C        (THE SIZE OF WORK(6)=HMAX SHOULD BE SET APPROPRIATELY IN
C         THE CALL TO SPRINT)
C     =0 THEN REMESHES AT CURRENT TIME TOUT BY THE CALL TO MONITOR WITH
C        INFORM(2)=4 AND WORK(1) SET TO CURRENT TOUT. IF THE SOLUTION
C        PROFILE CHANGES RAPIDLY FOR CERTAIN T THEN SET TOUT ACCORDINGLY
C
C ICOUNT USED TO COUNT UP TO NRMESH SO JUST INITIALISE TO ZERO
C     ICOUNT=0
C
C THE USER MUST ALSO SET THE FORM OF THE REMESH MONITOR FUNCTION.
C     WITHIN THE CONSTRAINTS DICTATED BY CONST AND STPRAT THIS MONITOR
C     IS USED TO CHOOSE A MESH WHICH EQUIDISTRIBUTES THE INTEGRAL OF
C     THE MONITOR AND SHOULD THUS RESEMBLE THE SPACE DISCRETISATION
C     ERROR TERM.
C     THE MONITOR SHOULD BE NON-NEGATIVE AND NOT EVERYWHERE ZERO.
C
C
C     SUBROUTINE EVNRMF( TIME, NIP, NPDE, X, U, R, MAXNPT, FMON)
C  EXAMPLE REMESH MONITOR FUNCTION ROUTINE FOR REMESHING BASED ON THE
C  DERIVATIVE OF THE FLUX FUNCTION R (DEFINED IN SPDEF1)
C  E.G. THE 2ND DERIVATIVE OF U FOR THE HEAT EQN.
C     R(J,I) CONTAINS THE FLUX FOR JTH P.D.E. AT POINT Y(I)
C        WHERE Y(1) = X(1) , Y(NIP+1) = X(NIP)
C              Y(I) =(X(I+1) - X(I)) * 0.5   I = 2,..., NIP
C     THE MONITOR FUNCTION IS RETURNED IN THE ARRAY FMON
C
C     INTEGER NIP, NPDE, MAXNPT, I, J, K, L
C     DOUBLE PRECISION U(NPDE,1), X(NIP), SUM,FMON(NIP), TIME,
C    1                 R(NPDE,1),  DRDX, H
C     DO 10 I=1,NIP
C       K = MAX0(1,I-1)
C       L = MIN0(NIP,I+1)
C       H = (X(L) - X(K)) * 0.5D0
C       SUM=0.0D0
C       DO 5 J=1,NPDE
C 2ND DERIV. MONITOR FUNCTION
C          DRDX = (R(J,I+1) - R(J,I)) / H
C   5      SUM = SUM + DRDX**2
C       FMON(I)=DSQRT(SUM)
C  10 CONTINUE
C     RETURN
C     END
C
C
C
C EXAMPLE PROBLEM ONE
C *******************
C     THE EASIEST WAY TO DESCRIBE HOW THE PROBLEM DESCRIPTION ROUTINES
C     SHOULD BE WRITTEN IS BY A SIMPLE EXAMPLE. CONSIDER THE FLAME
C     PROPAGATION PROBLEM DEFINED BY
C       Y  = ( Y  )  - Y * KT
C        T      X  X
C  AND                            (NPDE IS 2 )
C       W  = ( W  )  + Y * KT
C        T      X  X
C                      WHERE KT = 14.6* EXP( 4.0*(1.0 - 1./(Y + 0.01)))
C  AND   WHERE 0 =< X =< 10.0. AND  0=< T =< 2.8.
C
C     THE LEFT BOUNDARY CONDITION AT X = 0 (LEFT = .TRUE. ) IS GIVEN BY
C         W  = -T / 0.05 , 0 < T =< 0.05
C          X
C            = -COS( PI*0.5 *(T- 0.05) / 0.95)  0.05 =< T < 1.0
C            =  0   FOR T >= 1.0
C      AND
C         Y  = 0
C          X
C     THE RIGHT BOUNDARY CONDITION IS  (LEFT = .FALSE.)
C         W  = Y = 0 FOR ALL TIME T
C          X    X
C      THE ROUTINES SPDEF1 AND BNDR ARE LISTED IN TURN BELOW.
C      THE COMPONENT W OF THE P.D.E. AT THE ITH MESHPOINT IS HELD AS
C      U(1,I) IN THE PACKAGE AND THE COMPONENT Y IS HELD AS U(2,I)
C      THE INITIAL CONDITIONS ARE
C        W(I) = U(1,I) = 1.0D0  I = 1,.., NPTS
C        Y(I) = U(2,I) = 0.0D0  I = 1,...,NPTS
C
C**********************************************************************
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS)
C      DO 10 I = 1,NPTS
C         U(1,I) = 1.0D0
C10       U(2,I) = 0.0D0
C      RETURN
C      END
C
C      SUBROUTINE SPDEF1(T,X,NPDE, U, DUDX, C, F, R, NV, V, VD,IR)
C      ROUTINE TO DESCRIBE THE BODY OF THE P.D.E. SYSTEM.
C      THE P.D.E. IS WRITEN AS               -M   M
C        C (X,T,U, U  ) U  + F (X,T,U,U ) = X   (X  R (X,T,U,U ) )
C         I         X    T    I        X             I        X   X
C      THE FUNCTIONS C, F AND R MUST BE DEFINED IN THIS ROUTINE.
C      DEFINITIONS FOR THE MODEL PROBLEM ARE GIVEN
C      NOTE NV = 0 : THERE IS NO O.D.E PART.
C      INTEGER NPDE, NPTL, I, J, NV, IR
C      DOUBLE PRECISION T, KT, X, U(NPDE),DUDX(NPDE),
C    1   UDOT(NPDE), C(NPDE,NPDE), R(NPDE), V, VD
C         DO 9 J = 1,NPDE
C9           R(J) = DUDX(J)
C         KT = 14.6D0 * DEXP( 4.0D0 - 4.0D0/(U(2) + 0.01D0))
C         F(1) =   KT * U(1)
C         F(2) = - KT * U(1)
C         C(1,1) = 1.0D0
C         C(2,2) = 1.0D0
C         C(1,2) = 0.0D0
C         C(2,1) = 0.0D0
C10    CONTINUE
C      RETURN
C      END
C
C      SUBROUTINE BNDR( T, BETA, GAMMA, U, UX, NPDE, LEFT, NV, V, VD,IR)
C      BOUNDARY CONDITIONS ROUTINE
C      INTEGER NPDE, I, J, NV
C      DOUBLE PRECISION T, BETA(NPDE), GAMMA(NPDE), U(NPDE), HPI
C     1                 UX(NPDE), V, VD
C      LOGICAL LEFT
C      HPI = 3.14159265350D0 * 0.5D0 / 0.95D0
C      DO 10 I = 1,NPDE
C         BETA(I) = 1.0D0
C         GAMMA(I) = 0.0D0
C10    CONTINUE
C      IF(LEFT) THEN
C         IF(T .LE. 0.05)THEN
C            GAMMA(2) = -T/0.05D0
C         ELSE IF(T .LE. 1.0)THEN
C            GAMMA(2) = -DCOS(HPI*(T-0.05D0))
C         END IF
C      END IF
C      RETURN
C      END
C
C      DUMMY O.D.E. ROUTINES AS NV IS ZERO
C      SUBROUTINE SODEFN
C      RETURN
C      END
C      SUBROUTINE SVINIT
C      RETURN
C      END
C
C**********************************************************************
C EXAMPLE  PROBLEM 2
C SOLUTION OF MOVING BOUNDARY  PROBLEM BY CO-ORDINATE TRANSFORMATION.
C********************************************************************
C  THIS PROBLEM IS THE ONE PHASE STEFAN PROBLEM (HOFFMAN (1977) ) SEE
C  FURZELAND R.M. A COMPARATIVE STUDY OF NUMERICAL METHODS FOR MOVING
C                 BOUNDARY PROBLEMS. J.I.M.A. (1977) ,26, PP 411 - 429.
C  THE PROBLEM HAS  MELTING DUE TO HEAT INPUT AT THE FIXED
C  BOUNDARY . THE P.D.E. IS DEFINED BY THE EQUATIONS
C         U  =  U        0 < Y < S(T) , 0.1 < T < 1
C          T     YY
C            U  = - EXP(T) , Y = 0
C             Y            .
C            U  =  0  AND  S(T) = - U   ON THE MOVING BOUNDARY Y = S(T).
C                                    Y
C  AND THE INITIAL SOLUTION VALUES AT T = 0.1 ARE GIVEN BY THE ANALYTIC
C  SOLUTION
C            U = EXP(T-Y) - 1 , S(T) = T.
C  THE PROBLEM IS REWRITTEN BY USING THE CO-ORDINATE TRANSFORMATION
C  GIVEN BY  X(T)  =  Y / S(T)  . THE EQUATIONS THEN READ
C                      .
C     S * S * U  - S * S  * X * U   =  U     , X IN (0,1).
C              T                 X      XX
C  WITH THE NEUMANN TYPE BOUNDARY CONDITIONS
C                                               .
C     U  = - EXP(T)  AT X=0  AND  U  = - S(T) * S(T) AT X = 1
C      X                           X
C  AND THE O.D.E. COUPLING POINT EQUATION AT X = 1 WHICH IMPLICITLY
C  DEFINES S(T) IS GIVEN  BY
C     U(1,T) = 0
C  THE EXACT SOLUTION OF THE TRANSFORMED PROBLEM IS DEFINED BY
C     U(X,T) = EXP((T - X*S(T))  , S(T) = T
C
C WE SHALL NOW DETAIL THE ROUTINES NEEDED TO DESCRIBE THIS PROBLEM.
C        PROBLEM DESCRIPTION ROUTINES
C       ******************************
C EXACT SOLUTION
C      SUBROUTINE EXACT( TIME, NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. EXACT VALUES  (IF KNOWN)
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME, P
C      P=DSQRT(2.0D0)*0.5D0
C      DO 10 I = 1,NPTS
C10       U(1,I) = DEXP( TIME * (1 - X(I))) - 1.0D0
C      RETURN
C      END
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME
C      TIME=0.1D0
C      CALL EXACT(TIME,NPDE,NPTS,X,U)
C      RETURN
C      END
C
C      SUBROUTINE SPDEF1(T, X, NPDE, U, DUDX, C,F,R,NV,V,VD, IR)
C      PROBLEM INTERFACE  FOR THE MOVING BOUNDARY PROBLEM.
C      INTEGER NPTL, NPDE, NV, I, IR
C      DOUBLE PRECISION X, U(NPDE), DUDX(NPDE), T,
C    1    V(1), VD(1), F(NPDE), C(NPDE,NPDE) ,R(NPDE)
C         C(1,1) = V(1)*V(1)
C         R(1) = DUDX(1)
C         F(1) = -X*VD(I)*DUDX(1) * V(1)
C      RETURN
C      END
C      SUBROUTINE BNDR( T, BETA, GAMMA, U, UX, NPDE, LEFT, NV, V, VD,IR)
C  THIS SUBROUTINE PROVIDES THE LEFT AND RIGHT BOUNDARY VALUES
C  FOR THE MOVING BOUNDARY PROBLEM IN THE FORM.
C           BETA(I) * DU/DX(I) = GAMMA(I)
C  WHERE I = 1,NPDE AND GAMMA IS A FUNCTION OF U,X AND T
C
C      INTEGER NPDE, NV, IR
C      LOGICAL LEFT
C      DOUBLE PRECISION BETA(NPDE), GAMMA(NPDE), U(NPDE), UX(NPDE)
C    -                  ,T, V(1), VD(1)
C      BETA(1) = 1.0D0
C      IF(LEFT)THEN
C         GAMMA(1) = -V(1)*DEXP(T)
C      ELSE
C         GAMMA(1) = -V(1)*VD(1)
C      END IF
C      RETURN
C      END
C
C      SUBROUTINE SODEFN(T, NV, V, VDOT, NPDE, NXI, XI, UI, UXI, RI,
C    1                    UTI, UTXI, VRES, IRES)
C      ROUTINE TO PROVIDE RESIDUAL OF COUPLED ODE SYSTEM FOR THE
C      MOVING BOUNDARY PROBLEM.
C      NOTE HOW IRES CAN BE RESET TO COPE WIH ILLEGAL VALUES OF THE
C           MOVING BOUNDARY POSITION V(1).
C      INTEGER NPDE, NXI, NV, IRES
C      DOUBLE PRECISION T, XI(NXI), UI(NPDE,NXI), UXI(NPDE,NXI),
C    1         RI(NPDE,NXI), UTI(NPDE,NXI), UTXI(NPDE,NXI), VRES(NV),
C    2         V(NV), VDOT(NV), TEM
C      VRES(1) = UI(1,1)
C      TEM = 1.0D0
C      IF(IRES .EQ. -1)TEM = 0.0D0
C      IF(V(1) .LT. 0.0D0)IRES = 3
C      VRES(1) = TEM * UI(1,1)
C      RETURN
C      END
C
C      SUBROUTINE SVINIT( V,NV)
C      INTEGER NV
C      DOUBLE PRECISION V(NV)
C      V(1) = 0.1D0
C      ROUTINE TO PROVIDE THE INITIAL VALUES OF COUPLED ODES
C      RETURN
C      END
C
C***********************************************************************
C
        SUBROUTINE INITSK( NEQN, NPDE, NPTS, X, U, WK, IWK, M, TS,
     1                     IBAND, ITIME, REMESH, MAXNPT, NV, NXI, XI)
C***********************************************************************
C
C  OLD SETUP ROUTINE FOR SKEEL DISCRETISATION, SAME PARAMATER LIST.
C
C***********************************************************************
        INTEGER I, NPTS, NPDE, IBAND, M, ITIME, IWK, MAXNPT
        INTEGER NV, NXI, NVST, NEQN, NPMAX, NXFIX
        DOUBLE PRECISION WK(IWK), X(NPTS), TS, U(1), XI(1), XFIX
        LOGICAL REMESH
        NXFIX = 0
        CALL SETSKL( NEQN, NPDE, NPTS, X, U, WK, IWK, M, TS, IBAND,
     1               ITIME, REMESH, MAXNPT, NV, NXI, XI, XFIX, NXFIX)
        RETURN
        END
        SUBROUTINE SETSKL( NEQN, NPDE, NPTS, X, U, WK, IWK, M, TS,
     1                     IBAND, ITIME, REMESH, MAXNPT, NV,
     2                     NXI, XI, XFIX, NXFIX)
C
C***********************************************************************
C       PARAMETER LIST
C       --------------
C
C       NEQN : EMPTY ON ENTRY , ON EXIT CONTAINS THE SIZE OF THE O.D.E.
C              SYSTEM GENERATED WHEN THE P.D.E. IS DISCRETISED.
C              THIS VALUE IS NPDE * NPTS + NV
C
C       NPDE : THE NUMBER OF P.D.E. S
C
C       NPTS ; THE NUMBER OF SPATIAL MESH POINTS IF THE REMESH OPTION IS
C              USED THIS NUMBER MAY HAVE BEEN CHANGED ON EXIT. AN UPPER
C              BOUND MAY BE SET BY USING THE PARAMETER NPTMAX.
C
C       X(NPTS) ARRAY CONTAININMG THE USER SUPPLIED MESH POINTS IF THE
C               REMESH OPTION IS USED IT WILL CONTAIN THE MESH POINTS
C               THAT HAVE BEEN CHOSEN BY THE REMESH ROUTINE.
C
C       U(NEQN) ARRAY THAT ON EXIT HAS THE INITIAL P.D.E. VALUES. THIS
C              ARRAY MUST BE PASSED ACROSS AS A ONE DIMENSIONAL ARRAY OF
C              LENGTH NEQN TO THE SPRINT PACKAGE. THIS ARRAY IS ORDERED
C              BY THIS ROUTINE AS
C               U(1)           - U(NPDE*NPTS)  ; PDE COMPONENTS
C               U(NPDE*NPTS+1) - U(NEQN)       ; ODE COMPONENTS
C
C       WK(IWK) WORKSPACE FOR THE RESIDUAL ROUTINE RES WHICH
C               ACTUALLY PERFORMS THE SEMI-DISCRETISATION.
C
C       M INTEGER >= 0 WHICH DETERMINES THE CO-ORDINATE SYSTEM USED.
C                    0  : CARTESIAN CO-ORDINATES
C                    1  : POLAR CO-ORDINATES
C                    2  : SPHERICAL POLAR CO-ORDINATES
C
C       TS-THE TIME AT WHICH INTEGRATION COMMENCES
C
C       IBAND - AN UPPER BOUND ON THE HALF BANDWIDTH OF THE JACOBIAN
C               MATRIX WHEN THIS MODULE IS USED.
C               = 2*NPDE + 1
C               THIS PARAMETER SHOULD BE SUPPLIED TO MATSET IF SPRINT
C               IS CALLED WITH BANDED MATRIX ROUTINES.
C               N.B. IN THE CASE WHEN MIXED O.D.E./P.D.E. PROBLEMS ARE
C                    SOLVED AND THE BODY OF THE P.D.E. EQUATION ( ONE OF
C                    THE FUNCTIONS C,F OR R IN THE PROBLEM SPEC ABOVE )
C                    DEPENDS ON THE O.D.E. VARIABLES , OR THE O.D.E.
C                    VARIABLES DEPEND ON NON BOUNDARY P.D.E. VALUES THE
C                    NORMAL BANDED STRUCTURE OF THE O.D.E. SYSTEM IS
C                    DESTROYED . WHEN SOLVING MIXED O.D.E. /P.D.E.
C                    PROBLEMS SPARSE OR FULL LINEAR ALGEBRA ROUTINES
C                    SHOULD BE USED IN SPRINT.
C                    IBAND IS SET TO NEQN IF NV > 0.
C       ITIME   SHOULD BE SET TO ONE ON THE FIRST CALL . ON THIS CALL
C               THE USER SUPPLIED MESH X(NPTS) IS PUT IN THE FIRST
C               NPTS LOCATIONS OF WORK. ON A SUBSEQUENT CALL THE MESH
C               USED , WHICH MAY HAVE BEEN CHANGED BY A MESH
C               MODIFICATION MONITOR ROUTINE, MAY BE PUT INTO X(NPTS)
C               CALLING INITSK WITH ITIME = 2.
C               THIS PARAMETER IS SET TO -1 IF AN ERROR IS FOUND BY
C               THIS ROUTINE.
C       REMESH  LOGICAL PARAMETER THAT SHOULD BE SET TO .TRUE. ON ENTRY
C               IF REMESHING IS REQUIRED. OTHERWISE IT SHOULD BE SET TO
C               .FALSE.
C       NV      THE NUMBER OF AUXILARY O.D.E.S THAT ARE COUPLED TO
C               THE P.D.E.
C       NXI     THE NUMBER OF COUPLING POINTS AT WHICH P.D.E. VALUES
C               ARE USED TO DEFINE THE O.D.E.S
C       XI(NXI) A VECTOR SPECIFYING THE POSITION OF THESE POINTS.
C
C       XFIX(NXFIX) DOUBLE PRECISION ARRAY SPECIFYING WHERE ANY MATERIAL
C               INTERFACES IN THE SPATIAL DOIMAIN ARE. IF NXFIX = 0
C               THEN IT IS ASSUMED THAT THERE ARE NO SUCH INTERFACES.
C               THIS ARRAY NEED NOT BE ASSIGNED IF REMESH = .FALSE.
C               ABOVE. OTHERWISE IT IS ASSUMED THAT THE USER HAS DEFINED
C               XFIX SO THAT
C                               X(1) < XFIX(I) < X(NPTS) , I = 1,NXFIX
C---------------------------------------------------------------------
C   SEVEN IMPORTANT PARAMETERS ARE PASSED ACROSS FROM HERE IN
C   COMMON /SPSKEL/  NNPDE, NNPTS, MM, NNV, NNXI, NVST, NPMAX
C
C  NNPDE = NPDE     THE NUMBER OF P.D.E.S
C
C  NNPTS = NPTS     THE TOTAL NUMBER OF MESHPOINTS = NEL*(NPTL-1) + 1 ,
C                   IF REMESH THEN THE NO. MAY CHANGE ON OUTPUT.
C
C  MM = M           = 0,1,2 CARTESIAN, CYLINDRICAL OR SPHERICAL POLAR
C                           CO-ORDINATES IN USE.
C
C  NNV = NV         NUMBER OF AUXILIARY ODES
C
C  NNXI= NXI        THE NUMBER OF COUPLING POINTS
C
C  NVST             THE STARTING POINT OF THE ODE COMPONENTS IN THE
C                   SOLUTION VECTOR.
C  NPMAX            MAXNPT FROM THE PARAMETER LIST OF THIS ROUTINE.
C
C----------------------------------------------------------------------
C
C     DETAILED DESCRIPTION OF WORKSPACE
C    -----------------------------------
C     THE WORKSPACE WK(IWK) IS USED TO PASS ARRAYS AND VECTORS TO
C     THE ROUTINES  SKLRES , INTERX AND MVMESH .
C
C     SIZE  :  IWK MUST BE >=   NPDE*(NPTS*2+NXI*6+15+2*NPDE) + NXI + NV
C                           ( + 2*NPDE(MAXNPT-NPTS) + 4*MAXNPT + 2*NXFIX
C                              IF THE REMESH OPTION IS EMPLOYED )
C
C     STRUCTURE        NAME IN CODE   PURPOSE IN CODE IS TO HOLD
C     *********
C     WK(1)  -  WK(I1-1)            ARRAY OF SIZE (13,NPDE) USED BY THE
C                                   SKEEL4 DISCRETISATION ROUTINE.
C     ---------------------THE FOLLOWING PARTS OF THE WORKSPACE ARE USED
C                          IN SEMI-DISCRETISING MIXED O.D.E./P.D.E.
C                          PROBLEMS.
C     WK(I1)-  WK(I2-1) UI       ARRAY UI(NPDE,NXI) USED TO HOLD THE
C                                PDE COMPONENTS AT THE COUPLING PTS .
C     WK(I2)-  WK(I3-1) UXI      SPACE DERIVS CORRESSPONDING TO UI.
C     WK(I3)-  WK(I4-1) RI       FLUX CORRESS TO UI ARRAY
C     WK(I4)-  WK(I5-1) UTI      TIME DERIV CORRESSPONDING  TO UI .
C     WK(I5)-  WK(I6-1) UTXI     SPACE DERIV OF ARRAY UTI.
C     WK(I6)-  WK(I7-1) XI       COUPLING POINTS TO LINK PDE TO ODE.
C     WK(I7)-  WK(I8-1) VDUM     ARRAY OF ZEROES.
C
C     --------------------THE FOLLOWING THREE PARTS OF WK HOLD THE MESH
C                         POINT INFORMATION USED IN SEMI-DISCRETISATION.
C     WK(I8) -  WK(I9-1)  X(NPTS)   SPATIAL MESH POINTS USED BY THE CODE
C
C     WK(I9) -  WK(I10-1) R(NPDE,NPTS+1) FLUX VALUES AT THE BOUNDARIES
C                         AND AT THE POINTS MIDWAY BETWEEN THE MESH
C                         POINTS. USED IN MESH MODIFICATION ONLY.
C
C     WK(I10) - WK(I14)   ARRAYS USED IN THE REMESHING PROCESS TO
C                         GENERATE THE NEW MESH.
C
C     WK(I14) - WK(I16)   ARRAYS USED IN APPLYING REMESHING TO PROBLEMS
C                         WITH MATERIAL INTERFACES.
C
C**********************************************************************
        INTEGER I, NPTS, NPDE, IBAND, MM, ITIME, IWK, J, JPMINF, NOPUSR,
     1          IDEV, ITRACE, M, NNPDE, NNPTS, MAXNPT, NNXFIX
        INTEGER I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I13
        INTEGER NV, NNV, NXI, NNXI, NVST, NEQN, NPMAX, I14, IDUM, NXFIX
        DOUBLE PRECISION WK(IWK), X(NPTS), TS, U(1), XI(1), TDUM,
     1           XFIX(1), TEMP, TEMP2, CONST, STPRAT
        CHARACTER*6 PDCODE
        LOGICAL REMESH , REMSH1
        COMMON /DISCHK/ PDCODE
        COMMON /SDEV2/  ITRACE,IDEV
        COMMON /SPSKWK/ I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11,
     1                  I12, I13, I14, I15, I16
        COMMON /SPSKEL/ NNPDE, NNPTS, MM, NNV, NNXI, NVST, NPMAX, NNXFIX
        COMMON /MESHTI/ TDUM, REMSH1, IDUM(2)
        COMMON /MESHIN/ CONST, STPRAT, JPMINF, NOPUSR
        SAVE   /SPSKEL/, /DISCHK/, /SPSKWK/, /MESHIN/
        IF(ITIME .GT. 1)THEN
C          UPDATE THE NUMBER AND POSITIONING OF THE MESH POINTS.
           NPTS = NNPTS
           DO 300 I = 1, NPTS
 300           X(I) = WK(I8 + I -1)
           RETURN
        ELSE
C        ZERO THE WORKSPACE TO PREVENT UNDEFINED VARIABLES BEING USED.
         DO 400 I = 1,IWK
 400       WK(I) = 0.0D0
        ENDIF
        MM = M
        NNPDE = NPDE
C IF ITIME>1, NPTS AND NEQN UPDATED IN MVMESH REMESHING MONITOR SUBR.
        NNPTS = NPTS
        NEQN = NPDE*NPTS + NV
        NPMAX = MAXNPT
        NNXFIX = NXFIX
        IF(NV .LT. 0 .OR. NXI .LT. 0)THEN
           CALL SERROR(' SETSKL - ERROR ILLEGAL VALUES OF NXI(=I1) ON
     1                   OR OF NV (=I2) ON ENTRY ', 1 , 2, NXI, NV, 0,
     2                   0.0D0, 0.0D0)
           ITIME = -1
           RETURN
        END IF
        I1 = (15+2*NPDE) * NPDE + 1
        I2 = I1 + NPDE * NXI
        I3 = I2 + NPDE * NXI
        I4 = I3 + NPDE * NXI
        I5 = I4 + NPDE * NXI
        I6 = I5 + NPDE * NXI * 2
        I7 = I6 + NXI
        I8 = I7 + NV
        I9 = I8 + NPTS
        I10 =I9 + NPDE*(NPTS+1)
        IF(REMESH)THEN
           I11 = I10 + MAXNPT
           I12 = I11 + MAXNPT
           I13 = I12 + MAXNPT
           I14 = I13 + MAXNPT
           I15 = I14 + NXFIX
           I16 = I15 + NXFIX
           I = I8 + NPDE + (NPDE + 5) * MAXNPT + 2 * NXFIX
        ELSE
           I11 = I10
           I12 = I11
           I13 = I12
           I14 = I13
           I   = I8 + NPDE + NPTS * (NPDE+1)
           REMSH1 = .FALSE.
        END IF
        IF(I .GT. IWK)THEN
           CALL SERROR(' SETSKL - ERROR REAL WORKSPACE OF SIZE (=I1) IS
     1                   SMALLER THAN REQUIRED (=I2)', 1, 2, IWK, I, 0,
     2                   0.0D0, 0.0D0)
           ITIME = -1
        END IF
        NNV = NV
        NNXI = NXI
        NVST = NPDE * NPTS + 1
        IF(NV .EQ. 0)NVST = NVST -1
        IF(NPTS .LE. 2 .OR. NPDE .LT. 1)THEN
          CALL SERROR(' SETSKL - ILLEGAL VALUES OF NPTS(=I1) OR NPDE (=I
     1    2)',1,2,NPTS,NPDE, 0, 0.0D0, 0.0D0)
          ITIME = -1
        END IF
        IF(  MAXNPT .LT. NPTS) THEN
           CALL SERROR(' SETSKL - ILLEGAL VALUE OF MAXNPT (=I1) .LT.
     1                   NPTS (=I2) ', 1, 2, MAXNPT, NPTS, 0, 0.0D0,
     2                   0.0D0 )
           ITIME = -1
        END IF
C
C        CHECK THE  MESH
C
        DO 9 I = 2,NPTS
           IF( X(I) .LE. X(I-1))THEN
             CALL SERROR(' SETSKL-ERROR MESHPOINT NO (=I1) VALUE (=R1)
     1 HAS A SMALLER VALUE THAN THE NEXT MESHPOINT TO THE LEFT
     2 VALUE (=R2)', 1, 1, I, 0, 2, X(I), X(I-1))
             ITIME = -1
           END IF
 9     CONTINUE
C
C      CHECK THE COUPLING POINTS IF ANY.
C
       DO 91 I = 1,NXI
         IF(XI(I) .LT. X(1) .OR. XI(I) .GT. X(NPTS))THEN
            CALL SERROR(' SETSKL ERROR-O.D.E./P.D.E. COUPLING POINT NO
     1                   (=I1) HAS VALUE (=R1) WHICH IS OUTSIDE THE PDE
     2                   SPATIAL MESH', 1, 1, I, 0, 1, XI(I), 0.0D0)
            ITIME = -1
         END IF
         WK(I6+I-1) = XI(I)
 91   CONTINUE
C
C      CHECK THE MATERIAL INTERFACE POINTS IF ANY.
C
       IF(REMESH .AND. NXFIX .GT. 0)THEN
         DO 92 I = 1,NXFIX
           IF(XFIX(I) .LE. X(1) .OR. XFIX(I) .GE. X(NPTS))THEN
              CALL SERROR(' SETSKL ERROR-MATERIAL INTERFACE  POINT NO
     1                   (=I1) HAS VALUE (=R1) WHICH IS OUTSIDE THE PDE
     2                   SPATIAL MESH', 1, 1, I, 0, 1, XFIX(I), 0.0D0)
               ITIME = -1
           END IF
 92      CONTINUE
       END IF
       IF( M .LT. 0)THEN
         CALL SERROR(' SETSKL- POLAR PARAMETER M (=I1) LESS THAN ZERO',
     1   1, 1, M, 0, 0, 0.0D0, 0.0D0)
         ITIME = -1
        END IF
        IF(ITIME .EQ. -1)RETURN
C
C         INITIALISE O.D.E. AND P.D.E. VARIABLES BY APPROPRIATE CALLS.
C
        IF(NV .GT. 0) CALL SVINIT( U(NVST), NV)
        CALL UINIT( NPDE, NPTS, X, U )
        DO 10 I = 1,NPTS
           WK(I8+I-1) = X(I)
10      CONTINUE
C
C    CALL TO THE ROUTINE TO ADJUST THE INITIAL MESH.
C
        NNXFIX  = NXFIX
        IF(REMESH)THEN
           DO 20 I = 1,NXFIX
 20           WK(I14+I-1) = XFIX(I)
           TEMP2 = MIN( 1.D0, 10.D0 / (NPTS-1.D0) )
           TEMP  = CONST * (NPTS-1)
           IF( TEMP .LE. 0.1D0 .OR. TEMP .GE. 10.D0)THEN
             CALL SERROR(' INITSK- ILLEGAL VALUE OF CONST HAS BEEN
     1       DETECTED - CONST SHOULD LIE BETWEEN 0.1 / (NPTS-1) AND
     2       10 / (NPTS-1) ',  1, 0, 0, 0, 0, 0.0D0, 0.0D0)
             ITIME = -1
             RETURN
           END IF
C
           I = 1
           J = 1
           CALL INMESH(TS,U,NPDE,NNPTS,WK,IWK,U(NVST), NV, NPMAX, I,
     1                 WK(I14), WK(I15), NNXFIX, J)
           IF(I.NE.1 .OR. J .LT. 0)THEN
             IF(I.NE.1)THEN
              CALL SERROR(' ROUTINE SPDEF1 INDICATED ILLEGAL SOLUTION BY
     1        IRES (=I1) WHEN CALLED WITH INITIAL SOLUTION - ERROR IN
     2        PROBLEM SPECIFICATION ASSUMED', 1, 1, I,0,0,0.0D0,0.0D0)
             ELSE IF(J.LT.0)THEN
              CALL SERROR(' REMESH ROUTINES INDICATED THAT CALCULATED OR
     1        SUPPLIED SPATIAL MESH DID NOT MATCH DISCONTINUITY POINTS
     2        SUPPLIED IN THE ARRAY XFIX ', 1, 0, 0,0,0,0.0D0,0.0D0)
             END IF
              ITIME = -1
              RETURN
           END IF
C RESETS NNPTS AND NEQN AFTER CALL TO INMESH SINCE THIS MAY CHANGE NPTS
           NEQN = NPDE*NNPTS + NV
           NPTS = NNPTS
           NVST  = NPDE*NPTS
           IF(NV .GT. 0) THEN
              NVST  = NPDE*NPTS + 1
              CALL SVINIT( U(NVST), NV)
           END IF
C   UPDATE THE MESH RETURNED TO THE USER
           DO 600 I = 1, NPTS
 600          X(I) = WK(I8 + I -1)
        END IF
        IF(NV .EQ. 0)THEN
           IBAND = 2*NPDE - 1
        ELSE
           IBAND = NPDE*NPTS
        ENDIF
        PDCODE = 'SPKEEL'
        RETURN
        END
       SUBROUTINE SRMESH(NRMESH,DXMESH,TRMESH,IPMINF,NPUSER,XRATIO,
     1                   CONST, NPTS, ISET)
C
       INTEGER NRMESH, IPMINF, NPUSER, MRMESH, JPMINF, NOPUSR, ISET,
     1         IOVFLO, NPTS
       DOUBLE PRECISION DXMESH, TRMESH, XRATIO, DYMESH, CONST, STPRAT,
     1        TOUT, DUNFLO, TWOU, KONST
       CHARACTER *6 RMTYPE
       LOGICAL REMESH
C**********************************************************************
C
C ALL PARAMETERS EXCEPT ISET MUST BE SUPPLIED BY THE USER IN CALLING
C THIS ROUTINE.
C
C   NRMESH  < 0  MEANS THAT THE NEW MESH WILL BE DETERMINED SOLELY BY
C                USING THE PARAMETER DXMESH.
C           = 0  REMESHING WILL TAKE PLACE AT THE END OF THE FIRST
C                TIME STEP WHEN T > TRMESH.
C           > 0  REMESHING WILL TAKE PLACE EVERY NRMESH TIMESTEPS.
C
C   DXMESH  : PARAMETER USED TO DETERMINE WHEN REMESHING SHOULD TAKE
C             PLACE WHEN NRMESH IS SET < 0
C             A POSSIBLE NEW MESH IS CALCULATED AT THE END OF EVERY
C             TIME STEP BUT IS ONLY ADOPTED IF
C               NEW     OLD             OLD     OLD
C              X    > X     + DXMESH ( X    - X     )
C               I      I                I+1    I
C          OR
C               NEW     OLD             OLD     OLD
C              X    < X     - DXMESH ( X    - X     )
C               I      I                I      I-1
C          DXMESH THUS CONTROLS HOW MUCH THE MESH CAN DRIFT FROM ONE
C          REMESH TO THE NEXT.IF THIS OPTION IS USED THE PARAMETER
C          NOPUSR BELOW WILL BE SET TO 1 BY THIS ROUTINE
C
C
C   TRMESH: PARAMETER USED TO DETERMINE WHEN REMESHING WILL TAKE PLACE
C           WHEN NRMESH = 0 . REMESHING WILL OCCUR WHEN AT THE FIRST
C           TIME LEVEL REACHED WHEN  T , THE INTEGRATION TIME , IS
C           GREATER THAN TRMESH.
C
C IPMINF - CONTROLS PRINT OUT OF MESH INFORMATION. IPMINF=0 NO PRINT,
C          =1 SUMMARY PRINT OF MESH CHARACTERISITICS E.G. MESH RATIOS &
C          MAX. INTEGRAL OF MONITOR PER INTERVAL (APPROX.=INPUT CONST)
C          =2 PRINT OF MONITORS, OLD & NEW MESHES AND MESH SIZES
C
C NPUSER   THIS ALLOWS THE NUMBER OF MESH POINTS TO BE VARIED WHEN THE
C          REMESH ROUTINES ARE CALLED . IF IT IS SET TO ZERO THEN THE
C          REMESH ROUTINES WILL PICK THE NUMBER OF POINTS REQUIRED,
C          SUBJECT TO THE MAXNPT CONSTRAINT IN THE SETUP ROUTINE INITSK
C          OR SETSKL. THIS PARAMETER IS SET NON-ZERO IF NRMESH < 0.
C
C XRATIO - INPUT BOUND ON ADJACENT MESH RATIO ( > 1 TYPICALLY 1.5 TO 3)
C          THE REMSH ROUTINES WILL TRY AND ENSURE THAT
C             X    - X    <  XRATIO (X - X   )
C              I+1    I               I   I-1
C  AND
C             X    - X    <  XRATIO (X   - X   )
C              I      I-1             I+1   I
C
C
C CONST -  REMESHING PARAMETER DESCRIBED IN USER MANUAL PART 2
C          IF IN DOUBT SET CONST TO   2 / (NPTS -1 ) . THIS ROUTINE
C          WILL CHECK TO ENSURE THAT
C
C            0.1/ (NPTS-1)  <   CONST < 10 / (NPTS-1)
C
C
C ISET   -  SET TO -1 IF THERE IS AN ERROR IN ONE OF INPUT PARAMETERS
C           SET TO  0 OTHERWISE
C**********************************************************************
      COMMON /MESHIN/ KONST, STPRAT, JPMINF, NOPUSR
      COMMON /MESHTI/ TOUT, REMESH, MRMESH, ICOUNT
      COMMON /MESHTJ/ DYMESH, RMTYPE
      COMMON /SCONS1/ DUNFLO, TWOU, IOVFLO
      SAVE   /SCONS1/, /MESHIN/ , /MESHTI/ , /MESHTJ/
      ISET  = 0
      RMTYPE = 'REMSET'
C               RMTYPE IS USED TO SET CONST TO 2 / (NPTS-1)
      REMESH = .TRUE.
      MRMESH = NRMESH
      JPMINF = IPMINF
      IF( IPMINF .LT. 0 .OR. IPMINF .GT. 2)THEN
        CALL SERROR(' REMESH SETUP ROUTINE FOUND THAT AN ILLEGAL VALUE
     1  SUPPLIED FOR PARAMETER IPMINF , VALUE (=I1) SHOULD BE IN THE
     2  RANGE 0 TO 2 ', 1, 1, IPMINF, 0 , 0, 0.0D0, 0.0D0)
        ISET   = -1
        JPMINF = 0
      END IF
      ICOUNT = 0
      IF( XRATIO .LT. 1.0D0 )THEN
        CALL SERROR(' REMESH SETUP ROUTINE FOUND THAT AN ILLEGAL VALUE
     1  SUPPLIED FOR PARAMETER XRATIO , VALUE (=R1) SHOULD BE GREATER
     2  THAN 1.0D0 ', 1, 0, 0, 0 , 1, XRATIO, 0.0D0)
        ISET   = -1
        XRATIO = 1.5
      END IF
      STPRAT = XRATIO
      NOPUSR = NPUSER
      IF( MRMESH .GE. 0)THEN
          TEMP = CONST * (NPTS-1)
          IF(TEMP .LT. 0.1D0 .OR. TEMP .GT. 10.D0)THEN
             CALL SERROR(' SRMESH- ILLEGAL VALUE OF CONST HAS BEEN
     1       DETECTED - CONST SHOULD LIE BETWEEN 0.1 / (NPTS-1) AND
     2       10 / (NPTS-1) ',  1, 0, 0, 0, 0, 0.0D0, 0.0D0)
             ISET = -1
             CONST = 2.D0 / (NPTS-1)
          END IF
          KONST = CONST
      ELSE IF(MRMESH .LT. 0.0D0)THEN
          CONST  = 2.D0 / (NPTS-1)
          KONST  = CONST
          NOPUSR = 1
          DYMESH = DXMESH
        IF( DXMESH .LT. 0.0D0 )THEN
          CALL SERROR(' REMESH SETUP ROUTINE FOUND AN ILLEGAL VALUE
     1    SUPPLIED FOR PARAMETER DXMESH , VALUE (=R1) SHOULD BE GREATER
     2    THAN 0.0D0 ', 1, 0, 0, 0 , 1, DXMESH, 0.0D0)
          ISET   = -1
          DXMESH = 1.5
        END IF
      END IF
      IF(MRMESH .EQ. 0)THEN
          TOUT = TRMESH
      END IF
      RETURN
      END
        SUBROUTINE INMESH( T, U, NPDE, NPTS, RESWK, NRESWK, V, NV,
     1                     MAXNPT, IRES, XFIX, IXFIX, NXFIX, NF)
C**********************************************************************
C       ROUTINE TO PROVIDE INITIAL MESH BY THREE CALLS TO PADRMS.
C**********************************************************************
        INTEGER NRESWK, ITRACE, IDEV, NPTS, NPDE, MAXNPT, NV
        INTEGER I, J, K, K1, K2, K3, K4, IONE, ITWO, IRES
        INTEGER NOP, IPMINF, IFAIL, NIPP1, NOPUSR, NIP, ISTATE, NXFIX
        INTEGER I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I13
     1         , I14, I15, I16, NF, NRMESH, ICOUNT
        DOUBLE PRECISION  RESWK(NRESWK), U(NPDE,1), T, V(1),
     1          CONST, STPRAT, XP, XFIX(1), IXFIX(1), DXMESH, TOUT
        CHARACTER *6 RMTYPE
        LOGICAL REMESH
        COMMON /SPSKWK/ I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11,
     1                  I12, I13, I14, I15, I16
        COMMON /SDEV2/ ITRACE, IDEV
        COMMON /MESHTI/ TOUT, REMESH, NRMESH, ICOUNT
        COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOPUSR
        COMMON /MESHTJ/ DXMESH, RMTYPE
        SAVE /SPSKWK/, /MESHIN/
C
C NOP=0 THEN NOP CALCULATED BY PADRMS ELSE NOP SET = NIP BY PADRMS
C AND SO CONST CRITERION MAY BE OVER OR UNDER SATISFIED
C
      CALL FIXSET( RESWK(I8), NPTS, XFIX, NXFIX, IXFIX, NF)
      IF(NF .LT. 0)RETURN
      K1 = NPDE + 1
      K2 = K1 + NPDE
      K3 = K2 + NPDE * NPDE
      K4 = K3 + NPDE
      NIP=NPTS
      DO 700 K = 1,3
       NOP=0
       IF(NOPUSR.NE.0)NOP=NIP
C        NAG SOFT FAIL FOR REMESHING
       IF(NRMESH.LT.0)THEN
         CONST = 2.D0 / (NIP - 1.0D0)
       ENDIF
       IFAIL=1
       ISTATE = 0
       DO 5 I=1,NIP
    5     RESWK(I10+I-1) = RESWK(I8 + I -1)
       IF(K.EQ.1.AND.ITRACE.GE.1)WRITE(IDEV,900)(RESWK(I10+I-1),I=1,NIP)
  900  FORMAT('0 INITIAL MESH POINTS ',/20(5D12.4/))
C----------------------------------------------------------------------
C       FORMATION OF FIRST DERIVS AND THEN THE FLUX BY CALLING INTERX.
C----------------------------------------------------------------------
       IONE   = 1
       ITWO = 2
       NIPP1  = NIP + 1
       DO 50 I = 1, NIPP1
          IF(I .EQ. 1)THEN
             XP = RESWK(I10)
          ELSE IF(I .EQ. NIPP1)THEN
             XP = RESWK(I10 + NIP-1)
          ELSE
             XP = (RESWK(I10+I) + RESWK(I10+I-1)) * 0.5D0
          END IF
          CALL INTERX( XP, RESWK, IONE, RESWK(I10), U, NIP, NPDE, IFAIL
     1                  ,ITWO)
          CALL SPDEF1( T, XP, NPDE, RESWK, RESWK(K1), RESWK(K2),
     1                 RESWK(K3), RESWK(K4), NV, V, RESWK(I7), IRES)
          IF(IRES .NE. 1)RETURN
        DO 50 J = 1,NPDE
  50      RESWK(I9 + (I-1)*NPDE + J -1) = RESWK(K4+J-1)
C
      CALL EVNRMF( T, NIP, NPDE, RESWK(I10), U, RESWK(I9), MAXNPT,
     1             RESWK(I12))
      CALL PADRIV( T, NPDE, NIP, RESWK(I8), MAXNPT, CONST, STPRAT,
     1             IPMINF, RESWK(I12), NOP, RESWK(I10),RESWK(I11),
     2             RESWK(I13), IFAIL, XFIX, NXFIX, IXFIX)
C
      IF( RMTYPE .EQ. 'REMSET' )THEN
C       TEST IF NEW MESH REALLY NEEDED
        CALL METEST( RESWK(I8), NIP, RESWK(I10), NOP, I, DXMESH)
        IF(I .EQ. 0)THEN
C          NEW MESH NOT NEEDED
           IMON = 1
           RETURN
        ELSE IF(ITRACE .GT. 0 .OR. IPMINF .GT. 0)THEN
C          NEW MESH IS NEEDED
           IF(ITRACE .GE. 1)WRITE(IDEV,910)
        END IF
      ELSE IF( NRMESH .EQ. 0 .AND. T .LT. TOUT)THEN
C          NEW MESH NOT NEEDED
           IMON = 1
           RETURN
      END IF
      IF(ITRACE .GE. 1)WRITE(IDEV,901)NOP,(RESWK(I10+I-1),I=1,NOP)
  901                  FORMAT('0 ',I4,' NEW MESH POINTS ',/20(5D12.4/))
C---------------------------------------------------------------------
C       FIND THE CORRECT INITIAL VALUES AT NEW MESH POINTS.
C---------------------------------------------------------------------
      DO 666 I = 1, NOP
 666     RESWK(I8+I-1) = RESWK(I10+I-1)
      CALL UINIT( NPDE, NOP, RESWK(I10), U)
      CALL FIXSET( RESWK(I8), NOP, XFIX, NXFIX, IXFIX, NF)
      IF(NF .LT. 0)RETURN
      NIP = NOP
 700  NPTS = NOP
      IF(ITRACE.GE.1)WRITE(IDEV,902)(RESWK(I10+I-1),I=1,NIP)
 902  FORMAT('0 FINAL MESH POINTS ',/20(5D12.4/))
 910  FORMAT('  NEW MESH ADOPTED  ')
      RETURN
      END
C**********************************************************************
C
        SUBROUTINE SKLRES ( NEQN, T, U, UDOT, FF, IRES, RWK, NRWK)
C
C**********************************************************************
C       O.D.E. RESIDUAL ROUTINE THAT PERFORMS SPATIAL DISCRETISATION
C       OF MIXED O.D.E. / P.D.E. PROBLEMS IN ONE SPACE DIMENSION.
C       THE METHOD USED IS THE SKEEL  FINITE DIFFERENCE DISCRETISATION.
C----------------------------------------------------------------------
        INTEGER NEQN, IRES, NRWK, NV, NXI, NVST, IV, ID, IFL, IT
        INTEGER NPTS, NPDE, ITRACE, IDEV, M, I, N, IR, NXFIX
        INTEGER I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I13
        INTEGER J2, J3, J4, J5, J6, J7, J8, J9, J10, J11, I14, I15, I16
        DOUBLE PRECISION  U(1), UDOT(1), FF(1), T, RWK(NRWK)
        CHARACTER*6 PDCODE
        COMMON /DISCHK/ PDCODE
        COMMON /SPSKWK/ I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11,
     1                  I12, I13, I14, I15, I16
        COMMON /SDEV2/ ITRACE,IDEV
        COMMON /SPSKEL/ NPDE, NPTS, M, NV, NXI, NVST, ID, NXFIX
        SAVE   /SPSKEL/, /DISCHK/, /SPSKWK/
C
        IF(PDCODE .NE. 'SPKEEL')THEN
           CALL SERROR(' ERROR IN SKLRES ROUTINE - THE SETUP ROUTINE
     1     SETSKL WAS NOT CALLED PRIOR TO SPRINT ENTRY', 1, 0, 0, 0, 0,
     2     0.0D0, 0.0D0)
           IRES = 2
           RETURN
        ENDIF
C
C  CALL THE SKEEL4 DISCRETISATION ROUTINE USING PARTITIONED WORKSPACE.
C
           J2 =  1 + NPDE
           J3 = J2 + NPDE
           J4 = J3 + NPDE
           J5 = J4 + NPDE
           J6 = J5 + NPDE
           J7 = J6 + NPDE
           J8 = J7 + NPDE * NPDE
           J9 = J8 + NPDE * NPDE
           J10= J9 + NPDE
           J11= J10+ NPDE
           IF(NV .GT. 0) THEN
              DO 10 I = 1,NV
 10              RWK(I7+I-1) = 0.0D0
           END IF
           IR = 1
           CALL SKEEL4( RWK(I8), T, U, UDOT, NPDE, NPTS, M, FF, RWK,
     1           RWK(J2), RWK(J3), RWK(J4), RWK(J5), RWK(J6), RWK(J7),
     2           RWK(J8), RWK(I9), IRES, U(NVST), UDOT(NVST), NV,
     3           RWK(J9), RWK(J10), RWK(J11), RWK(I7), IR)
C
        IF( IR .NE. 1)THEN
            IRES = IR
            RETURN
        END IF
        N = NEQN
        IFL = 0
        IV = NVST
        IF(NV .GT. 0) THEN
          IF(NXI .GT. 0)THEN
           IT = 2
C          GENERATE U  VALUES AND SPACE DERIVS AT COUPLING POINTS
           CALL INTERX( RWK(I6), RWK(I1),NXI,RWK(I8),U,NPTS,NPDE,IFL,IT)
C          GENERATE FLUX VALUES AT COUPLING POINTS
           CALL FXINTR( RWK(I6), RWK(I3), NXI, RWK(I8), RWK(I9),
     1                  NPTS, NPDE, IFL)
C          GENERATE TIME DERIVS AND THEIR SPACE DERIVS AT COUPLING PTS
           CALL INTERX( RWK(I6), RWK(I4), NXI, RWK(I8), UDOT, NPTS,
     1                  NPDE, IFL, IT)
          END IF
C         DEFINE THE AUXILLARY ODE RESIDUAL.
C
          CALL SODEFN( T, NV, U(IV), UDOT(IV), NPDE,NXI,RWK(I6),RWK(I1),
     1                  RWK(I2),RWK(I3),RWK(I4),RWK(I5), FF(IV), IRES)
       END IF
       RETURN
C             END OF SKLRES
       END
      SUBROUTINE SKEEL4( X, T, U, UDOT, NPDE, NPTS, NC, RES, BETA,
     1                   GAMMA, UMEAN, UX, F, G, CL, CF, RFUN, IRES,
     2                   V, VDOT, NV, FT, BT, GT, VT, IR)
C***********************************************************************
C   SPRINT IMPLEMENTATION OF THE NEW SKEEL DISCRETISATION.
C***********************************************************************
      INTEGER NPDE, NPTS, IRES, IT, JT, J, L, L1, NC, NV, NVIT
      INTEGER ITRACE, IDEV, K, I, IZ(12), IR
      DOUBLE PRECISION  APLUS, BPLUS, EPLUS, H, PL, PR, SUMAB, T, XL,
     *        XR, XMEAN, XPOW, XSUM, HQ, HSUM
      DOUBLE PRECISION  X(NPTS),U(NPDE,NPTS),BETA(NPDE),
     *        GAMMA(NPDE),UX(NPDE),UMEAN(NPDE),F(NPDE),G(NPDE),
     *        CL(NPDE,NPDE),CF(NPDE,NPDE), RFUN(NPDE,1), RES(NPDE,NPTS),
     *        UDOT(NPDE,NPTS), V(1), VDOT(1),
     *        FT(NPDE), BT(NPDE), GT(NPDE), VT(1)
      COMMON /SDEV2/ ITRACE, IDEV
C---------------------------------------------------------------------
C     THIS SUBROUTINE SERVES AS AN INTERFACE BETWEEN A
C     PARABOLIC PARTIAL DIFFERENTIAL EQUATION IN ONE
C     SPACE VARIABLE AND AN INTEGRATOR OF INITIAL VALUE PROBLEMS,
C
C     THE DIFFERENTIAL EQUATION SHOULD BE OF THE FORM
C
C     C(I) * (D/DT)U(I) = (D/DX)(X**NC*F(I,X,T,U,UX)))/X**NC
C
C                                 - G(I,X,T,U,UX),I= 1,...,NPDE;
C
C     WITH BOUNDARY CONDITIONS
C
C     BETA(I)*F(I,X,U,UX) = GAMMA(I) ,I = 1,...,NPDE;
C                           AT X = X(1) OR X(NPTS)
C     SKEEL4 TRANSFORMS THE SYSTEM OF PARTIAL DIFFERENTIAL EQUATIONS
C     INTO A SYSTEM OF ORDINARY EQUATIONS BY SEMI-DISCRETIZATION IN
C     THE SPACE VARIABLE X; THIS SEMI-DISCRETIZATION IS PERFORMED
C     BY APPLICATION OF THE SKEEL 4 DISCRETISATION
C     IN ORDER TO MAKE AVAILABLE TO THE USER AN ESTIMATE OF THE FLUX
C     AT THE END-PTS FOR DIRICHLET B.C.S IT USES 3-PT END-ON FORMULAE
C     N.B.   MATERIAL DISCONTINUITIES MUST NOT BE PRESENT
C     ---    WITHIN THESE 3 MESH PTS.
C
C     DESCRIPTION OF THE PARAMETERS;
C
C     INPUT:
C
C     X:   DIMENSION X(NPTS);
C          X(1),...,X(NPTS) IS A PARTITION OF (X(1),X(NPTS))
C
C     T:   THE TIME VARIABLE;
C
C     U:   DIMENSION U(NPDE,NPTS);
C          U(I,J) IS AN APPROXIMATION OF U(I,X(J),T),I = 1,...,NPDE;
C                                                    J = 1,...,NPTS;
C     NPDE:   THE NUMBER OF PARTIAL DIFFERENTIAL EQUATIONS;
C
C     NPTS:   THE NUMBER OF GRIDPOINTS;
C
C     NC:   A NUMBER DESIGNING THE KIND OF SPACE COORDINATES;
C           NC = 0 : CARTESIAN COORDINATES;
C           NC = 1 : CIRCULAR COORDINATES;
C           NC = 2 : SPHERICAL COORDINATES;
C
C     RES(NPDE,NPTS) EMPTY ARRAY.
C
C     RFUN(NPDE,NPTS+1) EMPTY ARRAY
C
C     BETA,GAMMA,UMEAN,UX,F,G,CT,FT,GT,BT:WORK ARRAYS OF DIMENSION NPDE
C     VT : WORK ARRAY OF DIMENSION NV.
C
C     OUTPUT:
C
C     RES:   DIMENSION (NPDE,NPTS); THE  RESIDUAL OF THE SEMI-
C            DISCRETISED P.D.E.
C
C     RFUN   DIMENSION (NPDE,NPTS); CONTAINS THE VALUES OF THE FLUX.
C            RFUN(J,I) CONTAINS THE FLUX FOR THE JTH P.D.E. AT
C                      I = 1 OR NPTS+1 THE LEFT OR THE RIGHT BOUNDARY
C                      I = 2,NPTS MIDWAY BETWEEN X(I) AND X(I-1).
C
C     N.B. TO HANDLE IRES = -1 THE FOLLOWING OPTIONS ARE USED.
C
C     IF IRES = -1 AND NV = 0 THEN IT=0 AND NVIT = 1 TO CALCULATE ONLY
C     THOSE PARTS OF RESID DEPENDING ON THE TIME DERIV
C     IN THE CASE WHEN NV NON-ZERO THEN IT = 1 AND NVIT ONLY IS ZERO
C     WHEN IRES IS -1 .
C
C
C
C----------------------------------------------------------------------
       DO 5 J = 1, NPDE
 5        UX(J)   = 0.0D0
      IT = 1
      NVIT = 1
      IF(IRES .EQ. -1)THEN
         IT = 0
         IF(NV .GT. 0)THEN
            NVIT = 0
            IT =   1
         END IF
      END IF
C
      BPLUS = 0.0D0
      JT = 0
      XR = X(1)
      DO 150 L = 2,NPTS
        DO 6 I = 1,12
6          IZ(I) = 1
C
C       THE SEGMENTWISE ASSEMBLY OF THE RIGHT HAND SIDE BEGINS;
C      AT FIRST, SOME NUMBERS CHARACTERISTIC FOR NC ARE COMPUTED.
C
      XL = XR
      L1 = L - 1
      XR = X(L)
      H = XR - XL
      XSUM = XR + XL
      IF(NC .EQ. 0 )THEN
C        NON-POLAR CASE
         APLUS = H*0.5
         SUMAB = APLUS + BPLUS
         BPLUS = H*0.5
         EPLUS = 1.0D0
      ELSE IF(L .EQ. 2 .AND. XL .LE. 1.0D-8)THEN
C        POLAR SINGULARITY
            EPLUS = 2.0D0 * H**NC  / (NC+2)
            APLUS = XR**(NC+1) /(NC+2)/(NC+1)
            SUMAB = APLUS + BPLUS
            BPLUS = XR**(NC+1) / (NC+2)
       ELSE IF(NC .EQ. 1)THEN
C        CYLINDRICAL POLAR CO-ORDINATES
         APLUS = -XL**2 *0.5 + XSUM * H * 0.25 / DLOG(XR/XL)
         SUMAB = APLUS + BPLUS
         BPLUS =  XR**2 *0.5 - XSUM * H * 0.25 / DLOG(XR/XL)
         EPLUS =  H / DLOG(XR/XL)
       ELSE IF(NC .EQ. 2)THEN
         APLUS = -XL/6.0D0 * (2.0 * XL **2 - XL*XR - XR**2)
         SUMAB = APLUS + BPLUS
         BPLUS =  XR/6.0D0 * (2.0 * XR **2 - XL*XR - XL**2)
         EPLUS = XL * XR
       END IF
       IF(NC .EQ. 0)THEN
          PR = 0.5D0
       ELSE IF(NC .EQ. 1)THEN
          PR = 0.5D0 + H / (6.0D0 * XSUM)
       ELSE
C         PR = (XSUM**2 + 2.0D0*XR**2) * 0.25 / (XSUM**2 - XL*XR)
          PR = 0.5D0
       END IF
      PL = 1.0 - PR
      XMEAN = XL + H*PR
      IF (L .EQ. 2) THEN
C       FIRST SEGMENT,PROCESS THE BOUNDARY CONDITIONS; IF OF NEUMANN
C       TYPE THE VALUE OF THE FLUX F AT THE BOUNDARY IS UPDATED.
        HQ=X(3)-X(2)
        HSUM=H+HQ
        DO 40 J = 1,NPDE
C          FOR DIRICHLET B.C. ESTIMATE UX AND THE FLUX AT THE END-PT
C          FOR THE USER (NOT NEEDED FOR SKEEL4) BY 3-PT END ON
C          FORMULA  USED VIA SPDEF1 TO EVALUATE THE FLUX F OR RFUN
C          THIS FORMULA GIVES CONSTANT 2ND DERIV. AT X(1) AND X(2)
           UX(J)=-(2.0D0*H+HQ)*U(J,1)/(H*HSUM)+HSUM*U(J,2)/(H*HQ)
     1             -H*U(J,3)/(HQ*HSUM)
           RES(J,1) = 0.0D0
           DO 40 K = 1,NPDE
              CF(K,J) = 0.0D0
 40           CL(K,J) = 0.0D0
        CALL SPDEF1(T,XMEAN,NPDE,U,UX,CF, G, F,   NV, V, VDOT, IZ(3))
        CALL SPDEF1(T,X(1) ,NPDE,U,UX,CL,G,RFUN,  NV, V, VDOT, IZ(1))
        DO 41 I = 1,NPDE
 41        UX(I) = (U(I,2) - U(I,1)) / H
        CALL BNDR( T,BETA,GAMMA,U,UX,NPDE, .TRUE. ,NV, V, VDOT, IZ(2))
        IF(NVIT .EQ. 0)THEN
           CALL BNDR( T,BETA, BT, U,UX,NPDE,.TRUE.,NV, V, VT, IZ(4))
C---       CALL SPDEF1( T,X(1),NPDE,U,UX,CL,GT,FT,NV,V, VT, IZ(5))
           DO 42 I = 1,NPDE
              GAMMA(I) = GAMMA(I) - BT(I)
C---           G(I)    =  G(I)    - GT(I)
C---           F(I)    =  0.0D0
 42        CONTINUE
        END IF
        XPOW = 1.
        IF (NC .GT. 0) XPOW = XL**NC
        DO 80 J = 1,NPDE
           IF( BETA(J) .EQ. 0.0D0)THEN
               DO 78 K = 1,NPDE
 78              CL(J,K) = 0.0D0
           ELSE
               RFUN(J,1) =  NVIT*GAMMA(J)/BETA(J) + RFUN(J,1) *(1-NVIT)
               RES (J,1) = -XPOW*GAMMA(J)/BETA(J) * IT
               DO 79 K = 1,NPDE
 79              CL(J,K)   =  CF(J,K) * APLUS
           END IF
 80     CONTINUE
      ELSE IF(L .EQ. NPTS) THEN
C BECAUSE OF DO 140 CODE CAN ONLY SET RES(J,1) AFTER L=2
        DO 95 J = 1,NPDE
           IF (BETA(J) .EQ. 0.)RES(J,1) = GAMMA(J) * IT
 95      CONTINUE
      END IF
C
C       HERE THE IMPLEMENTATION OF THE LEFT BOUNDARY CONDITIONS ENDS
C       AND  THE REAL ASSEMBLY BEGINS.
C
120   DO 130 J = 1,NPDE
         UMEAN(J) = PL*U(J,L1) + PR*U(J,L)
         UX(J) = (U(J,L) - U(J,L1))/H
         DO 130 K = 1,NPDE
130         CF(K,J) = 0.0D0
        CALL SPDEF1(T, XMEAN, NPDE,UMEAN,UX,CF,G, F, NV,V,VDOT,IZ(6))
        IF(NVIT .EQ. 0)THEN
          CALL SPDEF1(T,XMEAN,NPDE,UMEAN,UX,CF,GT,FT,NV,V,VT, IZ(7))
           DO 135 I = 1,NPDE
               G(I)    =  G(I)    - GT(I)
               F(I)    =  0.0D0
 135       CONTINUE
        END IF
      DO 140 J = 1,NPDE
         RES(J,L1)  =   (RES(J,L1) -G(J)*APLUS + F(J)*EPLUS) / SUMAB*IT
         RES(J,L)   =              -G(J)*BPLUS - F(J)*EPLUS
C SET RFUN = F     (ONLY IF NVIT=1)
         RFUN(J,L)=RFUN(J,L)*(1-NVIT)+F(J)*NVIT
         DO 139 K = 1,NPDE
            RES(J,L1) = RES(J,L1) -
     1                (APLUS*CF(J,K)*JT + CL(J,K))/SUMAB * UDOT(K,L1)
            CL(J,K)      =  BPLUS*CF(J,K)
 139     CONTINUE
140   CONTINUE
      JT = 1
      DO 149 I = 1,7
         IF(IZ(I) .NE. 1)THEN
            IR = IZ(I)
            GOTO 190
         END IF
149   CONTINUE
150   CONTINUE
C
C       FINALLY, PROCESS THE RIGHT BOUNDARY CONDITIONS.
C
      XPOW = 1.0D0
      IF(NC .GT. 0)XPOW = XR**NC
C---  DO 155 J = 1,NPDE
C---     DO 155 K = 1,NPDE
C55         CL(K,J) = 0.0D0
      CALL BNDR(T,BETA,GAMMA,U(1,NPTS),UX,NPDE,.FALSE.,NV,V,VDOT,IZ(8))
C---  CALL SPDEF1(T,X(NPTS),NPDE,U(1,NPTS),UX,CL,G,F,NV,V,VDOT,IZ(9))
        IF(NVIT .EQ. 0)THEN
           CALL BNDR(T,BETA,BT,U(1,NPTS),UX,NPDE,.FALSE.,NV,V,VT,IZ(10))
C---  CALL SPDEF1(T,X(NPTS),NPDE,U(1,NPTS),UX,CL,GT,FT,NV,V,VT,IZ(11))
           DO 170 I = 1,NPDE
              GAMMA(I) = GAMMA(I) - BT(I)
C---           G(I)    =  G(I)    - GT(I)
C---           F(I)    =  0.0D0
C---           RES(I,NPTS)= -G(I)*BPLUS - F(I)*EPLUS
  170      CONTINUE
        END IF
      H=X(NPTS-1)-X(NPTS-2)
      HQ=X(NPTS)-X(NPTS-1)
      HSUM=H+HQ
      DO 160 J = 1,NPDE
C        FOR DIRICH CONDITION ESTIMATE UX AND THE FLUX AT THE END-PT SO
C        THAT IT IS AVAILABLE TO THE USER (NOT NEEDED FOR SKEEL4)
C        3-PT END-ON FORMULA IS USED VIA SPEDF1 TO FORM FLUX F OR RFUN
         UX(J)=(2.0D0*HQ+H)*U(J,NPTS)/(HQ*HSUM)-HSUM*U(J,NPTS-1)/(H*HQ)
     1             +HQ*U(J,NPTS-2)/(H*HSUM)
         DO 160 K = 1,NPDE
            CL(K,J) = 0.0D0
  160  CONTINUE
      I = NPTS + 1
      CALL SPDEF1( T, X(NPTS), NPDE, U(1,NPTS), UX, CL,
     1             G, RFUN(1,I), NV, V, VDOT, IZ(12))
      DO 180 J = 1,NPDE
         RES(J,NPTS) = RES(J,NPTS) * IT
         IF (BETA(J) .EQ. 0.) THEN
             RES(J,NPTS) = GAMMA(J) * IT
         ELSE
             DO 179 K = 1,NPDE
                CF(J,K) = CF(J,K) * BPLUS
 179            RES(J,NPTS) = RES(J,NPTS)  -CF(J,K)*UDOT(K,NPTS)
             RES(J,NPTS) = (RES(J,NPTS)+XPOW*IT*GAMMA(J)/BETA(J))/BPLUS
             RFUN(J,NPTS+1) = GAMMA(J)/BETA(J) * NVIT +
     1                                          RFUN(J,NPTS+1)*(1-NVIT)
         END IF
C        IF(ITRACE .GE. 1)WRITE(IDEV,181)J,IT,RES(J,NPTS),UDOT(J,NPTS)
C181     FORMAT(' J =',I4,' IT',I3,' RES = ',D13.5,'  UDOT',D11.3)
 180  CONTINUE
      DO 189 I = 8,12
         IF(IZ(I) .NE. 1)THEN
            IR = IZ(I)
            GOTO 190
         END IF
189   CONTINUE
190   CONTINUE
      RETURN
      END
        SUBROUTINE INTERX( XP, UP, IPTS, X, U, NPTS, NPDE, IFLAG, IT)
C**********************************************************************
C       ROUTINE TO COMPUTE SOLUTION AND POSSIBLY ITS FIRST DERIV
C       AT THE INTERPOLATION POINTS STORED IN THE ARRAY XP.
C       LINEAR INTERPOLATION IS USED.
C       PARAMETER LIST
C       --------------
C       XP(IPTS)       THE SPATIAL INTERPOLATION POINTS , FOR EFFICIENCY
C                      XP(I) < XP(I+1) , I = 1, IPTS-1.
C       UP(NPDE,IPTS,IT)  ARRAY TO HOLD THE VALUES FOUND BY LINEAR
C                         INTERPOLATION.
C          UP(J,K,1)   HOLDS U(XP(K),T) FOR P.D.E. J
C          UP(J,K,2)   HOLDS DU/DX OF UP(J,K,1)
C
C       X(NPTS)     IS AN ARRAY THAT CONTAINS THE ORIGINAL MESH.
C
C       U(NEQN)     HOLDS THE ORIGINAL SOLUTION ; THE FIRST NPDE*NPTS
C                   COMPONENTS OF THIS CONTAINS THE PDE SOLUTION AT THE
C                   MESH POINTS X(NPTS) AND THE LAST NV COMPONENTS ARE
C                   THE ADDITIONAL COUPLED ODE VARIABLES.
C       IFLAG       ERROR FLAG ; SET = I IF XP(I) LIES OUTSIDE THE RANGE
C                   (X(1) , X(NPTS)) .
C
C       IT          HAS VALUE 1 OR 2 DEPENDING ON HOW MANY COMPONENTS
C                   OF THE ARRAY UP ARE REQUIRED.
C***********************************************************************
        INTEGER IPTS,NPTS,NPDE,IFLAG,JI,I,J,ITRACE, IDEV, IOVFLO, IT
        DOUBLE PRECISION XI, XI1, UI1, UI, TWOU, X(NPTS),
     -          XP(IPTS), UP(NPDE,IPTS,IT), U(NPDE,1), DUNFLO
        COMMON /SCONS1/ DUNFLO, TWOU, IOVFLO
        COMMON /SDEV2/  ITRACE, IDEV
        SAVE   /SCONS1/
        DO 999 JI=1,IPTS
           IFLAG = 0
C
C       SEARCH THE GRID POINTS WHICH BOUND XP(JI)
C
           IF(XP(JI).LT.X(1)-TWOU) GOTO 15
           DO 10 I =2,NPTS
              IF(XP(JI).LE.X(I)) GOTO 20
10         CONTINUE
           IF(XP(JI).GT.X(NPTS)+TWOU) GOTO 15
           I = NPTS
           GOTO 20
C
C       XP(JI) IS OUTSIDE THE MESH
C
15         IFLAG = JI
           CALL SERROR(' INTERX ERROR: MESH POINT (=I1) VALUE (=R1) IS
     1                   OUTSIDE THE X RANGE ', 1, 1, JI, 0, 1, XP(JI),
     2                   0.0D0)
           RETURN
20         CONTINUE
C
C       THE POINT XP(JI) IS BOUNDED BY THE I-1TH AND THE ITH GRID PTS.
           XI1 = X(I-1)
           XI = X(I)
           DO 30 J = 1,NPDE
              UI1 = U(J,I-1)
              UI = U(J,I)
C             APPLY LINEAR INTERPOLATION
              UP(J,JI,1) = UI1 + (XP(JI) - XI1 )*(UI - UI1)/(XI - XI1)
              IF(IT .GE. 2)THEN
C                COMPUTE IST SPACE DERIV AT XP(JI)
                 IF(XP(JI) .LT. XI .OR. I .EQ. NPTS)THEN
                    UP(J,JI,2) = (UI - UI1)/(XI - XI1)
                 ELSE
C                   INTERPOLATION AT AN INTERIOR MESH POINT
                     UP(J,JI,2) = (U(J,I+1) - UI1)/(X(I+1) - XI1)
                 END IF
              END IF
 30        CONTINUE
999     CONTINUE
1000    CONTINUE
        RETURN
        END
        SUBROUTINE FXINTR( XP, UP, IPTS, X, U, NPTS, NPDE, IFLAG)
C**********************************************************************
C       ROUTINE TO COMPUTE FLUX AT THE INTERPOLATION POINTS XP(IP)
C       LINEAR INTERPOLATION IS USED.
C       DEBUGGED AND MODIFIED BY RON FURZELAND, MARCH, 1985
C       PARAMETER LIST
C       --------------
C       XP(IPTS)       THE SPATIAL INTERPOLATION POINTS , FOR EFFICIENCY
C                      XP(I) < XP(I+1) , I = 1, IPTS-1.
C       UP(NPDE,IPTS)  ARRAY TO HOLD THE VALUES FOUND BY LINEAR INTERP
C
C       X(NPTS)     IS AN ARRAY THAT CONTAINS THE ORIGINAL MESH.
C
C       U(NPDE, NPTS+1) : THE ORIGINAL FLUX ; U(J,1) ARE THE FLUXES AT
C                   X(1) AND U(J,NPTS) ARE THE FLUXES AT X(NPTS)
C                   WHILE  U(J,K) IS THE FLUX
C                   AT 0.5 * (X(K-1) + X(K)) , K = 2, NPTS , J = 1,NPDE.
C       IFLAG       ERROR FLAG ; SET = 1 IF EXTRAPOLATION TRIED ELSE 0.
C
C       IT          HAS VALUE 1 OR 2 DEPENDING ON HOW MANY COMPONENTS
C                   OF THE ARRAY UP ARE REQUIRED.
C***********************************************************************
        INTEGER IPTS, NPTS, NPDE, IFLAG, JI, I, J, ITRACE, IDEV, IOVFLO
        DOUBLE PRECISION XI, XI1, UI1, UI, TWOU, X(NPTS), XL, XR,
     -          XP(IPTS), UP(NPDE,IPTS), U(NPDE,1), DUNFLO
        COMMON /SCONS1/ DUNFLO, TWOU, IOVFLO
        COMMON /SDEV2/  ITRACE, IDEV
        SAVE   /SCONS1/
        DO 999 JI=1,IPTS
           IFLAG = 0
C
C       SEARCH THE GRID POINTS WHICH BOUND XP(JI)
C
           IF(XP(JI).LT.X(1)-TWOU) GOTO 15
           DO 10 I =1,NPTS
              XI1 = X(I)
              IF(I      .LT. NPTS)XI1 = (XI1 + X(I+1))  * 0.5D0
              IF(XP(JI) .LE. XI1) GOTO 20
10         CONTINUE
           IF(XP(JI).GT.X(NPTS)+TWOU) GOTO 15
           I = NPTS
           GOTO 20
C       XP(JI) IS OUTSIDE THE MESH
15         IFLAG = 1
           RETURN
20         CONTINUE
C    THE POINT XP(JI) IS BOUNDED BY THE I-1TH AND THE ITH  PTS.
           IF (I.GT.1) THEN
              IF (I.EQ.NPTS) THEN
C    I=NPTS, INTERPOLATE U(J,NPTS), U(J,NPTS+1) IN (LAST MID-PT, X(NPTS)
                    XL = 0.5D0 * ( X(NPTS-1) + X(NPTS) )
                    XR = X(NPTS)
              ELSE
C  I=2..NPTS, INTERPOLATE U(J,I), U(J,I+1) IN (LAST MID-PT, NEXT MID-PT)
                    XL = 0.5D0 * (X(I) + X(I-1))
                    XR = XI1
              ENDIF
           ELSE
C          I=1, INTERPOLATE U(J,1) AND U(J,2) IN (X(1),0.5(X(1)+X(2)))
               XL = X(1)
               XR = XI1
           ENDIF
           DO 30 J = 1,NPDE
              UI = U(J,I)
C             APPLY LINEAR INTERPOLATION
              UP(J,JI) = UI + (XP(JI)-XL)* (U(J,I+1) - UI ) / (XR-XL)
 30        CONTINUE
999     CONTINUE
1000    CONTINUE
        RETURN
        END
        SUBROUTINE JAC
C***********************************************************************
C       DUMMY ROUTINE FOR THE LOADER
        RETURN
        END
        SUBROUTINE INTDRV( XP, UP, IPTS, X, U, NPTS, NPDE, IFAIL, C,
     1                     IXFIX, NFIX)
C***********************************************************************
C   SPRINT CUBIC SPLINE INTERPOLATION ROUTINE,DRIVER FOR PIECE-POLY
C
C   N.B. THIS ROUTINE ASSUMES THAT THE ARRAY IFIX CONTAINS THE INDICES
C        OF THE FIXED POINTS OF THE EXISTING MESH
C   I.E.  X(IXFIX(I))  IS A FIXED POINT AT WHICH THERE MAY BE A P.D.E.
C        DISCONTINUITY.
C***********************************************************************
        INTEGER IPTS, NPTS, NPDE, IFAIL, NFIX, ITRACE, IDEV
        INTEGER N, JI, I, J, L, M, KK, K, NXFXP1, JK, NKPTS, JJ, IT
        DOUBLE PRECISION X(NPTS), XP(IPTS), UP(NPDE,IPTS),
     -          U(NPDE,NPTS), C(NPTS,3), IXFIX(1)
        COMMON /SDEV2/ ITRACE, IDEV
        IF( NFIX .EQ. 0)THEN
C           NO MATERIAL INTERFACES PRESENT USE ORDINARY CUBIC SPLINE.
            CALL INTCUB( XP, UP, IPTS, X, U, NPTS, NPDE, IFAIL, C)
        ELSE
C           CUBIC SPLINE ROUTINE MUST BE CALLED IN PIECEWISE FASHION
            NXFXP1 = NFIX + 1
            JJ = 1
            DO 100 I = 1, NXFXP1
               JK = 0
               IF(I.EQ.1)THEN
                 K = 1
               ELSE
                 K = IXFIX(I-1) + 0.1D0
               END IF
C                X(K) IS THE STARTING POINT FOR THE NEXT ELEMENT
               IF( I .EQ. NXFXP1)THEN
                   NKPTS = NPTS - K + 1
                   KK = NPTS
               ELSE
                   NKPTS = IXFIX(I) - K + 1.1D0
                   KK    = IXFIX(I) + 0.1D0
               END IF
 80            J = JJ + JK
               IF(XP(J) .LE. X(KK) .AND. J .LE. IPTS)THEN
                 JK = JK + 1
C                   COUNTS THE NO OF NEW POINTS IN THIS ELEMENT
                 GOTO 80
               ELSE
                 J = J-1
C                CAN INTERPOLATE TO FIND NEW SOLUTION VALUES FROM
C                XP(JJ)TH MESH POINT TO XP(J) TH  MESH POINT.
C**************  IF(ITRACE .GE. 1)WRITE(IDEV,90)JJ,J,JK,K,NKPTS
C 90             FORMAT(' XP(',I3,') TO XP(',I3,' USING',I4,'POINTS'/
C****1********   ' INTERP FROM X(',I3,') USING ',I4,' POINTS')
                 IF(JK .GT. 0 .AND. NKPTS .GE. 4)THEN
                                  CALL INTCUB( XP(JJ), UP(1,JJ), JK,
     1                             X(K), U(1,K), NKPTS, NPDE, IFAIL, C)
                 ELSE IF(JK.GT.0)THEN
                                  IT = 1
                                  CALL INTERX( XP(JJ), UP(1,JJ), JK,
     1                             X(K), U(1,K), NKPTS, NPDE, IFAIL, IT)
                 END IF
  10             JJ = J
               END IF
 100        CONTINUE
          END IF
          RETURN
          END
        SUBROUTINE INTCUB( XP, UP, IPTS, X, U, NPTS, NPDE, IFAIL, C)
C***********************************************************************
C   SPRINT CUBIC SPLINE INTERPOLATION ROUTINE .
C***********************************************************************
        INTEGER IPTS, NPTS, NPDE, IFAIL, IOVFLO
        INTEGER N, JI, I, J, L, M, KK, K
        DOUBLE PRECISION X(NPTS), XP(IPTS), UP(NPDE,IPTS),
     -          U(NPDE,NPTS), C(NPTS,3)
        DOUBLE PRECISION DIVDF1, DIVDF3, DX, P
        DOUBLE PRECISION DUNFLO, TWOU
C**********************************************************************
C       INPUTS
C       ******
C       NPDE           = THE NUMBER OF P.D.E.S (FIRST DIMENSION OF THE
C                        ARRAYS U AND UP ) , ASSUMED GREATER THAN 0.
C       NPTS           = NUMBER OF DATA POINTS ASSUMED GREATER THAN TWO
C       (X(I), U(K,I))   ABSCISSAE AND ORDINATES OF THE DATA POINTS.
C                        X IS ASSUMED TO BE STRICTLY INCREASING.
C              K = 1,NPDE AND I = 1,NPTS.
C       XP(J)  J =1,IPTS THE POINTS AT WHICH SOLUTION VALUES ARE
C                        REQUIRED.
C       UP(K,J)          EMPTY ARRAY THAT HOLDS THE SOLUTION VALUES.
C              K = 1,NPDE AND J = 1,IPTS
C       OUTPUTS
C       ******
C       C(I,J)           J = 1,3 I = 1, N-1 = THE POLYNOMIAL COEFFS OF
C                        THE CUBIC SPLINE INTERPOLATION WITH INTERIOR
C                        KNOTS X(2) TO X(N-1). IN THE INTERVAL
C                        X(I)..X(I+1) THE SPLINE IS GIVEN BY
C                F(X) = U(K,I) + H* (C(I,1) + H*(C(I,2) +H*C(I,4)))
C                        WHERE H = X -X(I).
C                N.B.    IN THE CASE WHEN NPDE > 1 THESE COEFFICIENTS
C                        WILL BE THOSE FOR TH NPDE TH P.D.E.
C
C       UP(K,JJ)         K = 1,NPDE . JJ = 1,NP .
C                        ARRAY HOLDING SOLUTION VALUES FOR K P.D.E.S AT
C                        THE POINTS DEFINED BY XP(JJ) , JJ =1,NP.
C       IFAIL            ERROR INDICATOR THAT IS SET TO 1 IF
C                        ANY OF THE POINTS XP(JJ) LIE OUTSIDE THE
C                        RANGE X(1),X(NPTS).
C
C*********************************************************************
       COMMON /SCONS1/ DUNFLO, TWOU, IOVFLO
       SAVE   /SCONS1/
       IF(NPTS .LT. 2 )THEN
          IFAIL = 1
          CALL SERROR(' INTCUB-  CUBIC SPLINE ROUTINE DOES NOT
     1                  WORK WIH NPTS LESS THAN 2   -ERROR',
     2                   1, 0, 0, 0, 0, 0.0D0, 0.0D0)
          RETURN
       END IF
       N = NPTS
       L = N-1
       DO 90 K = 1,NPDE
C        FORM SPLINE COEFFS FOR KTH P.D.E.
C        -------------------------------------------------------------
C        | A TRIDIAGONAL LINEAR SYSTEM FOR THE SLOPES S(I) OF U AT X(I)
C        | I = 1,N IS GENERATED AND THEN SOLVED BY GAUSSIAN ELIMINATION
C        | WITH S(I) ENDING UP AT IN C(2,I) FOR ALL I = 1,N.
C        | THE PRECISE FORM OF THE SYSTEM IS
C        |  FIRST EQUATION  NOT A KNOT CONDITION AT L.H.S.
C        |  --------------
C        |    DX(3) * S(1) + (DX(2) + DX(3)) * S(2) = R(1)
C        | WHERE
C        |    R(1) = (DX(3)*DY(2)*(3DX(2)+2DX(3)) + DY(3)*DX(2)**2 ) /
C        |            (DX(2) + DX(3))
C        |
C        |  MIDDLE EQUATIONS
C        |  ----------------
C        | DX(I+1)* S(I-1) + (DX(I+1)+DX(I))* S(I) + DX(I)* S(I+1)=R(I)
C        | WHERE
C        |    R(I) = 3*(DX(I+1)*DY(I) + DX(I)*DY(I+1)) , I= 2,...,N-1
C        |
C        |  LAST EQUATION  NOT A KNOT CONDITION AT R.H.S.
C        |  -------------
C        | (DX(N)+DX(N-1) * S(N-1) + DX(N-1) * S(N) = R(N)
C        |  WHERE
C        |    R(N) = (DX(N-1) * DY(N) * (3*DX(N) + 2*DX(N-1)) +
C        |                      DY(N-1) * DX(N) * DX(N)) /(DX(N)+DX(N-1)
C        |  AND
C        |    DX(I) = X(I)-X(I-1) , DY(I) = (C(I,1)-C(I-1,1))/DX(I)
C        |                I = 2, ... , N.
C        |
C        |  THE UPPER DIAGONAL OF THE MATRIX IS STORED IN C(I,3) AND
C        |  THE RIGHTHAND SIDE OF THE SYSTEM IS STORED IN C(I,2) AND IS
C        |  OVERWRITTEN BY THE SOLUTION S(I) THAT IS THE DERIV OF THE
C        |  SPLINE INTERPOLANT AT THE POINT X(I). THE ARRAYS C(I,3)
C        |  AND C(.,4) ARE USED INITIALLY FOR TEMPOARY STORAGE OF THE
C        |  DIFFERENCES OF THE MESH AND OF FIRST DIVIDED DIFFERENCES OF
C        |  THE SOLUTION.
C        |--------------------------------------------------------------
            DO 20 M = 2, NPTS
              C(M,2) = X(M) - X(M-1)
 20           C(M,3) = (U(K,M) - U(K,M-1))/C(M,2)
C
C           FORM THE TOP ROW OF THE MATRIX
            C(1,1) = (C(3,2) * C(2,3) * (3.0D0*C(2,2) + 2.0D0*C(3,2)) +
     1             C(3,3) * C(2,2)**2 ) / ((C(2,2) + C(3,2)) * C(3,2) )
            C(1,2) =  C(2,2) / C(3,2) + 1.0D0
C
            DO 30 I = 2,L
C             PERFORM THE INNER ELIMINATION LOOP
              P = 2.0D0 * (C(I,2) + C(I+1,2)) - C(I-1,2)*C(I+1,2)
              C(I,1) = (3.0D0 * (C(I+1,2)*C(I,3) + C(I,2)*C(I+1,3)) -
     1                  C(I-1,1)*C(I+1,2)) / P
              C(I,2) =  C(I,2) / P
C                       C(I,2) NO LONGER CONTAINS X(I) - X(I-1)
 30         CONTINUE
C
C           LAST EQUATION FOR R HAND NOT A KNOT CONDITION.
            DX = X(N-1) - X(N-2)
            P =  DX - C(N-1,2) * (X(N) - X(N-2))
            C(N,1)=((DX*C(N,3) * (3.0D0*C(N,2) + 2.0D0*DX) + C(N-1,3) *
     1           C(N,2)**2) / (C(N,2) + DX) - C(N-1,1)*(C(N,2) + DX))/P
C         --------------------------------------------------------
C         | THE EQUATIONS NOW READ                               |
C         |       S(I) + C(I,2)*S(I+1) = C(I,1) , I = 1,...,N-1  |
C         | AND                 S(N)   = C(N,1) .                |
C         | SO CARRY OUT BACK SUBSTITUTION                       |
C         --------------------------------------------------------
            DO 40 I = L,1,-1
 40           C(I,1) = C(I,1) - C(I,2) * C(I+1,1)
C             ---------------------------------------------------------
C             | GENERATE CUBIC COEFFS IN EACH INTERVAL I.E. THE DERIVS.
C             ---------------------------------------------------------
            DO 50 I = 2,NPTS
              DX = X(I) - X(I-1)
              DIVDF1 = (U(K,I) - U(K,I-1)) / DX
              DIVDF3 =  C(I-1,1) + C(I,1) - 2.0 * DIVDF1
              C(I-1,2) =  (DIVDF1 - C(I-1,1) - DIVDF3) / DX
 50           C(I-1,3) =   DIVDF3 / DX**2
            DO 90 JI =1,IPTS
C            -----------------------------------------------
C            |SEARCH FOR THE GRID POINTS THAT BOUND XP(JI) |
C            -----------------------------------------------
              DX = XP(JI)
              IF(DX .LT. (X(1)-TWOU) .OR. DX .GT. (X(NPTS)+TWOU)) THEN
C                DX IS OUTSIDE THE MESH
                 IFAIL = 1
                 RETURN
              END IF
              DO 60 I=2,NPTS
                J = I-1
                IF(DX .LT. X(I))GOTO 70
 60           CONTINUE
              J = NPTS
C           -----------------------------------------------
C           | XP(JI) LIES IN THE INTERVAL (X(J) , X(J+1)) |
C           | SET UP(K,JI) USING THE SPLINE COEFFS IN C . |
C           -----------------------------------------------
 70           DX = DX - X(J)
              UP(K,JI) = C(J,3)
             DO 80 KK = 2,1, -1
 80           UP(K,JI) = UP(K,JI) * DX + C(J,KK)
              UP(K,JI) = UP(K,JI) * DX + U(K,J)
 90    CONTINUE
       RETURN
       END
      SUBROUTINE MVMESH( NEQN, T, HLAST,H, Y, YDOT, YSAVE, NYH, R, ACOR,
     1       RESWK, NRESWK, WKMON, NWKMON, IMON, INLN, HMIN, HMAX)
C**********************************************************************
C  MONITOR ROUTINE FOR REMESHING WITH VARIABLE NO. OF PTS
C
C**********************************************************************
      INTEGER NEQN, NYH, NRESWK, NWKMON, IMON, INLN
      INTEGER ITRACE, IDEV, IOVFLO
      INTEGER NDUM, NQP1, NQ, NINTER, I, N, K, NOLD, NF
      INTEGER NOP, IPMINF, IFAIL, NIP, NRMESH, ICOUNT, NOPUSR, NXFIX
      INTEGER NPTS, NPDE , M, NV, NXI, NVST, MAXNPT, I14, I15, I16
      INTEGER I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I13
      DOUBLE PRECISION T, H, Y(1), YDOT(1), YSAVE(NYH,1), R(1),
     1        ACOR(1), RESWK(NRESWK), WKMON(NWKMON), HMIN, HMAX,
     2        CONST, STPRAT, TOUT, DUNFLO, UROUND, HLAST, DXMESH
      LOGICAL REMESH
      CHARACTER *6 RMTYPE
      COMMON /LSTATS/ NQ, NDUM(5), NINTER
      COMMON /SDEV2/  ITRACE, IDEV
      COMMON /SPSKWK/ I1, I2, I3, I4, I5, I6, I7, I8, I9, I10, I11,
     1                I12, I13, I14, I15, I16
      COMMON /SPSKEL/ NPDE, NPTS, M, NV, NXI, NVST, MAXNPT, NXFIX
      COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOPUSR
      COMMON /MESHTI/ TOUT, REMESH, NRMESH, ICOUNT
      COMMON /MESHTJ/ DXMESH, RMTYPE
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      SAVE   /LSTATS/, /SPSKEL/, /SCONS1/, /SPSKWK/
C-----------------------------------------------------------------------
C CHECK FOR STEP FAILURE , IF SO PASSES CONTROL BACK TO SPRINT
C-----------------------------------------------------------------------
      IF(IMON.LT.0)RETURN
C**   IF(T .GT. 0.46711)ITRACE = 3
      IF( I13 .EQ. I10 )THEN
         IF(REMESH)THEN
C         REMESHING IS BEING TRIED BUT THE WORKSPACE POINTERS HAVE NOT
C         BEEN SET UP
          CALL SERROR(' REMESHING MONITOR - NMVMESH - ERROR -THE SETUP
     1                  ROUTINE SETSKL WAS NOT CALLED WITH REMESH .TRUE.
     2                  BUT  REMESH WAS SET .TRUE. FOR  MVMESH IN COMMON
     3                  BLOCK /MESHTI/ ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
          IMON = -2
          RETURN
        END IF
      END IF
      IF(.NOT.REMESH)GO TO 10
      IF(NRMESH .NE. 0)THEN
         ICOUNT=ICOUNT+1
         IF(ICOUNT.EQ.NRMESH)GO TO 17
         IF(NRMESH .LT. 0)THEN
            CONST = 2.D0 / (NPTS - 1.0D0)
            IF((ICOUNT + NRMESH ) .EQ. 0)GOTO 171
         END IF
      ELSE IF(NRMESH .EQ. 0)THEN
         IF(DABS((T-TOUT)/TOUT).LT.UROUND)GO TO 17
      ENDIF
C NORMAL EXIT TO CONTINUE INTEGRATION
   10 IMON=1
      RETURN
C--------------------------------------------------------------------
C REMESHING FOLLOWED BY INTERPOLATION
C   WORKSPACE ORGANISATION - THE PARTS OF THE WORKSPACE TO DO WITH
C   **********************   REMESHING ARE ORGANISED AS FOLLOWS:
C   RESWK(I8)  ;  THE START OF THE CURRENT MESH
C   RESWK(I9)  ;  THE START OF THE P.D.E. FLUXES.
C   RESWK(I10) ;  THE START OF THE NEW MESH FOUND BY PADRMS. ARRAY XNP.
C   RESWK(I11) ;  THE START OF THE MESH SPACINGS , ARRAY DXP.
C   RESWK(I12) ;  THE START OF THE MONITOR FUNCTION ARRAY FMON.
C   RESWK(I13) ;  THE START OF THE ARRAY CHISTR.
C                 FROM RESWK(I11) ONWARDS THE WORKSPACE IS ALSO USED IN
C                 THE CALL TO THE CUBIC SPLINE ROUTINE INTCUB.
C   RESWK(I14) ;
C     TO          USED FOR REMESHING WITH MATERIAL INTERFACES.
C   RESWK(I16)
C---------------------------------------------------------------------
   17 ICOUNT=0
      IF(ITRACE .GT. 0 .OR. IPMINF .GT. 0)WRITE(IDEV,900)T
 900  FORMAT(' REMESHING PROCESS COMMENCED AT TIME T = ',D12.4)
 171  NOLD = NPTS*NPDE
      NIP=NPTS
C IF NOP=0 THEN NOP CALCULATED BY PADRMS ELSE NOP SET = NIP BY PADRMS
C IN WHICH CASE CONST CRITERION MAY BE OVER OR UNDER SATISFIED
      NOP=0
      IF(NOPUSR.NE.0)NOP=NIP
      IFAIL=0
      CALL EVNRMF( T, NIP, NPDE, RESWK(I8), Y, RESWK(I9), MAXNPT,
     1             RESWK(I12))
      CALL PADRIV( T, NPDE, NIP, RESWK(I8), MAXNPT, CONST, STPRAT,
     1             IPMINF, RESWK(I12), NOP, RESWK(I10),RESWK(I11),
     2             RESWK(I13), IFAIL, RESWK(I14), NXFIX, RESWK(I15))
      IF(IFAIL .NE. 0 .OR.  NF .LT. 0)THEN
         IMON = -2
         RETURN
      ENDIF
C
C       IF NRMESH < 0 OPTION TEST THE NEW MESH TO SEE IF IT IS NEEDED.
C
      IF( RMTYPE .EQ. 'REMSET')THEN
        CALL METEST( RESWK(I8), NIP, RESWK(I10), NOP, I, DXMESH)
        IF(I .EQ. 0)THEN
C          NEW MESH NOT NEEDED
           IMON = 1
           RETURN
        ELSE IF(ITRACE .GT. 0 .OR. IPMINF .GT. 0)THEN
C          NEW MESH IS NEEDED
           WRITE(IDEV,900)T
        END IF
      END IF
C NEW NO. OF PTS (NEEDED FOR COMMON SPSKEL) AND NEW VALUE OF NEQN
      NPTS = NOP
      N    = NPTS*NPDE
      NVST = NPTS*NPDE + 1
      NEQN = N + NV
      IF(ITRACE.GE.1) WRITE(IDEV,901) NOP, (RESWK(I10+I-1),I=1,NOP)
  901 FORMAT(1H ,I3, ' NEW MESH POINTS ',/20(5D12.4/))
C----------------------------------------------------------------------
C INTERPOLATE Y ONTO NEW MESH AND PUT NEW Y FROM R INTO Y USING SPSWOP.
C-----------------------------------------------------------------------
      CALL INTDRV( RESWK(I10), R, NOP, RESWK(I8), Y, NIP, NPDE, IFAIL,
     1             RESWK(I11), RESWK(I15), NXFIX)
      CALL SPSWOP( Y, R, N, NOLD, NV)
C-----------------------------------------------------------------------
C INTERPOLATE YDOT (YDOT IS OVERWRITTEN WITH NEW YDOT FROM R)
C-----------------------------------------------------------------------
      CALL INTDRV( RESWK(I10), R, NOP, RESWK(I8), YDOT, NIP, NPDE,
     1             IFAIL, RESWK(I11), RESWK(I15), NXFIX)
      CALL SPSWOP( YDOT, R, N, NOLD, NV)
      NQP1=NQ + 1
      DO 20 K = 1,NINTER
C ALLOWS FOR THETA METHOD USE (AS WELL AS ADAMS/GEAR)
C MAY NEED MODS FOR OTHER INTEGRATORS????
         IF(NINTER.EQ.4)GO TO 19
         IF(K.LT.NINTER .AND. K.GT.NQP1)GOTO 20
  19     CONTINUE
C        INTERPOLATE YSAVE (AND OVERWRITE WITH NEW YSAVE)
         CALL INTDRV( RESWK(I10), R, NOP, RESWK(I8), YSAVE(1,K), NIP,
     1                NPDE, IFAIL, RESWK(I11), RESWK(I15), NXFIX)
         CALL SPSWOP( YSAVE(1,K), R, N, NOLD, NV)
   20 CONTINUE
      CALL FIXSET( RESWK(I10), NOP, RESWK(I14), NXFIX, RESWK(I15), NF)
C      UPDATE RESWK WITH XNP(NOP)
       DO 25 I=1,NOP
   25     RESWK(I8+I-1) = RESWK(I10+I-1)
       IMON = 3
C      IMON=3 TELLS INTEGRATOR TO RESTART WITH GIVEN YSAVE VALUES
      IF(NF .LT. 0)IMON = -2
      RETURN
       END
C*********************************************************************
      SUBROUTINE SPSWOP( A, B, N, NOLD, NV)
C**********************************************************************
C AUXILIARY ROUTINE TO DO PUT THE CONTENTS OF VECTOR B INTO VECTOR A.
C     THE LAST NV COMPONENTS OF A ARE PASSED FORM THE LAST NV
C     COMPONENTS OF THE NEW VECTOR A , AFTER ALLOWING FOR N > NOLD .
C     THIS ALLOWS REMESH TO BE USED WITH MIXED O.D.E. /P.D.E. PROBLEMS.
C----------------------------------------------------------------------
      INTEGER N, I, NOLD, NV, NPNV, ITRACE, IDEV, NOPNV
      DOUBLE PRECISION A(1), B(1)
      COMMON /SDEV2/ ITRACE, IDEV
      NPNV = N + NV
      NOPNV =NOLD+NV
      IF(ITRACE .GE. 3)THEN
         WRITE(IDEV,100)(A(I), I = 1,NOPNV)
 100     FORMAT(' OLD IS',1X,6D10.3)
         WRITE(IDEV,101)(B(I), I = 1,NPNV)
 101     FORMAT(' NEW IS',1X,6D10.3)
      END IF
      IF(NV .GT. 0)THEN
         DO 5 I = 1,NV
 5         B(N+I) = A(NOLD+I)
      END IF
      DO 10 I = 1,NPNV
 10      A(I) = B(I)
      RETURN
      END
C
      SUBROUTINE PADRIV( TIME, NPDE, NIP, XOP, MAXNPT, CONST, STPRAT,
     1    IPMINF, FMON, NOP, XNP, DXW, CHISTR, IFDUM, XFIX, NFIX, IXFIX)
C***********************************************************************
C
C     VERSION OF REMESH FOR THORNTON/LEEDS SOFTWARE MODULES
C     INTERPOLATION OF SOLUTION VALUES ONTO NEW MESH IS LEFT
C     TO CALLING ROUTINE
C     REMESHING ALGORITHM IS THAT OF KAUTSKY AND NICHOLS
C
C        THIS ROUTINE IS ONLY A SIMPLE DRIVER FOR PADRMS ROUTINE.
C**********************************************************************
      INTEGER NPDE, NIP, MAXNPT, IPMINF, NOIP, IFDUM, IXN, NXFXP1,
     1       NIP1, IX, MANPTI, NOP1, I, NFIX, ITRACE, IDEV
      DOUBLE PRECISION XOP(NIP), XNP(MAXNPT), CHISTR(MAXNPT),
     1        XFIX(1),CONST, STPRAT, TIME, IXFIX(1), RMFMAX,
     2        RMXINT, RINT, FMON(NIP), DXW(MAXNPT)
      COMMON /SDEV2/ ITRACE, IDEV
      IF(NFIX .EQ. 0) THEN
C        NO FIXED POINTS SO USE THE ORIG PADRMS ACROSS WHOLE MESH
         CALL PADRMS( TIME, NPDE, NIP, XOP, MAXNPT, CONST, STPRAT,
     1                IPMINF, FMON, NOP, XNP, DXW, CHISTR, IFDUM)
      ELSE
         IXN = 1
         NXFXP1 = NFIX + 1
C        WRITE(IDEV,13)(IXFIX(I),I=1,NFIX)
C13      FORMAT(' FIXED POINT INDICES ARE ',3D11.3)
         DO 100 I = 1, NXFXP1
            IF(I .EQ. 1)THEN
               NIP1 = IXFIX(1) + 0.1D0
               IX   = 1
            ELSE IF(I .EQ. NXFXP1)THEN
               NIP1 = NIP - IXFIX(NFIX) + 1.1D0
               IX   = IXFIX(NFIX)
            ELSE
               NIP1 = IXFIX(I)- IXFIX(I-1) + 1.1D0
               IX   = IXFIX(I-1)
            END IF
C
C           SET MANPTI SO THAT NEW MESH POINTS ARE
C           IN THE SAME RATIO AS OLD ONES
C
            MANPTI = MAX0( MAXNPT * NIP1 /NIP , NIP1 )
            CONST1 = CONST * (NIP-1.D0) / (NIP1-1.D0)
            NOP1 =  1
            IF(NOP .EQ. 0)NOP1 = 0
            IF(IPMINF .GE. 1)THEN
               WRITE(IDEV,90)NIP1,MANPTI,NOP1, IX, IXN
               WRITE(IDEV,89)(XOP(IX+K-1), K = 1,NIP1)
 89            FORMAT(' OLD MESH',5D11.3)
            END IF
 90         FORMAT(' PADRMS CALLED WITH',I4,' PTS ; MAX=',I4,' NOP=',I5/
     1             ' START POS OF MESH SEG = ',I4,' FOR NEW MESH=',I4)
            CALL PADRMS( TIME, NPDE, NIP1, XOP(IX), MANPTI, CONST1,
     1                   STPRAT, IPMINF, FMON(IX), NOP1, XNP(IXN),
     2                   DXW(IXN), CHISTR(IXN), IFDUM)
C
C            ENSURE INTERFACE POINTS ARE O.K.
C
            XNP(IXN)        = XOP(IX)
            XNP(IXN+NOP1-1) = XOP(IX + NIP1 -1)
            IF(IPMINF .GE. 1)THEN
               WRITE(IDEV,91)NOP1
 91            FORMAT(' NO OF OUTPUT PTS FROM PADRMS = ',I5)
               WRITE(IDEV,92)(XNP(IXN+K-1), K = 1,NOP1)
 92            FORMAT(' NEW MESH',5D11.3)
            END IF
            IXN = IXN + NOP1 - 1
 100      CONTINUE
          NOP = IXN
          IF(IPMINF .GE.1)WRITE(IDEV,110)NOP
 110      FORMAT(' FINAL NO OF OUTPUT POINTS FROM PADRIV =',I5)
        END IF
        RETURN
        END
C**********************************************************************
      SUBROUTINE PADRMS( TIME, NPDE, NIP, XOP, MAXNPT, CONST, STPRAT,
     1                   IPMINF, FMON, NOP, XNP, DXW, CHISTR, IFDUM)
C***********************************************************************
C
C     VERSION OF REMESH FOR THORNTON/LEEDS SOFTWARE MODULES
C     INTERPOLATION OF SOLUTION VALUES ONTO NEW MESH IS LEFT
C     TO CALLING ROUTINE
C     REMESHING ALGORITHM IS THAT OF KAUTSKY AND NICHOLS
C     USING PADDED MONITOR FUNCTION
C     CALCULATES A NON-UNIFORM MESH WHERE THE MESH IS ADAPTED
C     TO FIT THE SOLUTION BEHAVIOUR AND SO IS IDEALLY SUITED
C     FOR BOUNDARY LAYER TYPE BEHAVIOUR. THE METHOD CAN BE USED
C     FOR 2-PT ODE BOUNDARY VALUE PROBLEMS OR FOR PDE PROBLEMS
C     WITH WAVE/FLAME/SHOCK FRONTS .FOR A METHOD OF LINES
C     FOR HYPERBOLIC PROBLEMS ONE SHOULD COMBINE REMESHING
C     WITH SUITABLE SPACE DISCRETISATIONS E.G. 4-PT UPWIND BIAS
C
C
C ADAPTIVE SPACE REMESHING ROUTINE BY R.M.FURZELAND (TRC,13/10/83)
C METHOD:
C -------
C GRADED MESH BASED ON PADDED MONITOR FUNCTION OF KAUTSKY AND NICHOLS
C GIVEN THE EVEN SPACED MESH PTS CHISTR(I), GRADED MESH POINTS
C XNP(CHISTR) ARE FORMED BY LINEAR INTERPOLATION FROM THE
C MAPPING CHI(XOP)=(1/G)*INTEGRAL(M(X)) FROM X0 TO X WHERE THE
C MONITOR FUNCTION M(X) IS A UNIFORMILY DISTR. VARIATION OF F(X) VECTORS
C BASED ON EVNRMF NORM (E.G. L2) AND WEIGHTING
C AND IS NORMALISED USING G=TOTAL INTEGRAL (M(X))
C THE MIN. VALUE OF M(X) (AND HENCE MAX DELX) IS LIMITED BY PEREYRA
C CONCEPT AND, FURTHER, THE ADJACENT MESH RATIO IS CONTROLLED BY
C THE EXTRA PADDING OF M(X) AS DESCRIBED BY KAUTSKY AND NICHOLS
C
C USER PARAMETERS:
C ----------------
C TIME - CURRENT TIME VALUE (ZERO IF PROBLEM DOES NOT INVOLVE TIME)
C NPDE - NO. OF P.D.E.S OR UNKNOWNS
C NIP -  NUMBER OF INPUT SPACE MESH POINTS
C XOP - ARRAY(NIP) OF INITIAL SPACE MESH - MUST BE MONOTONICALLY INCR.
C MAXNPT - MAXIMUM 2ND DIMENSION OF DU AS DEFINED IN DRIVER PROGRAM
C          ALSO USED AS UPPER BOUND FOR NOP
C CONST - INPUT BOUND ON INTEGRAL OF MONITOR OVER EACH STEP. SIZE
C         DEPENDS ON NIP AND MONITOR. TYPICAL VALUE IS 5.0D-1.
C         CONTROLS THE LOCAL TRUNCATION ERROR IF THE MONITOR IS CHOSEN
C         ACCORDINGLY
C         DECREASING C INCREASES THE MESH RATIOS, DECREASING C THEN
C         MESH TENDS TO UNIFORM. THE USER SHOULD CHECK THAT CONST
C         AND THE MAX. SUB-INTEGRAL ACTUALLY PRODUCED ARE APPROX.EQUAL.
C
C STPRAT - INPUT BOUND ON ADJACENT MESH RATIO ( > 1 TYPICALLY 1.5 TO 3)
C          THIS ALSO CONTROLS LOCAL TRUNCATION ERRORS.
C
C IPMINF - CONTROLS PRINT OUT OF MESH INFORMATION. IPMINF=0 NO PRINT,
C          =1 SUMMARY PRINT OF MESH CHARACTERISITICS E.G. MESH RATIOS &
C          MAX. INTEGRAL OF MONITOR PER INTERVAL (APPROX.=INPUT CONST)
C          =2 PRINT OF MONITORS, OLD & NEW MESHES AND MESH SIZES
C FMON -  INPUT ARRAY OF MONITOR VALUES EVALUATED AT XOP(I=1,..,NIP)
C         VALUES ARE CALCULATED ACCORDING TO THE USER SUBROUTINE EVNRMF
C
C    N.B. PADRMS SCALES THE INPUT VALUES BY INTEGRAL OF FMON .
C    ---- PADRMS ALSO CHECKS FMON NON-NEGATIVE AND NOT ALL FMON(I) ZERO.
C         IF IPMINF.GT.0 THEN ON OUTPUT CHISTR CONTAINS MONITOR
C         VALUES AT XNP(I), I=1,...,NOP
C NOP -   IF ZERO ON INPUT THEN NOP CALCULATED BY PROGRAM ELSE NOP SET
C         TO NIP BY PROGRAM
C         OUTPUT CALCULATED NO. OF MESH POINTS NEEDED FOR NEAR OPTIMUM
C         SET OF MESH POINTS XNP
C XNP - OUTPUT ARRAY WITH NEW MESH VALUES I=1,...,NOP
C DXW -   ARRAY(NOP) USED AS WORK SPACE FOR MONITOR G
C                     AND ALSO FOR CHI ARRAY (SEE ABOVE)
C         ON OUTPUT DXW(I)=MESH INTERVALS XNP(I+1)-XNP(I)  I=1,NOP-1
C CHISTR(NOP)   - TREAT AS WORK SPACE
C         IF IPMINF.GT.0 THEN ON OUTPUT CHISTR CONTAINS MONITOR
C         VALUES AT XNP(I), I=1,...,NOP
C IFDUM - ERROR FLAG SET TO 0 ON EXIT IF REMESH CALCULATION WAS OK.
C         USES NAG SOFT/HARD FAIL PRINCIPLES.
C**********************************************************************
      INTEGER NIN, NOUT, ITRACE, IDEV, IFAIL, NIP, NOP, NIP1
     1       ,MAXNPT, ISTAGE, IPMINF, IFDUM, I, J, IND, IOVFLO,
     2        NOP1, NPDE, IMODE, NOM2, I1 ,IDRMAX
      DOUBLE PRECISION XOP(NIP), XNP(MAXNPT), CHISTR(MAXNPT),
     1        RLAM, FLNP, CONST, STPRAT, TIME, DRAT, DRMAX, RMFMAX,
     2        RMXINT, RINT, FMON(NIP), DXW(MAXNPT), SUM,
     3        DUNFLO,UROUND,DUN100
      COMMON/SDEV1/ NIN,NOUT
      COMMON/SDEV2/ ITRACE,IDEV
C IBM M/C PARAMETERS DUNFLO=1.0D-78, UROUND=1.0D-15, IOVFLO=2147483640
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      SAVE /SCONS1/
C ---------------------------------------------------------------------
C VALIDATES INPUT PARAMETERS
C ---------------------------------------------------------------------
      ISTAGE=0
      IFAIL=0
      DUN100=10000.0D0*DUNFLO
      IF(STPRAT.LT.1.0D0)THEN
         WRITE(IDEV,913)STPRAT
         STPRAT=1.5
       ELSE IF(CONST.LT.DUN100)THEN
         WRITE(IDEV,914)DUN100,CONST
C                    OR SHOULD IT BE NOP ......  IN LINE BELOW.
         CONST=2.0/ (NIP -1)
      ENDIF
C
C CHECKS VALIDITY OF INPUT MESH AND MONITOR FUNCTION
C XOP() SHOULD BE MONOTONICALLY INCREASING
C AND FMON SHOULD BE NON-NEGATIVE AND NOT ALL FMON ZERO
      SUM=0.0D0
      IF(FMON(1).LT.0.0)GO TO 999
      DO 2 I=2,NIP
         IF(XOP(I).LE.XOP(I-1))GO TO 999
         IF(FMON(I).LT.0.0D0)GO TO 999
         SUM=SUM+0.5D0*(FMON(I)+FMON(I-1))*(XOP(I)-XOP(I-1))
    2 CONTINUE
      IF(SUM.LT.DUN100)THEN
C        ZERO MONITOR FUNCTION LEAVE THE MESH AS IT IS AFTER
C        PRINTING OUT A WARNING
         NOP = NIP
         DO 942 I = 1,NIP
 942        XNP(I) = XOP(I)
         WRITE(IDEV,941)
 941     FORMAT(' WARNING FROM REMESH ROUTINE MONITOR FUNCTION IS'/
     1          ' ZERO EVERYWHERE - THE MESH IS UNCHANGED')
        IFDUM = 0
        RETURN
       END IF
C
C SCALES INPUT MONITOR FUNCTION BY INTEGRAL OF MONITOR FUNCTION
      DO 3 I=1,NIP
    3    FMON(I)=FMON(I)/SUM
      IF(IPMINF.EQ.0)GO TO 7
      WRITE(IDEV,932)TIME,SUM
C COMPUTES MAX. SUB-INTEGRAL OF INPUT SCALED MONITOR
      RMXINT=0.0D0
      IDRMAX=1
      DO 6 I=2,NIP
         RINT=0.5D0*(FMON(I)+FMON(I-1))*(XOP(I)-XOP(I-1))
         IF(RINT.LE.RMXINT)GO TO 6
         IDRMAX=I
         RMXINT=RINT
    6 CONTINUE
      WRITE(IDEV,936)RMXINT,IDRMAX
C SETS MODE OF WORKING IMODE=0 IS KAUTSKY AND NICHOLS, IMODE=1 IS
C FIXED NO. OUTPUT POINTS NOP=NIP
    7 IMODE=0
      IF(NOP.EQ.0)GO TO 8
      IMODE=1
      NOP=NIP
    8 NIP1=NIP-1
C-----------------------------------------------------------------------
C FORMATION OF PADDED MONITOR FUNCTION IN THE WORK ARRAY DXW ( = G)
C-----------------------------------------------------------------------
C FORWARD SWEEP:
      ISTAGE=1
      IF(IPMINF.GT.1)WRITE(IDEV,910)(XOP(I),FMON(I),I=1,NIP)
      I=1
      IND=0
      RLAM=DLOG(STPRAT)/CONST
    9 J=I+1
   10 IF(J.GT.NIP)GO TO 40
C PADDING FUNCTION DXW(J) FITTED AT POINT XOP(I) AS A FUNCTION OF XOP(J)
      DXW(J) = FMON(I) / (1.0D0+RLAM*(XOP(J)-XOP(I))*FMON(I))
      IF(DXW(J) .LE. FMON(J))GO TO 20
C PADDING FUNCTION DXW GREATER THAN MONITOR FMON
      IND=IND+1
      FMON(J)=DXW(J)
      J=J+1
      GO TO 10
C PADDING FUNCTION < MONITOR FUNCTION
   20 IF(IND.NE.0)GO TO 30
C FITS PADDING FUNCTION AT NEXT POINT XOP(I) AND RESTARTS
      I=I+1
      GO TO 9
C COMPLETION OF PADDING IN INTERVAL XOP(I)...XOP(I+IND)
   30 I=J
      IND=0
      GO TO 9
C REVERSE SWEEP:
   40 IF(IPMINF.GT.1)WRITE(IDEV,907)(XOP(I),FMON(I),I=1,NIP)
      I=NIP
      IND=0
   45 J=I-1
   50 IF(J.LT.1)GO TO 100
C PADDING FUNCTION DXW(J) FITTED AT POINT XOP(I) AS A FUNCTION OF XOP(J)
      DXW(J) = FMON(I) / (1.0D0-RLAM*(XOP(J)-XOP(I))*FMON(I))
      IF(DXW(J) .LE. FMON(J))GO TO 70
C PADDING FUNCTION DXW GREATER THAN MONITOR FMON
      IND=IND+1
      FMON(J) = DXW(J)
      J=J-1
      GO TO 50
C PADDING FUNCTION < MONITOR FUNCTION
   70 IF(IND.NE.0)GO TO 80
C FITS PADDING FUNCTION AT NEXT POINT XOP(I) AND RESTARTS
      I=I-1
      GO TO 45
C COMPLETION OF PADDING IN INTERVAL XOP(I)...XOP(I-IND)
   80 I=J
      IND=0
      GO TO 45
C-----------------------------------------------------------------------
C FORMATION OF NEW MESH EQUI-DISTRIBUTED ACCORDING TO THE PADDED MONITOR
C WORK ARRAY DXW NOW CONTAINS CHI
C-----------------------------------------------------------------------
  100 IF(IPMINF.GT.1)WRITE(IDEV,908)(XOP(I),FMON(I),I=1,NIP)
      ISTAGE=2
C SUM FOR PADDED MONITOR FUNCTION
      SUM=0.0D0
      DO 150 I=2,NIP
         SUM=SUM+0.5D0*(FMON(I)+FMON(I-1))*(XOP(I)-XOP(I-1))
  150 DXW(I)=SUM
      IF(DABS(SUM).GT.DUN100)GO TO 155
      WRITE(IDEV,900)SUM
      GO TO 999
C NORMALISE PARTIAL INTEGRALS CHI( HELD IN DXW) USING SUM
  155 DO 160 I=2,NIP1
  160    DXW(I)=DXW(I)/SUM
      DXW(1)=0.0D0
      DXW(NIP)=1.0D0
      IF(IMODE.EQ.1)GO TO 165
C CALCULATES VARIABLE NUMBER OF OUTPUT POINTS FOR NEAR OPTIMUM MESH XNP
C IF IMODE=0 (ELSE IF IMODE=1 THEN FIXED NOP=NIP)
      ISTAGE=3
      NOP=IDINT(SUM/CONST)+2
      IF(NOP.GT.MAXNPT)THEN
C NO. OUTPUT PTS. NEEDED EXCEEDS MAXNPT SETS NOP=MAXNPT & CONTINUES
          WRITE(IDEV,909)NOP
          NOP=MAXNPT
        ENDIF
  165 IF(IPMINF.GT.0)WRITE(IDEV,940)NIP,NOP,SUM,CONST
C----------------------------------------------------------------------
C GIVEN TRANSFORMATION CHI(XOP) IN DXW(I) THEN PRODUCES GRADED MESH
C CORRESPONDING TO EQUI-SPACED CHISTR VALUES  (INVERSE INTERP.)
C LINEAR INTERP. (NOT SPLINE) TO ENSURE MONOTICITY OF XNP + TESTS XOP
C----------------------------------------------------------------------
      ISTAGE=4
      FLNP=NOP-1
      DO 170 I=1,NOP
  170    CHISTR(I)=(I-1)/FLNP
C XNP(1)=XOP(1) AND XNP)NOP)=XOP(NIP) ENSURED BY CHISTR END VALUES 0 & 1
      I1=1
      CALL INTERX( CHISTR, XNP, NOP, DXW, XOP, NIP, I1, IFAIL, I1)
      IF(IFAIL.NE.0)GO TO 999
      NOP1=NOP-1
      DO 180 I=1,NOP1
         DXW(I)=XNP(I+1)-XNP(I)
         IF(DXW(I).GT.DUN100)GO TO 180
         WRITE(IDEV,912)I
         GO TO 999
  180 CONTINUE
      DXW(NOP)=0.0D0
      IF(IPMINF.EQ.0)GO TO 800
C                              END OF MAIN CALCULATION OF NEW MESH
C --------------------------------------------------------------------
C
C PRINTS AND CALCULATES EXTRA MESH INFO. IF IPMINF>0
C MAX. ADJACENT MESH RATIO
      DRMAX=0.0D0
      NOM2=NOP1-1
      IDRMAX=1
      DO 190 I=1,NOM2
         DRAT=DABS(DXW(I)/DXW(I+1))
         DRAT=DMAX1(DRAT,1.0D0/DRAT)
         IF(DRAT.LE.DRMAX)GO TO 190
         IDRMAX=I
         DRMAX=DRAT
  190 CONTINUE
C RATIO OF MAX. MESH TO EVEN MESH
      RMFMAX=0.0D0
      DO 210 I=1,NOP1
  210    RMFMAX=DMAX1(RMFMAX,DABS(DXW(I)))
      RMFMAX=RMFMAX*FLNP/ (XNP(NOP)-XNP(1))
      WRITE(IDEV,906)IDRMAX,DRMAX,RMFMAX
      IF(IPMINF.LT.2)GO TO 215
      WRITE(IDEV,911)(XOP(I),I=1,NIP)
      WRITE(IDEV,902)NOP,(XNP(I),I=1,NOP)
      WRITE(IDEV,905)(DXW(I),I=1,NOP1)
C----------------------------------------------------------------------
C CALCULATES MONITOR ON NEW MESH POINTS AND MAX INTEGRAL OF MONITOR
C OVER EACH NEW MESH INTERVAL
C----------------------------------------------------------------------
 215  ISTAGE=5
C   LINEAR INTERPOLATION TO FIND MONITOR ON NEW XNP
      CALL INTERX( XNP, CHISTR, NOP, XOP, FMON, NIP, I1, IFAIL, I1)
      IF(IFAIL.NE.0)GO TO 999
C MAX.INTEGRAL OF NEW MONITOR OVER EACH MESH INTERVAL
C FMON ON XNP HELD IN CHISTR
      RMXINT=0.0D0
      IDRMAX=1
      DO 270 I=2,NOP
         RINT=0.5D0*(CHISTR(I-1)+CHISTR(I))*DXW(I-1)
         IF(RINT.LE.RMXINT)GO TO 270
         IDRMAX=I
         RMXINT=RINT
  270 CONTINUE
      WRITE(IDEV,935)RMXINT,IDRMAX
  800 IFDUM=IFAIL
      RETURN
C----------------------------------------------------------------------
C ERROR CONDITION ENCOUNTERED
C----------------------------------------------------------------------
999   IFAIL=ISTAGE
      WRITE(IDEV,903)ISTAGE
C IF SOFT FAIL - RETURNS TO CALLING PROGRAM
      IF(IFDUM.NE.0)GO TO 800
C HARD FAIL - PRINTS USEFUL INFO AND THEN STOPS
      IF(ISTAGE.EQ.0)THEN
         WRITE(IDEV,915)(XOP(I),FMON(I),I=1,NIP)
         WRITE(IDEV,904)
         RETURN
       ELSE
         WRITE(IDEV,910)(XOP(I),FMON(I),I=1,NIP)
      ENDIF
      IF(ISTAGE.LT.4)RETURN
C ISTAGE=4 FAILURE FROM INVERSE INTERPOLATION
      WRITE(IDEV,902)NOP,(XNP(I),I=1,NOP)
      IF(ISTAGE.GT.4) WRITE(IDEV,901)(CHISTR(I),I=1,NOP)
      RETURN
940   FORMAT(' NO. OF INPUT AND OUTPUT POINTS = ',2I4,/
     1' SUM AND CONST ',2D12.4)
936   FORMAT(' MAX.INTEGRAL OF SCALED MONITOR OVER EACH INPUT MESH'
     1       ,' INTERVAL IS ',D12.4,/'  AT POINT ',I5)
935   FORMAT(' MAX.INTEGRAL OF NEW MONITOR OVER EACH NEW MESH INTERVAL'
     1       ,' IS ',D12.4,/'  AT POINT ',I5)
932   FORMAT(/' REMESH ROUTINE CALLED AT TIME = ',D12.4,/
     1' SCALING FACTOR FOR MONITOR IS ',D12.4)
915   FORMAT(' INITIAL MESH AND MONITOR FUNCTION ',/50(2D12.4/))
914   FORMAT(' **REMESH**  INVALID CONST IS LESS THAN= ',D12.4,/
     -       '  CONTINUES WITH A DEFAULT VALUE OF ',D12.4)
913   FORMAT(' **REMESH**  INVALID STPRAT = ',D12.4,/
     -       '  CONTINUES WITH A DEFAULT VALUE OF 1.5')
912   FORMAT(' REMESHING HAS PRODUCED ZERO OR NEGATIVE MESH SPACING'
     -   ,' AT POINT ',I5,'0 CHECK SIZE/SCALING OF MONITOR FUNCTION')
911   FORMAT(' OLD MESH ',/50(5D12.4/))
910   FORMAT(' INITIAL MESH AND SCALED MONITOR FUNCTION ',
     -        /50(2D12.4/))
909   FORMAT(' **REMESH**  NO. OUTPUT POINTS EXCEEDS MAX. ALLOWED ',
     -        I8,/'  SETS NOP=MAXNPT AND CONTINUES ')
908   FORMAT(' PADDED MONITOR AFTER REVERSE SWEEP ',/50(2D12.4/))
907   FORMAT(' PADDED MONITOR AFTER FORWARD SWEEP ',/50(2D12.4/))
906   FORMAT(' MAX ADJACENT MESH SIZE AT POINT ',I5,'  IS ',D12.4,/
     1 ' RATIO MAX/EVEN = ',D12.4)
905   FORMAT(' MESH SIZES ARE ',/50(5D12.4/))
904   FORMAT(' INVALID INPUT MESH OR MONITOR FUNCTION ',
     1   /' XOP(NIP) NOT MONOTONICALLY INCREASING OR ',
     2   /' MONITOR FUNCTION FMON NEGATIVE OR EVERYWHERE ZERO'/)
903   FORMAT(' *** ERROR IN REMESH ROUTINE AT STAGE ',I2,'  ***')
902   FORMAT(' NEW NO. PTS = ',I3,' NEW MESH =',/50(5D12.4/))
901   FORMAT(' NEW MONITOR ',/50(5D12.4/))
900   FORMAT(' MONITOR FUNCTION ZERO EVERYWHERE SO G = ',D12.4)
      END
      SUBROUTINE FIXSET( X, NPTS, XFIX, NXFIX, IXFIX, NF)
C******************************************************************
C   ROUTINE TO DETERMINE WHICH OF THE MESH POINTS ARE FIXED.
C
C   PARAMETER LIST
C   **************
C
C   NPTS    ; THE NUMBER OF MESH POINTS
C   X       ; ARRAY OF LENGTH NPTS CONTAINING THE MESH POINTS
C   XFIX    ; AN ARRAY OF LENGTH NFIX CONTAINING THE MESH POINTS
C             WHICH MUST REMAIN FIXED.
C   IXFIX   ; THE ARRAY THAT IS SET BY THIS ROUTINE AS FOLLOWS
C             IXFIX(I) = K WHEN X(K) = XFIX(I)
C   NF      ; = 1 ON NORMAL EXIT
C           ; = -1 ON EXIT IF NXFIX > 0 AND NO K CAN BE FOUND SUCH THAT
C               X(K) = XFIX(I)
C
C**********************************************************************
      INTEGER NF, NXFIX, IOVFLO, I,J, K, NPTS
      DOUBLE PRECISION X(NPTS), XFIX(1), IXFIX(1), DUNFLO, UROUND,
     1        URTEMP, TEMP
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      SAVE /SCONS1/
C
      URTEMP = UROUND * 1000.0D0
      NF = 1
      IF(NXFIX .EQ. 0)RETURN
      K = 1
      DO 100 I = 1, NXFIX
        IXFIX(I) = 0.0D0
        DO 10 J = K,NPTS
          TEMP  = DABS( XFIX(I) - X(J))
          IF( TEMP .LE. URTEMP)THEN
              IXFIX(I) = J
              K = J + 1
              GOTO 20
          END IF
 10     CONTINUE
 20     CONTINUE
        IF(IXFIX(I) .EQ. 0.0D0)THEN
           NF = -1
           CALL SERROR(' FIXSET ROUTINE CANNOT MATCH MATERIAL INTERFACE
     1     POINT NO (=I1) WITH VALUE (=R1) WITH ANY OF THE SPATIAL MESH
     2     POINTS', 1, 1, I, 0, 1, XFIX(I), 0.0D0)
        END IF
100    CONTINUE
       RETURN
       END
      SUBROUTINE METEST( XOP, NIP, XNP, NOP, ID, DXMESH)
      INTEGER NIP, NOP, ID, JK, JJ, I
      DOUBLE PRECISION XOP(1), XNP(1), DXMESH, D1, D2, D3
      ID = 0
      DO 100 I = 1, NIP
         JJ = MIN0( NIP, I+1 )
         JK = MAX0( 1, I-1 )
         D1 = XOP(JJ) - XOP(I)
         D2 = XOP(I)  - XOP(JK)
         D3 = DMAX1( D1, D2 ) * DXMESH
         D4 = DABS( XOP(I) - XNP(I) )
         IF( D4 .GT. D3)GOTO 200
  100  CONTINUE
       RETURN
  200  ID =1
       RETURN
       END
