C   GENCOL DISCRETISATION MODULE
C   ****************************
C   THIS MODULE DISCRETISES MIXED SYSTEMS OF PARTIAL DIFFERENTIAL
C   EQUATIONS IN ONE SPACE VARIABLE . THE CLASS OF EQUATIONS THAT CAN BE
C   HANDLED IS GIVEN BY
C
C                                    .
C   Q (X,T, U, U  ,  U  , U    , V , V )
C    I      -  - X   -T   - XT   -   -
C
C                          -M   M                               .
C                      =  X   (X  R (X,T, U, U  , U   , U  , V, V ))
C                                  I      -  -X   - XT  -T   -  -   X
C     WHERE
C           U = ( U  , ... , U ) TRANSPOSE  AND  I = 1,... , NPDE.
C           -       1         NPDE
C
C     THE P.D.E. FLUX FUNCTION R(.......) IS ASSUMED TO BE CONTINUOUS
C     W.R.T. THE SPACE VARIABLE R BUT THE FUNCTION  Q(.... ) IS
C     ALLOWED TO BE ONLY PIECEWISE CONTINUOUS PROVIDED THAT THE
C     DISCONTINUITIES ARE PLACED AT SPATIAL MESH POINTS.
C     THE OTHER VECTORS ARE DEFINED IN THE SAME WAY AS U EXCEPT THAT
C                                              .       -
C           V = ( V  , ... , V ) TRANSPOSE AND V  IS SIMILARLY DEFD.
C           -      1          NV               -
C
C                 .
C     WHERE V AND V    ARE THE SOLUTION OF A COUPLED SYSTEM OF
C           -     -            ORDINARY  DIFFERENTIAL EQUATIONS OF
C                              DIMENSION NV.
C     IN THE CASE WHEN NV > 0 THIS SYSTEM OF DIFFERENTIAL EQUATIONS IS
C     ASSUMED TO HAVE THE FORM.
C                .
C        FV ( V, V , XI, UI, UI , RI , UI , UI  , ) = 0
C        --   -  -   -   -   - X  -    - T  - XT
C
C                                             .
C     WHERE ALL THE VECTORS APART FROM V  AND V  ARE OF LENGTH NPDE*NXI.
C                                      -      -
C     AND CONTAIN THE VALUES OF THE P.D.E. VARIABLES U , U  , R, U ,U
C                                                         X       T  XT
C     AT THE SPATIAL O.D.E. /P.D.E. COUPLING POINTS DEINED BY
C     THE VECTOR XI(NXI) .
C
C   THE SPATIAL MESH IS BOUNDED BY A AND B  A < X < B.
C   THE BOUNDARY CONDITIONS HAVE THE FORM
C                                 .                       .
C          B (T) R(X,T, U, U , V, V )  = G (T, U , U , V, V)  AT X = A
C           A    -      -  -X  -  -      -A    -   -X  -  -
C  AND
C                                 .                       .
C          B (T) R(X,T, U, U , V, V )  = G (T, U , U , V, V)  AT X = B.
C           B    -      -  -X  -  -      -B    -   -X  -  -
C
C  WHERE   NOT ALL OF THE FUNCTIONS B AND G ARE SET TO ZERO.
C
C  THE INITIAL CONDITIONS ARE GIVEN BY
C          U (X, 0)  = K (X)  AND V(O) = K
C          -           -1         -      -2                     0
C  THE DISCRETISATION METHOD USED BY THIS MODULE IS BASED ON A C  -
C  COLLOCATION  METHOD AND EVALUATES THE P.D.E. FUNCTIONS INBETWEEN
C  THE USER SUPPLIED MESH POINTS . ANY DISCONTINUITIES IN THE P.D.E.
C  DEFINING FUNCTION  Q MUST THEREFORE BE AT THE USER SUPPLIED
C  BREAK POINTS .
C  REFERENCE BERZINS M. AND DEW P.M. DEPT OF COMPUTER STUDIES
C                  THE UNIVERSITY , LEEDS , LS2 9JT ,REPORT NO 180 .
C
C----------------------------------------------------------------------
C
C   HOW TO USE THIS MODULE
C   **********************
C (1) DECIDE ON THE FORM OF THE SPATIAL DISCRETISATION METHOD TO BE
C     USED. THIS MODULE ALLOWS YOU TO DEFINE A SET OF (NEL + 1) SPATIAL
C     BREAKPOINTS , XBK(I) I = 1,NEL+1 . THESE BREAKPOINTS IN TURN
C     DEFINE NEL SPATIAL ELEMENTS. I.E.
C              XBK(I) =< ELEMENT I =< XBK(I+1).
C     IN THE CASE WHEN THE FUNCTION Q(.....) IN THE P.D.E. DEFINITION
C     HAS DISCONTINIUITIES YOU MUST PLACE A BREAKPOINT AT EACH
C     DISCONTINUITY.
C     SGENCO WILL APPROXIMATE THE SOLUTION TO THE P.D.E. IN THE SPACE
C     DIMENSION BY USING A PIECEWISE CHEBYSHEV POLYNOMIAL BETWEEN EACH
C     PAIR OF BREAKPOINTS. THE DEGREE OF THIS POLYNOMIAL IS SPECIFIED
C     BY DEFINING THE TOTAL NUMBER OF SPATIAL POINTS TO BE USED IN
C     EACH ELEMENT. THIS NUMBER , NPTL , MUST BE GREATER THAN OR
C     EQUAL TO 2. WHEN IT IS 2 THE SPATIAL MESH CONSISTS ONLY OF THE
C     BREAKPOINTS AND A LINEAR POLYNOMIAL IS USED TO APPROXIMATE THE
C     SOLUTION BETWEEN THESE POINTS. IN GENERAL THE DEGREE OF THE
C     POLYNOMIAL USED TO APPROXIMATE THE SOLUTION BETWEEN EACH PAIR
C     OF BREAK-POINTS IS GIVEN BY
C              NPTL   DEGREE OF POLYNOMIAL USED
C               I       I - 1
C     THE IS NO PRE-SET UPPER LIMIT TO THE DEGREE OF POLYNOMIAL THAT
C     CAN BE USED.
C
C
C (2) SET NPDE = NUMBER OF P.D.E.S TO BE SOLVED , MUST BE >= 0 .
C     SET NEL  = NUMBER OF SPATIAL ELEMENTS TO BE USED , MUST BE >= 0.
C     DEFINE AN ARRAY OF BREAKPOINTS IN THE DOUBLE PRECISION ARRAY
C     XBK(IBK) WHERE IBK = NEL + 1 AND
C              XBK(I) < XBK(I+1) , I = 1,NEL
C     SET NPTL  TO THE NUMBER OF MESHPOINTS IN EACH ELEMENT , > 2 .
C     (THE DEGREE OF POLYNOMIAL USED IS THEN (NPTL -1) IN EACH ELEMENT.
C     SET M FOR SPACE CO-ORDINATE TYPE
C         = 0 FOR CARTESIAN,  = 1 FOR CYLINDRICAL, = 2 FOR SPHERICAL
C     SET NV = NUMBER OF O.D.E.S COUPLED TO THE P.D.E.S
C     SET NXI = NUMBER OF SPACE POINTS WHERE THE O.D.E.S ARE COUPLED
C     SET XI(NXI) TO THE VALUES OF THE COUPLING POINTS
C     FOR USE BY THE ROUTINE SGERES WHICH DEFINES THE O.D.E. SYSTEM
C     BEING SOLVED BY THE INTEGRATOR.
C     SET NPTS = NEL * NPTL-1 , THIS IS THE TOTAL NUMBER OF SPATIAL
C     DISCRETISATION POINTS USED BY THIS MODULE.
C     DECLARE A DOUBLE PRECISION ARRAY X, OF DIMENSION NPTS ; X(NPTS)
C     AND PUT X(1) = XBK(1) = A AND X(NPTS) = XBK(NEL+1) = B
C     WHERE A AND B ARE THE LEFT AND RIGHT EDGES OF THE SPTAIAL MESH.
C     THIS ARRAY WILL BE USED TO RETURN TO YOU THE SPATIAL MESH POINTS
C     USED BY THIS SPATIAL DISCRETISATION MODULE.
C     DECLARE A DOUBLE PRECISION ARRAY OF LENGTH NEQN WHERE
C           NEQN = NPDE * NPTS + NV
C     THAT IS USED TO HOLD THE SOLUTION VECTOR . NEQN IS THE NUMBER OF
C     ORDINARY DIFFERENTIAL EQUATIONS THAT MUST BE PASSED ACROSS TO
C     THE SPRINT PACKAGE . THE SOLUTION TO THIS SYSTEM OF ORDINARY
C     DIFFFERENTIAL EQUATIONS THAT IS GENERATED BY DISCRETISING THE
C     CLASS OF P.D.E.S DEFINED ABOVE IS ORDERED IN ,SAY, U(NEQN) AS
C     FOLLOWS.
C       U(I) , I = (J-1) * NPDE + K , K = 1,...,NPDE , J = 1 ,.. ,NPTS
C            CONTAINS THE SOLUTION FOR P.D.E. K AT MESH POINT X(J).
C       U(L) , L = NPDE*NPTS + L1 , L1 = 1,..., NV
C            CONTAINS THE COUPLED O.D.E. COMPONENT V(L1)
C     DEFINE A DOUBLE PRECISION WORKSPACE OF LENGTH IWK WHERE
C       IWK = NPTL*(3*NPTL + 2 + 7*NPDE + NEL) +NXI*(5*NPDE+1) + NV + 2
C     THIS IS THE WORKSPACE THAT MUST BE PASSED ACROSS TO THE SPRINT
C     ROUTINE AS THE WORKSPACE FOR THE O.D.E. RESIDUAL DEFINING ROUTINE
C     SGERES.
C
C (3) CALL THE INITIALISATION ROUTINE INITCC, SEE THE DOCUMENTATION AT
C     THE HEAD OF THIS ROUTINE FOR THE PRECISE DETAILS OF THE CALL.
C     SET TS AND TOUT FOR START AND END INTEGRATION TIMES
C     SET INFORM, WORK ARRAYS AS REQUIRED FOR TIME INTEGRATION
C                - SEE SPRINT DOCUMENTATION
C     CALL THE SPRINT ROUTINE WITH THE RESIDUAL ROUTINE SGERES
C
C (4) PROVIDE A SET OF ROUTINES WHICH DESCRIBE THE PRECISE FORM OF THE
C     P.D.E. TO BE SOLVED. THREE ROUTINES MUST BE PROVIDED AND THE NAMES
C     OF THESE ROUTINES ARE FIXED. THESE ROUTINES ARE:
C
C       SPDEFN : FORMS THE FUNCTIONS Q AND R IN THE P.D.E. DESCRIBED
C                ABOVE. THIS ROUTINE FORMS THE VALUES OF THE FUNCTIONS
C                Q AND R OVER SEVERAL MESHPOINTS SIMULTANEOUSLY.
C                IN FACT AT THE X(NPTL) POINTS IN ONE ELEMENT AT A TIME.
C                THE MESH POINTS USED ARE INTERNALLY
C                GENERATED BY THE DISCRETISATION ROUTINE AND ARE
C                BETWEEN THE USER DEFINED BREAKPOINTS.
C        SBNDR  : FORMS THE FUNCTIONS B AND G ASSOCIATED WITH THE
C                BOUNDARY CONDITIONS FOR THE P.D.E. ABOVE.
C       UINIT  : SUPPLIES THE INITIAL VALUES OF THE P.D.E. PART.
C       SVINIT : SUPPLIES THE INITIAL VALUES OF THE O.D.E. PART.
C       SODEFN : SUPPLIES THE ODE RESIDUAL AS DEFINED BY THE FUNCTION
C                FV ABOVE.
C           NOTE: THE P.D.E. SOLUTION VALUES AT THE COUPLING POINTS
C           PASSED INTO SODEFN ARE DEFINED BY POLYNOMIAL INTERPOLATION
C           ON THE VALUES AT THE P.D.E. SPATIAL MESH POINTS.
C
C EXAMPLE PROBLEM ONE
C *******************
C     THE EASIEST WAY TO DESCRIBE HOW THE PROBLEM DESCRIPTION ROUTINES
C     SHOULD BE WRITTEN IS BY A SIMPLE EXAMPLE. CONSIDER THE FLAME
C     PROPAGATION PROBLEM DEFINED BY
C       Y  = ( Y  )  - Y * KT
C        T      X  X
C  AND                            (NPDE IS 2 )
C       W  = ( W  )  + Y * KT
C        T      X  X
C                      WHERE KT = 14.6* EXP( 4.0*(1.0 - 1./(V + 0.01)))
C  AND   WHERE 0 =< X =< 10.0. AND  0=< T =< 2.8.
C
C     THE LEFT BOUNDARY CONDITION AT X = 0 (LEFT = .TRUE. ) IS GIVEN BY
C         W  = -T / 0.05 , 0 < T =< 0.05
C          X
C            = -COS( PI*0.5 *(T- 0.05) / 0.95)  0.05 =< T < 1.0
C            =  0   FOR T >= 1.0
C      AND
C         Y  = 0
C          X
C     THE RIGHT BOUNDARY CONDITION IS  (LEFT = .FALSE.)
C         W  = Y = 0 FOR ALL TIME T
C          X    X
C      THE ROUTINES SPDEFN AND SBNDR ARE LISTED IN TURN BELOW.
C      THE COMPONENT W OF THE P.D.E. AT THE ITH MESHPOINT IS HELD AS
C      U(1,I) IN THE PACKAGE AND THE COMPONENT Y IS HELD AS U(2,I)
C      THE INITIAL CONDITIONS ARE
C        W(I) = U(1,I) = 1.0D0  I = 1,.., NPTS
C        Y(I) = U(2,I) = 0.0D0  I = 1,...,NPTS
C
C**********************************************************************
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS)
C      DO 10 I = 1,NPTS
C         U(1,I) = 1.0D0
C10       U(2,I) = 0.0D0
C      RETURN
C      END
C
C      SUBROUTINE SPDEFN( T, X, NPTL, NPDE, U, DUDX, UDOT, UTDX, Q, R,
C     1                   NV, V, VDOT, IRES)
C      ROUTINE TO DESCRIBE THE BODY OF THE P.D.E.
C      THE P.D.E. IS WRITEN AS       -M   M
C         Q(X,T,U, U  , U  , U  ) = X   (X  R(X,T,U,U , U , U  ))
C                   X    T    TX                     X   T   TX  X
C      THE FUNCTIONS Q AND R MUST BE DEFINED IN THIS ROUTINE.
C      DEFINITIONS FOR THE MODEL PROBLEM ARE GIVEN
C      NOTE NV = 0 : THERE IS NO O.D.E PART.
C      INTEGER NPDE, NPTL, I, J, NV, IRES
C      DOUBLE PRECISION T, KT, X(NPTL), U(NPDE,NPTL), DUDX(NPDE,NPTL),
C    1         UDOT(NPDE,NPTL), Q(NPDE,NPTL), R(NPDE,NPTL), V, VDOT,
C    2         UTDX(NPDE,NPTL)
C      DO 10 I = 1,NPTL
C         DO 9 J = 1,NPDE
C9           R(J,I) = DUDX(J,I)
C         KT = 14.6D0 * DEXP( 4.0D0 - 4.0D0/(U(2,I) + 0.01D0))
C         Q(1,I) = UDOT(1,I) +  KT * U(1,I)
C         Q(2,I) = UDOT(2,I) - KT * U(1,I)
C10    CONTINUE
C      RETURN
C      END
C
C      SUBROUTINE SBNDR( T, BETA, GAMMA, U, UX, ,UDOT, UTDX, NPDE,
C     1                  LEFT, NV, V, VDOT, IRES)
C      BOUNDARY CONDITIONS ROUTINE
C      INTEGER NPDE, I, J, NV, IRES
C      DOUBLE PRECISION T, BETA(NPDE), GAMMA(NPDE), U(NPDE), HPI
C     1                 UX(NPDE), V, VDOT, UDOT(NPDE), UTDX(NPDE)
C      LOGICAL LEFT
C      HPI = 3.14159265350D0 * 0.5D0 / 0.95D0
C      DO 10 I = 1,NPDE
C         BETA(I) = 1.0D0
C         GAMMA(I) = 0.0D0
C10    CONTINUE
C      IF(LEFT) THEN
C         IF(T .LE. 0.05)THEN
C            GAMMA(2) = -T/0.05D0
C         ELSE IF(T .LE. 1.0)THEN
C            GAMMA(2) = -DCOS(HPI*(T-0.05D0))
C         END IF
C      END IF
C      RETURN
C      END
C
C      DUMMY O.D.E. ROUTINES AS NV IS ZERO
C      SUBROUTINE SODEFN
C      RETURN
C      END
C      SUBROUTINE SVINIT
C      RETURN
C      END
C
C**********************************************************************
C EXAMPLE  PROBLEM 2
C SOLUTION OF MOVING BOUNDARY  PROBLEM BY CO-ORDINATE TRANSFORMATION.
C********************************************************************
C  THIS PROBLEM IS THE ONE PHASE STEFAN PROBLEM (HOFFMAN (1977) ) SEE
C  FURZELAND R.M. A COMPARATIVE STUDY OF NUMERICAL METHODS FOR MOVING
C                 BOUNDARY PROBLEMS. J.I.M.A. (1977) ,26, PP 411 - 429.
C  THE PROBLEM HAS  MELTING DUE TO HEAT INPUT AT THE FIXED
C  BOUNDARY . THE P.D.E. IS DEFINED BY THE EQUATIONS
C         U  =  U        0 < Y < S(T) , 0.1 < T < 1
C          T     YY
C            U  = - EXP(T) , Y = 0
C             Y            .
C            U  =  0  AND  S(T) = - U   ON THE MOVING BOUNDARY Y = S(T).
C                                    Y
C  AND THE INITIAL SOLUTION VALUES AT T = 0.1 ARE GIVEN BY THE ANALYTIC
C  SOLUTION
C            U = EXP(T-Y) - 1 , S(T) = T.
C  THE PROBLEM IS REWRITTEN BY USING THE CO-ORDINATE TRANSFORMATION
C  GIVEN BY  X(T)  =  Y / S(T)  . THE EQUATIONS THEN READ
C                      .
C     S * S * U  - S * S  * X * U   =  U     , X IN (0,1).
C              T                 X      XX
C  WITH THE NEUMANN TYPE BOUNDARY CONDITIONS
C                                               .
C     U  = - EXP(T)  AT X=0  AND  U  = - S(T) * S(T) AT X = 1
C      X                           X
C  AND THE O.D.E. COUPLING POINT EQUATION AT X = 1 WHICH IMPLICITLY
C  DEFINES S(T) IS GIVEN  BY
C     U(1,T) = 0
C  THE EXACT SOLUTION IS NOW DEFINED BY
C     U(X,T) = EXP((T - X*S(T))  , S(T) = T
C
C WE SHALL NOW DETAIL THE ROUTINES NEEDED TO DESCRIBE THIS PROBLEM.
C        PROBLEM DESCRIPTION ROUTINES
C       ******************************
C EXACT SOLUTION
C      SUBROUTINE EXACT( TIME, NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. EXACT VALUES  (IF KNOWN)
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME, P
C      P=DSQRT(2.0D0)*0.5D0
C      DO 10 I = 1,NPTS
C10       U(1,I) = DEXP( TIME * (1 - X(I))) - 1.0D0
C      RETURN
C      END
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME
C      TIME=0.1D0
C      CALL EXACT(TIME,NPDE,NPTS,X,U)
C      RETURN
C      END
C
C      SUBROUTINE SPDEFN(T, X, NPTL, NPDE, U, DUDX, UDOT, UTDX, Q, R,
C     1                  NV, V, VDOT, IRES)
C      PROBLEM INTERFACE  FOR THE MOVING BOUNDARY PROBLEM.
C      INTEGER NPTL, NPDE, NV, I, IRES
C      DOUBLE PRECISION X(NPTL), U(NPDE,NPTL), DUDX(NPDE,NPTL), T,
C    1         V(1), VDOT(1), F(NPDE,NPTL), Q(NPDE,NPTL) ,R(NPDE,NPTL),
C    2         UDOT(NPDE,NPTL), UTDX(NPDE,NPTL)
C      DO 10 I = 1,NPTL
C         R(1,I) = DUDX(1,I)
C         Q(1,I) = V(1)*V(1)*UDOT(1,I) -X(I)*VD(I)*DUDX(1,I) * V(1)
C10    CONTINUE
C      RETURN
C      END
C      SUBROUTINE SBNDR( T, BETA, GAMMA, U, UX, UDOT, UTDX, NPDE,
C     1                  LEFT, NV, V, VDOT, IRES)
C  THIS SUBROUTINE PROVIDES THE LEFT AND RIGHT BOUNDARY VALUES
C  FOR THE MOVING BOUNDARY PROBLEM IN THE FORM.
C           BETA(I) * DU/DX(I) = GAMMA(I)
C  WHERE I = 1,NPDE AND GAMMA IS A FUNCTION OF U,X AND T
C
C      INTEGER NPDE, NV, IRES
C      LOGICAL LEFT
C      DOUBLE PRECISION BETA(NPDE), GAMMA(NPDE), U(NPDE), UX(NPDE)
C    -                  ,T, V(1), VDOT(1), UDOT(NPDE), UTDX(NPDE)
C      BETA(1) = 1.0D0
C      IF(LEFT)THEN
C         GAMMA(1) = -V(1)*DEXP(T)
C      ELSE
C         GAMMA(1) = -V(1)*VD(1)
C      END IF
C      RETURN
C      END
C
C      SUBROUTINE SODEFN(T, NV, V, VDOT, NPDE, NXI, XI, UI, UXI, RI,
C    1                    UTI, UTXI, VRES, IRES)
C      ROUTINE TO PROVIDE RESIDUAL OF COUPLED ODE SYSTEM FOR THE
C      MOVING BOUNDARY PROBLEM.
C      NOTE HOW IRES CAN BE RESET TO COPE WIH ILLEGAL VALUES OF THE
C           MOVING BOUNDARY POSITION V(1).
C      INTEGER NPDE, NXI, NV, IRES
C      DOUBLE PRECISION T, XI(NXI), UI(NPDE,NXI), UXI(NPDE,NXI),
C    1         RI(NPDE,NXI), UTI(NPDE,NXI), UTXI(NPDE,NXI), VRES(NV),
C    2         V(NV), VDOT(NV), TEM
C      VRES(1) = UI(1,1)
C      TEM = 1.0D0
C      IF(IRES .EQ. -1)TEM = 0.0D0
C      IF(V(1) .LT. 0.0D0)IRES = 3
C      VRES(1) = TEM * UI(1,1)
C      RETURN
C      END
C
C      SUBROUTINE SVINIT( V,NV)
C      INTEGER NV
C      DOUBLE PRECISION V(NV)
C      V(1) = 0.1D0
C      ROUTINE TO PROVIDE THE INITIAL VALUES OF COUPLED ODES
C      RETURN
C      END
C***********************************************************************
        SUBROUTINE INITCC(NEQN, NPDE, NPTS, X, U, WK, IWK, M, TS, IBAND,
     1                     ITIME, XBK, IBK, NEL, NPTL, NV, NXI, XI)
C***********************************************************************
C ROUTINE FOR INITIALISATION OF CHEBYSHEV GENERALIZED COLLOCATION METHOD
C
C  PARAMETER LIST
C ----------------
C  NEQN:          EMPTY ON ENTRY, ON EXIT IT CONTAINS THE NUMBER OF
C                 O.D.E.S GENERATED BY THE DISCRETISED FORM OF THE
C                 P.D.E. , GIVEN BY  NPDE*NEL*(NPTL-1) + NPDE + NV.
C
C  NPDE           NUMBER OF PARABOLIC P.D.E.S IN ONE SPACE DIMENSION
C
C  NPTS           NUMBER OF SPATIAL GRID POINTS USED IN M.O.L. SOLUTION.
C                 NOTE THIS SHOULD BE EQUAL TO (NPTL-1)*NEL + 1
C
C  X(NPTS)        EMPTY ARRAY ON ENTRY APART FROM X(1) AND X(NPTS)
C                 WHICH CONTAIN XLEFT AND XRIGHT FOR THE MESH. ON EXIT
C                 THIS ARRAY CONTAINS THE MESH USED IN SEMI-
C                 DISCRETISATION.
C
C  M              =0,1,2 IF CARTESIAN CYLINDRICAL OR SPHERICAL POLARS.
C
C  U(NEQN)        SOLUTION VECTOR  EMPTY ON ENTRY CONTAINS INITIAL
C                 VALUES ON EXIT. THIS ARRAY IS ORDERED AS FOLLOWS.
C                 U(1) - U(NPDE*NPTS)      P.D.E. SOLUTION COMPONENTS.
C                 U(NPDE*NPTS+1) - U(NEQN) O.D.E. COMPONENTS THAT ARE
C                 COUPLED TO THE P.D.E.
C
C  WK(IWK)        REAL WORKSPACE USED TO PASS FOUR MATRICES AND VARIOUS
C                 USEFUL VECTORS TO THE O.D.E.FUNCTION CALL ROUTINE
C                 RESID SEE BELOW FOR A DETAILED DESCRIPTION.
C
C   TS            STARTING LEVEL OF TIME INTEGRATION.
C
C   XBK(IBK)      REAL ARRAY OF BREAK POINTS  IBK = NEL +1  WHERE
C                 NEL IS THE NUMBER OF SPATIAL ELEMENTS.
C                      XBK(1) = XLEFT
C                      XBK(I) =< XBK(I+1)  I = 1,...,NEL .
C                      XBK(IBK) = XRIGHT.
C  NEL            THE NUMBER OF SPATIAL ELEMENTS , >= 1
C
C  NPTL           THE NUMBER OF POINTS IN EACH ELEMENT
C                 I.E. THE DEGREE OF THE APPROX POLYNOMIAL IN
C                 THE ELEMENT + ONE
C  ITIME          THIS MUST BE SET = 1 ON THE CALL OF THIS MODULE
C                 PRIOR TO THE SPRINT PACKAGE BEING CALLED.
C                 ONCE SPRINT HAS RETURNED THIS ROUTINE MAY BE CALLED
C                 WITH ITIME = 2 TO RECOVER THE SPATIAL MESH USED
C                 (THIS IS PLACED IN THE ARRAY X(NPTS) ).
C
C  NV             THE NUMBER OF AUXILARY O.D.E.S THAT ARE COUPLED TO
C                 THE P.D.E.
C  NXI            THE NUMBER OF COUPLING POINTS AT WHICH P.D.E. VALUES
C                 ARE USED TO DEFINE THE O.D.E.S
C
C  XI(NXI)        A VECTOR SPECIFYING THE POSITION OF THESE POINTS.
C---------------------------------------------------------------------
C   EIGHT IMPORTANT PARAMETERS ARE PASSED ACROSS FROM HERE IN
C   COMMON /SCHSZ1/ NNEL, NNPTL, NNPDE, NNPTS, MM, NNV, NNXI, NVST
C
C  NNEL  = NEL
C
C  NNPTL = NPTL
C
C  NNPDE = NPDE     THE NUMBER OF P.D.E.S
C
C  NNPTS = NPTS     THE TOTAL NUMBER OF MESHPOINTS = NEL*(NPTL-1) + 1
C
C  MM = M           = 0,1,2 CARTESIAN, CYLINDRICAL OR SPHERICAL POLAR
C                           CO-ORDINATES IN USE.
C  NNV = NV         NUMBER OF AUXILLARY ODES
C
C  NNXI= NXI        THE NUMBER OF COUPLING POINTS
C
C  NVST             THE STARTING POINT OF THE ODE COMPONENTS IN THE
C                   SOLUTION VECTOR.
C----------------------------------------------------------------------
C
C     DETAILED DESCRIPTION OF WORKSPACE
C    -----------------------------------
C        THE WORKSPACE WK(IWK) IS USED TO PASS ARRAYS AND VECTORS TO
C        THE ROUTINES  RESID AND CHINTR.
C
C        SIZE  :  IWK MUST BE >= 2*NPTL*(NPTL+NEL+1) + 2 + NV +
C                                  NPDE*(7 * (NPTL + NXI) + 8)
C
C     STRUCTURE        NAME IN CODE   PURPOSE IN CODE IS TO HOLD
C     *********
C     WK(1)  -  WK(I2-1)  OMEGA     MATRIX OMEGA FOR MAPPING TO COEFFS
C     WK(I2) -  WK(I3-1)  DU        MATRIX DU FOR FORMING DUDX
C     WK(I3) -  WK(I4-1)  DUTEM     TEMPORARY WORK AREA
C     WK(I4) -  WK(I7-1)  XC        CHEBYSHEV POINTS IN (-1,1)
C     WK(I7) -  WK(I8-1)  BETA      BETA (NPDE,4)       ARRAY FOR B.C.S
C     WK(I8) -  WK(I9-1)  GAMMA     GAMMA(NPDE,4)       ARRAY FOR B.C.S.
C     WK(I9) -  WK(I10-1)  DUDX     DUDX(NPDE,NPTL)     |WORKSPACES
C     WK(I10)-  WK(I11-1)   RT      R(3*NPDE,NPTS)      |USED INSIDE
C     WK(I11)-  WK(I12-1)   QT      Q(3*NPDE,NPTS)      |ROUTINE CRES
C     WK(I12)-  WK(I13-1)   CCR     COEFFS OF CLENSHAW CURTIS RULE OF
C                                   DEGREE NPTL-1.
C     ---------------------THE FOLLOWING PARTS OF THE WORKSPACE ARE USED
C                          IN SEMI-DISCRETISING MIXED O.D.E./P.D.E.
C                          PROBLEMS.
C     WK(I13)-  WK(I14-1) UI        ARRAY UI(NPDE,NXI) USED TO HOLD THE
C                                   PDE COMPONENTS AT THE COUPLING PTS .
C     WK(I14)-  WK(I15-1) UXI       SPACE DERIVS CORRESS TO ARRAY UI.
C     WK(I15)-  WK(I16-1) RI        FLUX CORRESS TO UI ARRAY
C     WK(I16)-  WK(I17-1) UTI       TIME DERIV CORRESS TO UI ARRAY.
C     WK(I17)-  WK(I18-1) UTXI      SPACE DERIV OF ARRAY UTI.
C     WK(I18)-  WK(I19-1) XI        COUPLING POINTS TO LINK PDE TO ODE.
C     WK(I19)-  WK(I20)   VDUM      ARRAY OF ZEROES.
C
C     --------------------THE FOLLOWING TWO PARTS OF WK HOLD THE MESH
C                         POINT INFORMATION USED IN SEMI-DISCRETISATION.
C     WK(I5) -  WK(I6-1)  XBK(IBK)  BREAKPOINTS USED BY THE CODE
C     WK(I6) -  WK(I13)   X(NPTS)   SPATIAL MESH POINTS USED BY THE CODE
C
C   THE ARRAYS BETA AND GAMMA ARE USED IN THE FORMATION OF THE
C   BOUNDARY CONDITIONS. THE ARRAYS DUDX R AND Q HOLD THE TEMPORARY
C   VARIABLES NEEDED IN THE CONSTRUCTION OF THE RESIDUAL OF THE O.D.E.
C   SYSTEM.
C   NOTE ; THE MESH POINTS AND BREAK POINTS ARE STORED AT THE END OF THE
C   *****  WORKSPACE WK SO THAT THE BREAK POINTS MAY BE INCREASED OR
C          DECREASED BY A MESH MODIFICATION ROUTINE. IN SUCH CASES THE
C          MESH POINTS MUST BE RECOMPUTED AS THEY ARE THE CHEBYSHEV
C          POINTS IN  (-1,1) MAPPED ONTO THE INTERVAL. THE CODE TO DO
C          THIS IS GIVEN BY
C          DO 10 I = 1, NEL
C             H1 = XBK(I+1) - XBK(I)
C             H2 = XBK(I=1) + XBK(I)
C             DO 10 J = 1, NPTL
C                IJ = (I-1)*(NPTL -1) + J
C      10        X(IJ) = (XC(J)) * H1 + H2) * 0.5D0
C         WHERE THE VALUES OF XBK AND XC MUST BE EXTRACTED FROM THE
C         WORKSPACE AND THE NEW VALUES OF X PUT IN THE WORKSPACE.
C
C   THE PARAMETERS I2,...,I20  ARE DEFINED BELOW.
C
C**********************************************************************
        INTEGER NPDE, NPTS, IWK, ITIME, IBAND, M, MM, NNPDE, NNPTS
        INTEGER NEL, NPTL, IBK, ITRACE, IDEV, NSQ,  NNEL, NNPTL, NEQN
        INTEGER I2, I3, I4, I5, I6, I7, I8, I9 ,I10, I11, I12, I13, I14
     1         ,I10A, I10B, I11A, I11B, NV, NXI, I15, I16, I17, I18,
     2          I19, IT, NNV, NNXI, NVST, IV, I20, I
        INTEGER K1, K2, K3, K4, JTIMES, ILOC, NNNPTL
        DOUBLE PRECISION X(NPTS), WK(IWK), TS, XBK(IBK), U(1), XI(1), TO
        CHARACTER*6 PDCODE
        COMMON /DISCHK/ PDCODE
        COMMON /MVTRC1/ K1, K2, K3, K4, JTIMES, ILOC, TO
        COMMON /SCHSZ/I2,I3,I4,I5,I6,I7,I8,I9,I10,I10A,I10B,I11,I11A
     1               ,I11B,I12,I13,I14,I15,I16,I17,I18,I19
        COMMON /SCHSZ1/ NNEL, NNPTL, NNPDE, NNPTS, MM, NNV, NNXI, NVST
        COMMON /SDEV2/ ITRACE, IDEV
        COMMON /SCHSZ5/  NNNPTL
        SAVE   /SCHSZ1/, /SCHSZ/, /DISCHK/, /MVTRC1/, /SCHSZ5/, /SDEV2/
         IF(ITIME .LT. 1 .OR. ITIME .GT. 2)THEN
           CALL SERROR(' INITCC DETECTED THAT THE SUPPLIED VALUE OF
     1     ITIME (VALUE = I1 ) IS NOT IN THE RANGE ONE TO TWO ',
     2     1, 1, ITIME, 0, 0, 0.0D0, 0.0D0)
           ITIME = -1
           RETURN
        END IF
        IF(ITIME .EQ. 2 .AND. PDCODE .EQ. 'C0CHEB')GOTO 200
        PDCODE = 'C0CHEB'
        TO = TS
        ILOC = -1
        MM = M
        NNPDE = NPDE
        NNPTS  = (NPTL-1)*NEL + 1
        NNEL  = NEL
        NNPTL = NPTL
        NNV  = NV
        NNXI = NXI
        NVST = NPDE*NPTS + 1
        NEQN = NPDE*NPTS + NV
        IF(NV.GT.0)THEN
          IBAND = NEQN
        ELSE
          IBAND = NPTL * NPDE  -  1
        END IF
        IBK = NEL + 1
        ITIME = 1
        DO 8 I = 1, IWK
 8         WK(I) = 0.0D0
        IF(NPTS .LT. 3)THEN
          CALL SERROR(' INIT FOR C0 COLLOC- VALUE OF NPTS (=I1) IS TOO
     1     SMALL.', 1, 1, NPTS, 0, 0, 0.0D0, 0.0D0)
          ITIME = -1
          RETURN
        END IF
        IF(X(NPTS) .LE. X(1))THEN
          CALL SERROR(' INIT FOR C0 COLLOC-MESH END POINT (=R1) BEFORE
     1    BEGINING (=R2)', 1, 0, 0, 0, 2, X(NPTS),X(1))
          X(1) = XBK(1)
          X(NPTS) = XBK(NEL)
        END IF
        IF(NEL .LT. 1  .OR.  NPTL .LT. 2)THEN
          CALL SERROR(' INIT FOR C0 COLLOC- VALUES OF NEL (=I1) OR NPTL
     1    (=I2) MAY BE ILLEGAL', 1, 2, NEL, NPTL, 0, 0.0D0, 0.0D0)
          ITIME = -1
          RETURN
        END IF
        I = NEL*(NPTL -1) + 1
        IF(I.NE.NPTS)THEN
          CALL SERROR(' INIT FOR C0 COLLOC- VALUES OF NEL AND NPTL FORM
     1    A MESH OF (=I1) POINTS BUT NPTS IS SET TO ( =I2)', 1, 2, I,
     2    NPTS, 0, 0.0D0, 0.0D0)
          ITIME = -1
          RETURN
        END IF
        DO 10 I = 2,IBK
           IF(XBK(I) .LE. XBK(I-1))THEN
             CALL SERROR(' INITCC - BREAKPOINT NO (=I1) HAS VALUE (=R1)
     1       WHICH IS SMALLER THAN OR EQUAL TO BREAKPOINT NO I1-1  WITH
     2       VALUE (=R2)', 1, 1, I, 0, 2, XBK(I), XBK(I-1))
             ITIME = -1
             RETURN
           END IF
 10     CONTINUE
C
           IF(ITRACE.GE.2)WRITE(IDEV,9)(XBK(I), I = 1,IBK)
  9        FORMAT(' BREAKPOINTS ARE',2X,5D11.3)
C
C  FORM INTEGER CONSTANTS FOR WKSPACE MANIPULATION
C
        NSQ = NPTL * NPTL
        IT = NPDE*NXI
        I2 = NSQ + 1
        I3 = I2 + NSQ
        I4 = I3 + NSQ
        I7 = I4 + NPTL
        I8 = I7 + NPDE*4
        I9 = I8 + NPDE*4
        I10 = I9 +NPDE*NPTL
        I11 = I10 + NPDE*NPTL * 3
        I12 = I11 + NPDE*NPTL * 3
        I13 = I12 + NPTL
        I14 = I13 + IT
        I15 = I14 + IT
        I16 = I15 + IT
        I17 = I16 + IT
        I18 = I17 + IT
        I19 = I18 + NXI
        I5  = I19 + NV
        I6  = I5  + NEL + 1
        I20 = I6  + NEL*(NPTL-1) + 1
        I10A  = I10 + NPDE*NPTL
        I10B  = I10A+ NPDE*NPTL
        I11A  = I11 + NPDE*NPTL
        I11B  = I11A  + NPDE*NPTL
        K1 = 1
        K2 = K1 + NEQN
        K3 = K2 + NEQN
        K4 = K3 + NEQN
        IF(I20 .GT. IWK)THEN
          CALL SERROR(' INITCC-ROUTINE WORKSPACE OF SIZE (=I1) IS
     1    SMALLER THAN REQUIRED SIZE (=I2)', 1, IWK, I13, 0,0.0D0,0.0D0)
          ITIME = -1
          RETURN
        END IF
        IV = NPDE*NPTS
        IF(NV .GT. 0)THEN
           IV = NVST
C          COPY ACROSS THE COUPLING POINTS
           DO 20 I = 1,NXI
 20           WK(I18+I-1) = XI(I)
           DO 21 I = 2,NXI
              IF(XI(I).LE.XI(I-1))
     1                            CALL SERROR(' INITCC WARNING THE ODE/
     2        PDE COUPLING POINTS ARE NOT IN STRICTLY INCREASING ORDER'
     3        , 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
 21        CONTINUE
        END IF
C
        CALL CSET(NNPDE,NNPTS, TS, U, WK(I6), WK, WK(I2), WK(I5),NNEL,
     1            NNPTL, WK(I4), WK(I12) , XBK, IBK, WK(I3), U(IV), NV)
C
 200    DO 22 I = 1,NPTS
           X(I) = WK(I6+I-1)
 22     CONTINUE
        NNNPTL = NPTL
        RETURN
        END
        SUBROUTINE CSET( NPDE, NPTS, TSTART, U, X, OMEGA, DU, XBK,
     1                   NEL, NPTL, XC, CCR, XBH, IBK, DUTEM, V, NV)
C***********************************************************************
C       FORTRAN FUNCTIONS USED: DSIN DCOS .
C***********************************************************************
        INTEGER NPTS, NPDE, NEL,NPTL, IDEV, ITRACE, NM1, I, J, K, NV,
     1          IJ, IBK, ITEM, NT, NTP1
        DOUBLE PRECISION TSTART, U(NPDE,NPTS),OMEGA(NPTL,NPTL), V(1),
     1          XC(NPTL), CCR(NPTL), DU(NPTL,NPTL), XBK(IBK), X(NPTS),
     2          DUTEM(NPTL,NPTL), XBH(IBK), TEMP, SINT, SUM, PI, H1, H2
     3          , CCRULE
        COMMON /SDEV2/ITRACE,IDEV
        COMMON /SCHSZ6/  CCRULE(50)
        SAVE  /SCHSZ6/
C
C  FORM  CONSTANTS FOR WKSPACE INITIALISATION
C
        NM1 = NPTL -1
        PI = 3.1415926535897930D0
C
C  FORMATION OF GRID AND INITIAL VALUES OF U
C
        DO 10 I = 1,NEL
           H1   = XBH(I+1) - XBH(I)
           H2   = XBH(I+1) + XBH(I)
           XBK(I) = XBH(I)
           DO 10 J = 1,NPTL
               IJ = (I-1)*NM1+J
               IF(I.EQ.1)   XC(J) = DCOS( PI * (J-NPTL) / NM1 )
                            X(IJ) = (XC(J)*H1 + H2 ) * 0.5D0
               IF(J.EQ.1)   X(IJ) = XBH(I)
               IF(J.EQ.NPTL)X(IJ) = XBH(I+1)
10      CONTINUE
        CALL UINIT(NPDE, NPTS, X, U)
        IF(ITRACE .GE. 1)THEN
          DO 11 IJ = 1,NPTS
             WRITE(IDEV,101)(U(K,IJ), K = 1,NPDE)
 101         FORMAT(' INITIAL PDE SOLUTION IS ',5D11.3)
11        CONTINUE
        END IF
        XBK(IBK) =  XBH(IBK)
        XC(1)    = -1.0D0
        XC(NPTL) =  1.0D0
        IF(ITRACE .GE. 1)WRITE(IDEV,103)NPTS,NEL,NV
103     FORMAT(' NPTS-',I3,' NEL=',I3,' NV',I3)
C
C  FIND THE INITIAL VALUES OF THE COUPLED O.D.E. COMPONENTS.
C
        IF(NV .GT. 0)THEN
           CALL SVINIT(V,NV)
           IF(ITRACE .GE. 1)WRITE(IDEV,102)(V(I),I = 1,NV)
 102       FORMAT('   INIT ODE SOL IS',D11.3)
        END IF
C
C  FORM THE MATRIX OMEGA
C
        DO 20 J = 1,NPTL
           DO 20 I = 1,NPTL
20          OMEGA(I,J) = 2.D0 * DCOS( PI*(I-1)*(NPTL-J)/NM1 ) / NM1
C
C   MODIFY EDGES OF OMEGA AND FORM EDGES OF INTERMEDIATE DU MATRIX
C
        ITEM = 1
        DO 30 I = 1,NPTL
           OMEGA(I,1)    = OMEGA(I,1)    * 0.5D0
           OMEGA(1,I)    = OMEGA(1,I)    * 0.5D0
           OMEGA(NPTL,I) = OMEGA(NPTL,I) * 0.5D0
           OMEGA(I,NPTL) = OMEGA(I,NPTL) * 0.5D0
           DUTEM(I,1)    = 0.0D0
           DUTEM(1,I)    = -((I-1)**2*ITEM)
           DUTEM(NPTL,I) =  ((I-1)**2)
30      ITEM = -ITEM
C
C FINISH FORMING REST OF INTERMEDIATE DU MATRIX THAT IS HELD IN DUTEM.
C
        IF(NPTL .GT. 2)THEN
           DO 50 I = 2,NM1
              TEMP = PI * (I-NPTL) / NM1
              SINT=DSIN(TEMP)
              DO 40 J = 2,NM1
 40              DUTEM(I,J)  = DSIN( TEMP*(J-1) ) / SINT * (J-1)
 50        DUTEM(I,NPTL) = 0.0D0
        END IF
C
C  FORM FULL DU BY MATRIX MULTIPLICATION
C
        DO 60 I = 1,NPTL
           DO 60 J = 1,NPTL
              DU(I,J) = 0.0D0
              DO 60  K = 1,NPTL
                 DU(I,J) = DU(I,J) + DUTEM(I,K) * OMEGA(K,J)
 60     CONTINUE
C
C        CALCULATE THE COEFFS OF THE CLENSHAW CURTIS RULE
C
         NT = NM1/2
         IF((2*NT).NE.NM1)NT = (NM1-1)/2
         NTP1 = NT + 1
         SUM = 0.0D0
         DO 100 I = 1,NPTL
            TEMP = 0.5D0
            CCR(I) = 0.0D0
            DO 90 K = 1,NTP1
               IF(K.EQ.NTP1.AND.((2*NT).EQ.NM1))TEMP = 0.5D0
               CCR(I) = CCR(I) + DCOS(2.0D0*(I-1)*(K-1)*PI/NM1) * TEMP
     1                           /  (4.0D0*(K-1)**2 - 1.0D0)
 90         TEMP = 1.0D0
            IF(I.EQ.1.OR.I.EQ.NPTL)TEMP = 0.5D0
            CCR(I) = CCR(I) * (-4.0D0) * TEMP / NM1
            SUM = SUM + CCR(I)
 100     CONTINUE
         IF(ITRACE.GE.1)WRITE(IDEV,112)SUM,CCR(1)
        DO 125 I = 1,NPTL
125        CCRULE(I) = CCR(I)
         DO 110 I = 2,NM1
            CCR(I) = CCR(I) / CCR(1)
            IF(ITRACE.GE.3)WRITE(IDEV,111)CCR(I)
 110      CONTINUE
 111     FORMAT(' COEFF OF CC RULE=',D11.3)
 112     FORMAT(' SUM AND FIRST COEFF ARE',2D11.3)
         RETURN
C
C-----------END  OF  CSET ROUTINE---------------------------------------
C
         END
        SUBROUTINE SGERES( NEQN, T, U, UDOT, RESD, IRES, WK, IWK)
C***********************************************************************
C
C THIS IS THE CHEBYSHEV GLOBAL ELEMENT ROUTINE TO EVALUATE THE
C RESIDUAL OF THE IMPLICIT SET OF O.D.E.'S DEFINED BY
C
C        RESIDUAL  =  A(U,T)*DU/DT  -  F(U,T)
C
C PARAMETER LIST
C----------------
C  NEQN(1) = N     NUMBER OF O.D.E.S IN TIME.
C  M               POLAR PARAMETER FOR P.D.E.S.
C  NPDE            NUMBER OF PARABOLIC EQUATIONS
C  NPTS            NUMBER OF SPATIAL GRID POINTS
C  T               CURRENT TIME INTEGRATION LEVEL , > 0.0
C  U(N)            CURRENT SOLUTION VECTOR
C  RESD(N)          VECTOR WHICH WILL CONTAIN THE RESIDUAL ON EXIT
C  UDOT(N)         CURRENT ESTIMATE OF DU/DT
C  WK(IWK)         WORKSPACE - DEFINED IN INITAL
C  IRES            INDICATOR FOR RESIDUAL ROUTINE.
C                  ON ENTRY = -1 THEN EVALUATE THOSE PARTS OF THE
C                                RESIDUAL ONLY WHICH INVOLVE UDOT
C                           =  0 EVALUATE THE FULL RESIDUAL.
C
C  ONLY  RES(N) IS ALTERED ON EXIT : IT CONTAINS THE CURRENT RESIDUAL
C***********************************************************************
        INTEGER NEQN(1), IRES, IWK, N, J, K, IJ, I
        INTEGER NPTS, NPDE, M, IDEV, ITRACE, NEL, NPTL, I10A, I11A,
     1          I2, I3, I4, I5, I6, I7, I8, I9, I10, I11, I12, I11B,
     2          I10B, I13, I14, I15, I16, I17, I18, I19, NV, NXI,
     3          NVST, IFL, IV, ITYPE, IBK, IR
        DOUBLE PRECISION U(1), RESD(1), WK(IWK), T, UDOT(1)
        CHARACTER*6 PDCODE
        COMMON /DISCHK/ PDCODE
        COMMON /SDEV2/  ITRACE,IDEV
        COMMON /SCHSZ1/ NEL, NPTL, NPDE, NPTS, M, NV, NXI, NVST
        COMMON /SCHSZ/I2,I3,I4,I5,I6,I7,I8,I9,I10,I10A,I10B,I11,
     1                I11A,I11B,I12,I13,I14,I15,I16,I17,I18,I19
        SAVE   /SCHSZ1/, /SCHSZ/, /DISCHK/
C
        IF(PDCODE .NE. 'C0CHEB')THEN
          CALL SERROR(' C0CHEB-RES ROUTINE ERROR- THE SETUP ROUTINE
     1    INITCC WAS NOT CALLED BEFORE SPRINT WAS ENTERED', 1, 0, 0, 0,
     2    0, 0.0D0, 0.0D0)
          IRES = 3
          RETURN
        END IF
C
        N = NEQN(1)
        DO 20  J = 1,N
20         RESD(J) = 0.0D0
        IBK = NEL+1
        IV = NPTS * NPDE
        IF(NV .GT. 0)THEN
          IV = NVST
          IF(NXI .GT. 0)THEN
C          GENERATE THE SOLUTION VALUES SPACE DERIVS AND FLUXES AT THE
C                                COUPLING POINTS
           ITYPE = 3
           CALL INTCHE( NXI, WK(I18), WK(I13), ITYPE, U, NPTS, NPDE,
     1                  NEL, NPTL, WK, WK(I10), WK(I5), IBK, IFL, NV,
     2                  U(IV), UDOT(IV), WK(I11), T, IR)
           IF( IR .NE. 1)GOTO 60
C
           IF(IRES .EQ. -1)THEN
C             GENERATE THOSE PARTS OF FLUXES THAT DEPEND ON TIME DERIVS
              ITYPE = 4
              DO 30 J = 1,NXI
                 DO 30 K = 1,NPDE
                    IJ = NPDE*(J-1) + K -1
 30                 WK(I16+IJ) = WK(I15+IJ)
              CALL INTCHE( NXI, WK(I18), WK(I13), ITYPE, U, NPTS, NPDE,
     1                     NEL, NPTL, WK, WK(I10), WK(I5), IBK, IFL, NV,
     2                     U(IV), WK(I19), WK(I11), T, IR)
              IF(IR.NE.1)GOTO 60
              DO 40 J = 1,NXI
                 DO 40  K = 1,NPDE
                    IJ = NPDE*(J-1) + K - 1
 40                 WK(I15+IJ) = WK(I16+IJ) - WK(I15+IJ)
           END IF
C
C          GENERATE TIME DERIV VALUES AND THEIR SPACE DERIVS AT THE
C                                COUPLING POINTS.
           ITYPE = 2
           CALL INTCHE( NXI, WK(I18), WK(I16), ITYPE, UDOT, NPTS, NPDE,
     1                  NEL, NPTL, WK, WK(I10), WK(I5), IBK, IFL, NV,
     2                  U(IV), UDOT(IV), WK(I11), T, IR)
           IF(IR.NE.1)GOTO 60
          END IF
C
C          CALL THE ROUTINE TO DEFINE THE AUXILLARY ODE RESIDUAL.
C
           CALL SODEFN( T, NV, U(IV), UDOT(IV), NPDE, NXI, WK(I18),
     1                  WK(I13), WK(I14), WK(I15), WK(I16) ,WK(I17),
     2                  RESD(IV), IRES)
           IF( IABS(IRES) .NE. 1) RETURN
        END IF
C       CALL THE CO COLLOCATION DISCRETISATION ROUTINE
        IR = 1
        CALL CRES( NPDE, NPTS, T, U, RESD, UDOT, M, WK(I6), WK, WK(I2),
     1          WK(I5), WK(I7), WK(I8), WK(I9), WK(I10), WK(I11), NEL,
     2          NPTL,WK(I4), WK(I12) ,IRES, WK(I10A), WK(I11A),WK(I11B),
     3          WK(I10B), NV, U(IV), UDOT(IV), WK(I19))
C
          DO 50 I = 1, N
             RESD(I) = -RESD(I)
C            IF(ITRACE.GE.3)WRITE(IDEV,49)U(I),UDOT(I),RESD(I)
C49          FORMAT(' UI=',D11.3,' UDOTI =',D11.3,'  RESI=',D11.3)
 50       CONTINUE
        RETURN
 60     IRES = IR
        RETURN
C
C---------------------------END OF SGERES-----------------------------
C
        END
        SUBROUTINE CRES( NPDE, NPTS, T, U, RES, UDOT, M, X,
     1                   OMEGA, DU, XBK, BETA, GAMMA, DUDX, R, Q,
     2                   NEL, NPTL, XC, CCR, IRES, RT, QT, UDT, UTDX,
     3                   NV, V, VDOT, VDUM)
C**********************************************************************
C       CHEBYSHEV C0 COLLOCATION ROUTINE
C       REFERENCE  ; C0 CHEBYSHEV METHODS FOR PARABOLIC EQUATIONS.
C                    BY M. BERZINS AND P.M. DEW
C**********************************************************************
C
        INTEGER NPTS,NPDE,M,NEL,NPTL, IDEV,ITRACE, NM1, I,J,K,KJ,
     1          IJ,JJ,JK,II, IRES, IT, IOVFLO, NV, IV, IZ(6), IK
        DOUBLE PRECISION T,U(NPDE,NPTS), RES(NPDE,NPTS),
     2          OMEGA(NPTL,NPTL), DU(NPTL,NPTL), XBK(1), BETA(NPDE,4),
     3          GAMMA(NPDE,4), DUDX(NPDE,NPTL), R(NPDE,NPTL),
     4          Q(NPDE,NPTL), XC(NPTL), CCR(NPTL),UDOT(NPDE,NPTS),
     5          RT(NPDE,NPTL), QT(NPDE,NPTL), V(1), VDUM(1),
     6          UDT(NPDE,NPTL), X(NPTS), UTDX(NPDE,NPTL), VDOT(1)
        DOUBLE PRECISION D1, TWOU, TEM, MP1, H, SAVEL, SAVER, SFIRST,
     1          CCLF, CCRT
        COMMON /SCONS1/ D1, TWOU, IOVFLO
        COMMON/SDEV2/ITRACE,IDEV
        SAVE
        NM1 = NPTL - 1
        IT = 1
        IV = MAX0(1,NV)
        MP1 = 1.0D0
        DO 100 I = 1, NEL
           JJ = (I-1)*NM1
           IJ = JJ + 1
           H  = 2.0D0 / (XBK(I+1) - XBK(I))
           DO 9 IK = 1,6
  9           IZ(IK) = 1
C**********************************************************************
C          MAIN LOOP OVER ALL THE SPATIAL ELEMENTS START BY FORMING THE
C          SPACE DERIVS OF U AND UDOT IN DUDX AND UTDX RESPECTIVELY.
C**********************************************************************
           DO 10 K = 1,NPDE
              DO 10 II = 1,NPTL
                 DUDX(K,II) = 0.0D0
                 UTDX(K,II) = 0.0D0
                 DO 10 J = 1,NPTL
                    UTDX(K,II) = UTDX(K,II) + DU(II,J)*UDOT(K,JJ+J) * H
                    DUDX(K,II) = DUDX(K,II) + DU(II,J) *  U(K,JJ+J) * H
10         CONTINUE
C
           IF(I .EQ. 1)THEN
C             SAVE THE VALUES NEEDED FOR LEFT BOUNDARY CONDITIONS
              DO 11 J = 1,NPDE
                 BETA(J,3) = DUDX(J,1)
 11              BETA(J,4) = UTDX(J,1)
           END IF
           IF(I. EQ. NEL)THEN
C             SAVE THE VALUES NEEDED FOR RIGHT BOUNDARY CONDITIONS
              DO 12 J = 1,NPDE
                 GAMMA(J,3) = DUDX(J,NPTL)
 12              GAMMA(J,4) = UTDX(J,NPTL)
           END IF
C----------------------------------------------------------------------
C       EVALUATE THE FUNCTIONS Q AND R IN THIS ELEMENT
C----------------------------------------------------------------------
           CALL SPDEFN( T, X(IJ), NPTL, NPDE, U(1,IJ), DUDX,
     1                  UDOT(1,IJ), UTDX, Q, R, IV, V, VDOT, IZ(1))
           IF(IRES .EQ. -1)THEN
C            FORM THOSE PARTS OF Q AND R INVOLVING UDOT.
             DO 16 J = 1,NPDE
                DO 16 K = 1,NPTL
                   UTDX(J,K) = 0.0D0
 16                UDT (J,K) = 0.0D0
             CALL SPDEFN( T, X(IJ), NPTL, NPDE, U(1,IJ), DUDX,
     1                    UDT, UTDX, QT, RT, IV, V ,VDUM, IZ(2))
             DO 17 J = 1,NPDE
                DO 17 K = 1,NPTL
                   Q(J,K) = Q(J,K) - QT(J,K)
                   R(J,K) = R(J,K) - RT(J,K)
 17          CONTINUE
           END IF
           IF(M .GT. 0)THEN
C             MODIFY Q FUNCTION IF POLAR CO-ORDINATES
              KJ = 1
              IF(X(IJ) .LE. TWOU)THEN
                 MP1 = 1.0D0 + M
                 KJ =2
                 DO 18 K = 1,NPDE
C                   R(K,1) = 0.0D0
 18                 Q(K,1) = Q(K,1)/(M+1)
              END IF
              DO 20 J = KJ,NPTL
                 DO 20 K = 1,NPDE
                    Q(K,J) = Q(K,J) - R(K,J)*M/X(JJ+J)
20            CONTINUE
           END IF
C-----------------------------------------------------------------------
C      SET UP SAVEL AND SAVER  FOR THE BOUNDARY AND INTERFACE CONDITIONS
C      AND FORM DRDX  BY OVERWRITING DUDX
C-----------------------------------------------------------------------
           KJ = MAX0(2,I)
           JK = MIN0(NEL,I+1) + 1
           SAVEL = 1.0D0 / (XBK(KJ)+XBK(I+1)-XBK(KJ-1)-XBK(I))
           SAVER = 1.0D0 / (XBK(JK)+XBK(I+1)-XBK(JK-1)-XBK(I))
           IF(I.EQ.1)SFIRST = SAVEL
           DO 27 K = 1,NPDE
              DO 27 II = 2,NM1
                 DUDX(K,II) = 0.0D0
                 DO 27 J = 1,NPTL
                    DUDX(K,II) = DUDX(K,II) + DU(II,J)*R(K,J)
27         CONTINUE
C-----------------------------------------------------------------------
C    FORM THE RESIDUAL AND THE INTERFACE CONDITIONS
C-----------------------------------------------------------------------
           DO 90 J = 1, NPDE
              CCLF = 0.0D0
              CCRT = 0.0D0
              DO 80 K = 2,NM1
                 RES(J,JJ+K) = Q(J,K) - DUDX(J,K)*H
C                CLENSHAW CURTIS COMPONENTS FOR BOUNDARIES
                 CCLF = CCLF + CCR(K)*(DUDX(J,K)*(1.0D0-XC(K))-R(J,K))
                 CCRT = CCRT + CCR(K)*(DUDX(J,K)*(XC(K)+1.0D0)+R(J,K))
80            CONTINUE
              JK = IJ + NM1
              TEM  =  R(J,1) + R(J,NPTL)
              RES(J,IJ) = RES(J,IJ) +
     1                    (Q(J,1)   *2.0/H - TEM + CCLF) * SAVEL
              RES(J,JK) = (Q(J,NPTL)*2.0/H + TEM + CCRT) * SAVER
90         CONTINUE
C
C           TEST TO SEE IF ILLEGAL SOLUTION VALUES HAVE BEEN FOUND.
C
           DO 95 IK = 1,2
              IF( IZ(IK) .NE. 1)THEN
                IRES = IZ(IK)
                GOTO 202
              END IF
95         CONTINUE
100     CONTINUE
C**********************************************************************
C    EVALUATE THE FUNCTIONS BETA AND GAMMA AT THE BOUNDARY CONDITIONS
C**********************************************************************
C
        CALL SBNDR( T, BETA(1,1), GAMMA(1,1), U(1,1), BETA(1,3),
     1          UDOT(1,1), BETA(1,4), NPDE, .TRUE., IV, V, VDOT, IZ(3))
        CALL SBNDR( T, BETA(1,2), GAMMA(1,2), U(1,NPTS), GAMMA(1,3),
     1      UDOT(1,NPTS), GAMMA(1,4),NPDE, .FALSE., IV, V, VDOT, IZ(4))
C
        IF(IRES .EQ. -1)THEN
C          ISOLATE PARTS OF GAMMA(J,I) I = 1,2  DEPENDING ON VDOT,YDOT.
           CALL SBNDR( T, BETA(1,1), BETA(1,4),  U(1,1),    BETA(1,3),
     1          UDT(1,1), UTDX(1,1), NPDE, .TRUE. , IV, V, VDUM, IZ(5))
           CALL SBNDR( T, BETA(1,2), GAMMA(1,4), U(1,NPTS), GAMMA(1,3),
     1          UDT(1,2), UTDX(1,2), NPDE, .FALSE., IV, V, VDUM, IZ(6))
           DO 8 J = 1,NPDE
             GAMMA(J,1) = GAMMA(J,1) -  BETA(J,4)
 8           GAMMA(J,2) = GAMMA(J,2) - GAMMA(J,4)
        END IF
C                             PROCESS THE BOUNDARY CONDITIONS
        DO 200   J = 1,NPDE
C          L.H.--BOUNDARY CONDITION IS PROCESSED
           RES(J,1) = MP1 * (RES(J,1)*BETA(J,1)*2.0D0
     1                    +  GAMMA(J,1)*4.0D0/CCR(1) * SFIRST)
C          R.H.---BOUNDARY CONDITION IS PROCESSED
           RES(J,NPTS) = RES(J,NPTS) * BETA(J,2) * 2.0D0
     1                 - GAMMA(J,2) * 4.0D0/CCR(1) * SAVER
200     CONTINUE
        DO 201 IK = 3,6
           IF( IZ(IK) .NE. 1)IRES = IZ(IK)
201     CONTINUE
202     CONTINUE
        RETURN
C-------END OF C0RES----------------------------------------------------
C
         END
         SUBROUTINE CHINTR( XP, UP, NP, X, U, NPTS, NPDE, IFLAG,
     1                      ITYPE, IWK, WK, UDOT, T)
C********************************************************************
C        ROUTINE TO PROVIDE VALUES OF THE SOLUTION AND POSSIBLY THE
C        FIRST DERIV IN SPACE AND THE FLUX ON THE MESH XP(NP).
C
C        INPUT VALUES
C       --------------
C        NPDE     NO OF PARABOLIC EQUATIONS
C        NPTS     NO OF SPATIAL MESH POINTS USED IN TIME INTEGRATION.
C        NP       THE NUMBER OF SPATIAL INTERPOLATION POINTS
C        XP(NP)   ARRAY CONTAINING THE SPATIAL INTERPOLATION POINTS
C                 WE ASSUME THAT
C                    XP(I) <  XP(I+1)  ,  I = 1,...,NP-1
C        UP(NPDE,NP,ITYPE)  EMPTY ARRAY FOR THE INTERPOLATED VALUES AT
C                           THE CURRENT TIME LEVEL
C        U(NPDE,NPTS) THE CURRENT SOLUTION VECTOR COMPUTED BY THE ODE
C                  TIME INTEGRATOR
C        IFLAG          ERROR FLAG       = 0 ON SUCCESSFUL RETURN
C                                        = 1 IF EXTRAPOLATION TRIED.
C                                        = 2 IF WORKSPACE NOT INITIAL
C                                               ISED ON ENTRY BY INITCC
C        ITYPE = 1  ONLY THE SOLUTION IS OUTPUT IN THE ARRAY UP
C                   UP(J,K,1) HOLDS U(XP(K),T) FOR PDE J
C                2  AS FOR 1 BUT THE FIRST DERIV IS ALSO OUTPUT.
C                   UP(J,K,2) HOLDS D/DX U(XP(K),T).
C                3  AS FOR 2 BUT THE FLUX IS ALSO RETURNED IN UP.
C                   UP(J,K,3) HOLDS THE FLUX AT THE POINT XP(K) .
C                INCORRECT VALUES OF ITYPE ARE HANDLED BY SETTING
C        ITYPE = MAX0(1,MIN0(3,ITYPE))
C
C
C        WK(IWK) THE WORKSPACE USED BY THE CHEBYSHEV METHOD.
C
C        UDOT(1) ARRAY CONTAINING THE TIME DERIVS.
C
C        T       THE  CURRENT TIME
C    NOTE THE PARAMETERS UDOT AND T ARE ONLY USED IF IT = 3
C    **** OTHERWISE DUMMY PARAMETERS MAY BE PASSED ACROSS.
C
C**********************************************************************
          INTEGER NPDE, NP, IBK, IFLAG, NEL, NPTL, IC, NPTS, ITYPE, IWK,
     1            ID, IA, I5, IB, I9, I19, I11, I10,
     2            NNPDE, NNPTS, MM, NV, NXI, NVST, IV
          DOUBLE PRECISION XP(NP), UP(NPDE,NP,ITYPE), U(1), X(NPTS),
     1                     WK(IWK), UDOT(1), T
          CHARACTER*6 PDCODE
          COMMON /DISCHK/ PDCODE
          COMMON /SCHSZ1/ NEL, NPTL, NNPDE, NNPTS, MM, NV, NXI, NVST
          COMMON /SCHSZ/  IA(3),I5,IB(3),I9,I10,IC(2),I11,ID(9),I19
          SAVE   /SCHSZ1/, /SCHSZ/, /DISCHK/
          IF(PDCODE .NE. 'C0CHEB')THEN
             IFLAG = 1
             GOTO 70
          END IF
          IFLAG = 0
          ITYPE = MIN0(3,ITYPE)
          ITYPE = MAX0(1,ITYPE)
          IBK   = NEL + 1
          IV = NVST
          IF(NV .EQ. 0)IV = NPDE*NPTS
C
C    CALL THE INTERPOLATION ROUTINE.
C
          CALL INTCHE( NP, XP, UP, ITYPE, U, NPTS, NPDE, NEL, NPTL,
     1                 WK, WK(I10), WK(I5), IBK, IFLAG, NV, U(IV),
     2                 UDOT(IV), WK(I11), T, IRES)
 70       CONTINUE
          RETURN
          END
C   FUNCTION FOR L2 CHEBY NORM
C
       DOUBLE PRECISION FUNCTION CNORM(U,N)
       INTEGER N,I
       DOUBLE PRECISION SUM,U(N),PI,WT
C
       PI = 3.141592653589793D0
       WT = 1.0D0
       SUM = 0.0D0
       DO 10 I = 1,N
          SUM = SUM + U(I)**2*WT
          IF(I.EQ.1)WT = 0.5D0
10     CONTINUE
       CNORM = SUM/PI
       RETURN
       END
        SUBROUTINE JACHEB
C       DUMMY ROUTINE TO SATISFY THE LOADER
        RETURN
        END
      SUBROUTINE SGEMON(NEQN, T, HLAST, H, Y, YDOT, YSAVE, NYH, R, ACOR,
     1       RESWK, NRESWK, WKMON, NWKMON, IMON, INLN, HMIN, HMAX)
C**********************************************************************
C    DUMMY MONITOR ROUTINE FOR COLLOCATION CODE
C**********************************************************************
      INTEGER NEQN, NYH, NRESWK, NWKMON, IMON, INLN
      DOUBLE PRECISION T, HLAST, H, Y(1), YDOT(1), YSAVE(NYH,1), R(1),
     1        ACOR(NYH,1), RESWK(NRESWK), WKMON(NWKMON), HMIN,HMAX,TOLD
       RETURN
       END
          SUBROUTINE INTCHE( NP, XP, UP, ITYPE, U, NPTS, NPDE, NEL,
     1        NPTL, OMEGA, COEFF, XBK, IBK, IFLAG, NV, V, VDOT, RT,T,IR)
C***********************************************************************
C         PARAMETER LIST
C         **************
C         XP(NP)         THE MESH POINTS AT WHICH INTERPOLATED VALUES
C                        ARE REQUIRED. THESE POINTS SUCH BE IN
C                        INCREASING ORDER.
C         UP(NPDE,NP,ITYPE)  ARRAY THAT HOLDS THE VALUES FOUND BY
C                            INTERPOLATION.
C         IF ITYPE >= 1  UP(J,K,1) HOLDS THE SOLUTION VALUE AT MESH
C                                  POINT XP(K) FOR JTH PDE
C         IF ITYPE >= 2  UP(J,K,2) HOLDS THE SPACE DERIV OF THE SOLUTION
C                                  AT POINT XP(K) FOR JTH PDE.
C         IF ITYPE >= 3  UP(J,K,3) HOLDS THE FLUX R(..) AT THE POINT
C                                  XP(K) FOR THE JTH PDE.
C
C         U(1..NEQN)     ORIGINAL SOLUTION VECTOR FROM THE ODE CODE.
C
C         NPTS           THE NUMBER OF MESH POINTS USED IN COMPUTING U.
C         NPDE           THE NUMBER OF PDES IN THE PROBLEM.
C         NEL            THE NUMBER OF SPATIAL ELEMENTS IN THE MESH.
C         NPTL           THE NUMBER OF MESH POINTS PER ELEMENT.
C                        THEREFORE NPTS = NEL*(NPTL-1) + 1
C         OMEGA          MATRIX USED IN MAPPING FROM THE SOLUTION ON A
C                        SPATIAL INTERVAL TO ITS CHEBYSHEV COEFFS.
C         COEFFS         WORKSPACE USED TO HOLD THESE COEFFS.
C         XBK(IBK)       ARRAY USED TO HOLD THE BREAKPOINTS BETWEEN THE
C                        SPATIAL ELEMENTS.
C         IFLAG          ERROR FLAG SET TO 0 UNLESS EXTRAPOLATION IS
C                        TRIED AND THEN SET TO 1.
C         NV             THE SIZE OF THE ADDITIONAL ODE SYSTEM THAT IS
C                        COUPLED TO THE PDE SYSTEM.
C         V(NV)          COUPLED ODE VARIABLES
C         VDOT(NV)       AND THEIR TIME DERIVS.
C         T              THE CURRENT VALUE OF THE TIME VARIABLE.
C   NOTE --- THESE LAST FOUR VARIABLES ARE ONLY USED IF ITYPE = 3
C   ****     OTHERWISE DUMMY VARIABLES MAY BE PASSED ACROSS.
C         IR ; IRES PARAM TO TEST FOR ILLEGAL VALUES
C                THE METHOD USED IS DECOMPOSITION OF THE SOLUTION
C         PER ELEMENT INTO CHEBYSHEV COEFFICIENTS. THIS IS DONE BY
C         MATRIX MULTIPLICATION USING THE OMEGA MATRIX .  F.F.T.
C         COULD ALSO BE USED. INTERPOLATION IS USED TO PROVIDE THOSE
C         SOLUTION VALUES IN THE ELEMENT (USING CLENSHAWS ALGORITHM).
C
C***********************************************************************
          INTEGER NP, ITYPE, NPTS, NPDE, NPTL, NEL, IBK, IFLAG, NV, IONE
          INTEGER ITRACE, IDEV, I, J, K, NM1, IX, IY, IP, II, IR, IP1,
     1            IZ, IOVFLO
          DOUBLE PRECISION XP(NP), UP(NPDE,NP,1), RT(NPDE,NPTL,3),
     1            U(NPDE,1), COEFF(NPDE,NPTL,2), OMEGA(NPTL,NPTL),
     2            XBK(IBK), V(1), VDOT(1), T, D1
          DOUBLE PRECISION BR1, BR2, BR, AL, XCON(2), TEM, TEM1
          COMMON /SCONS1/ D1, TWOU, IOVFLO
          SAVE
C
C  TREAT EACH ELEMENT SEPARATELY
C
          TEM =  1.0D0 + TWOU
          TEM1 = 1.0D0 - TWOU
          IONE = 1
          IP = 0
          NM1 = NPTL-1
          IZ  = 0
          DO 60 I = 1,NEL
             IP1 = I+1
             IF( XBK(I) .GT.(XBK(I+1)*TEM1-TWOU))THEN
                CALL SERROR(' INTERC ROUTINE BREAKPOINT NUMBER (=I1)
     1          WITH VALUE (=R1) IS TOO CLOSE OR LARGER THAN BREAK
     2          POINT NO (=I2) WITH VALUE (=R2). INCORRECT CALL TO
     3          INTERC ASSUMED OR WORKSPACE CORRUPTED', 1, 2, I,IP1, 2,
     4          XBK(I), XBK(IP1))
                GOTO 70
             END IF
10           IP = IP+1
             IF(IP     .EQ. (NP+1))  GOTO 70
             IF(XP(IP) .LT. (XBK(I) * TEM1 - TWOU ))GOTO 10
             IF(XP(IP) .GT. (XBK(I+1) * TEM + TWOU))GOTO 50
             IF(XP(IP) .GT. (XBK(I+1) * TEM1 - TWOU))THEN
                IF( I .LT. NEL .AND. ITYPE .GE.2)IZ = 1
C                  IZ = 1 MEANS THAT WEIGHTED AVERAGE MUST BE USED
C                  FOR DERIVATIVE VALUES THAT ARE REQUESTED AT XBK(I+1)
             END IF
C***********************************************************************
C          PROCESS A SEQUENCE OF XP(J) VALUES IN ELEMENT I             *
C          IX = START OF CORRECT PART OF SOLUTION VECTOR U             *
C          FORM THE CHEBYSHEV COEFFS IN THE ARRAY COEFF.               *
C***********************************************************************
             IX = NM1*(I-1)
             DO 15 K = 1,NPDE
                DO 15 J = 1,NPTL
                   COEFF(K,J,1) = 0.0D0
                   DO 15 II = 1,NPTL
 15                   COEFF(K,J,1) = COEFF(K,J,1)+OMEGA(J,II)*U(K,IX+II)
C
C            FORM THE CHEBYSHEV COEFFS OF THE SPACE DERIV.
C
             IF(ITYPE .GE. 2)THEN
                DO 17 K = 1,NPDE
                   COEFF(K,NPTL,2) = 0.0D0
                   COEFF(K,NPTL-1,2) = 2.0D0 * NM1 * COEFF(K,NPTL,1)
                   DO 16 J = 2,NM1
                      COEFF(K,NPTL-J,2) = COEFF(K,NPTL-J+2,2) +
     1                                    COEFF(K,NPTL-J+1,1)*2*(NPTL-J)
 16                CONTINUE
                   COEFF(K,1,2) = COEFF(K,1,2) * 0.5D0
 17             CONTINUE
             END IF
             XCON(1) = 2.0D0 / (XBK(I+1) - XBK(I))
             XCON(2) = -0.5D0*XCON(1)*(XBK(I+1) + XBK(I))
             IY = MIN0(2,ITYPE)
20           DO 40 II = 1,IY
             DO 40  K = 1,NPDE
                BR1 = 0.0D0
                BR2 = 0.0D0
C
C   COEFF(K,NPTL) IS THE NPTL-TH  COEFF OF SOLUTION OF PDE K
C
                AL = (XP(IP)*XCON(1)+XCON(2))*2.0D0
                BR = COEFF(K,NPTL,II)
                DO 30 J = 1,NM1
                   BR2 = COEFF(K,NPTL-J,II)+AL*BR-BR1
                   BR1 = BR
                   BR  = BR2
30              CONTINUE
                IF(II .EQ. 1)THEN
                   UP(K,IP,II) = BR - BR1*AL*0.5D0
                ELSE IF(IZ .LT. 2)THEN
                   UP(K,IP,II) = (BR - BR1 * AL * 0.5)*XCON(1)
                ELSE
                   UP(K,IP,II) = 1.D0 /( XBK(I+1) - XBK(I-1))*
     1                          (UP(K,IP,II) * (XBK(I) - XBK(I-1)) +
     2            (BR - BR1 * AL * 0.5)*XCON(1) * (XBK(I+1) - XBK(I)) )
                END IF
40           CONTINUE
C
C            IF REQUIRED FORM THE FLUX AT THE INTERPLOATED POINTS
C            (UNLESS DERIV IS BEING FORMED BY WEIGHTED AVERAGE IN
C             WHICH CASE WAIT UNTIL THE FORMATION IS COMPLETE.
C
44           IF(ITYPE .GE. 3 .AND. IZ .NE. 1)THEN
C               ZERO WORKSPACES USED IN THE FLUX CALL.
                DO 47 J = 1, 3
                   DO 47 K = 1,NPDE
 47                   RT(K,1,J) = 0.0D0
                IR = 1
C               FORM THE FLUX AT THE INTERPOLATED POINTS.
                CALL SPDEFN( T, XP(IP), IONE , NPDE, UP(1,IP,1),
     1               UP(1,IP,2), RT(1,1,1), RT(1,1,2),
     2               RT(1,1,3), UP(1,IP,3), NV, V, VDOT, IR)
                IF( IR.NE.1)THEN
                  CALL SERROR(' ROUTINE SPDEFN SET IRES (=I1) WHEN
     1            CALLED FROM THE INTERPOLATION ROUTINE TO CALCULATE
     2            THE FLUX VALUES ', 1, 1, IR, 0, 0, 0.0D0, 0.0D0)
                  GOTO 70
                END IF
             END IF
           IF(IP.EQ.NP)GOTO 60
           IP = IP+1
           IF( IZ .EQ. 1)THEN
               IZ = 2
               GOTO 50
C                TO CALCULATE THE OTHER ELEMENTS CONTRIBUTION TO DERIV.
           END IF
           IF(IZ .EQ. 2)IZ = 0
           IF(XP(IP) .LT. (XBK(I+1)*TEM1 -TWOU) )THEN
C             PROCESS ANOTHER POINT IN THIS ELEMENT
              GOTO 20
           ELSE IF(XP(IP) .LT. (XBK(I+1)*TEM +TWOU) )THEN
                IF( (I+1) .LT. NEL .AND. ITYPE .GE.2)IZ = 1
C                  IZ = 1 MEANS THAT WEIGHTED AVERAGE MUST BE USED
C                  FOR DERIVATIVE VALUES THAT ARE REQUESTED AT XBK(I+1)
                GOTO 20
           END IF
50         IP = IP - 2
60       CONTINUE
         RETURN
70       IFLAG =  1
         RETURN
C---------END OF INTCHE--------------------------------------------
C
         END
         SUBROUTINE INTERC( XP, UP, NP, U, NEQ, NPDE, IFLAG,
     1                      ITYPE, WK, IWK)
C********************************************************************
C        ROUTINE TO PROVIDE VALUES OF THE SOLUTION AND POSSIBLY THE
C        FIRST DERIV IN SPACE AND THE FLUX ON THE MESH XP(NP).
C
C        INPUT VALUES
C       --------------
C        NPDE     NO OF PARABOLIC EQUATIONS
C        NPTS     NO OF SPATIAL MESH POINTS USED IN TIME INTEGRATION.
C        NP       THE NUMBER OF SPATIAL INTERPOLATION POINTS
C        XP(NP)   ARRAY CONTAINING THE SPATIAL INTERPOLATION POINTS
C                 WE ASSUME THAT
C                    XP(I) <  XP(I+1)  ,  I = 1,...,NP-1
C        UP(NPDE,NP,ITYPE)  EMPTY ARRAY FOR THE INTERPOLATED VALUES AT
C                           THE CURRENT TIME LEVEL
C        U(NPDE,NPTS) THE CURRENT SOLUTION VECTOR COMPUTED BY THE ODE
C                  TIME INTEGRATOR
C        IFLAG          ERROR FLAG       = 0 ON SUCCESSFUL RETURN
C                                        = 1 IF EXTRAPOLATION TRIED.
C                                        = 2 IF WORKSPACE NOT INITIAL
C                                               ISED ON ENTRY BY INITCC
C        ITYPE = 1  ONLY THE SOLUTION IS OUTPUT IN THE ARRAY UP
C                   UP(J,K,1) HOLDS U(XP(K),T) FOR PDE J
C                2  AS FOR 1 BUT THE FIRST DERIV IS ALSO OUTPUT.
C                   UP(J,K,2) HOLDS D/DX U(XP(K),T).
C                INCORRECT VALUES OF ITYPE ARE HANDLED BY SETTING
C        ITYPE = MAX0(1,MIN0(2,ITYPE))
C
C
C        WK(IWK) THE WORKSPACE USED BY THE CHEBYSHEV METHOD.
C
C**********************************************************************
          INTEGER NPDE, NP, IBK, IFLAG, NEL, NPTL, IC, NPTS, ITYPE, IWK,
     1            ID, IA, I5, IB, I9, I19, I11, I10, K, I, J,
     2            NNPDE, NNPTS, MM, NV, NXI, NVST, IOVFLO, NEQ
          DOUBLE PRECISION XP(NP), UP(NPDE,NP,ITYPE), U(NEQ),
     1                     WK(IWK), TWOU, TEMP, D1
          CHARACTER*6 PDCODE
          COMMON /DISCHK/ PDCODE
          COMMON /SCHSZ1/ NEL, NPTL, NNPDE, NNPTS, MM, NV, NXI, NVST
          COMMON /SCHSZ/  IA(3),I5,IB(3),I9,I10,IC(2),I11,ID(9),I19
          COMMON /SCONS1/ D1, TWOU, IOVFLO
          SAVE   /SCHSZ1/, /SCHSZ/, /DISCHK/, /SCONS1/
          IF(PDCODE .NE. 'C0CHEB')THEN
             IFLAG = 1
             GOTO 70
          END IF
          IFLAG = 0
          ITYPE = MIN0(2,ITYPE)
          ITYPE = MAX0(1,ITYPE)
          IBK   = NEL + 1
C
C   TEST THE INTERPOLATION POINTS XP(NP) TO ENSURE THAT THEY ARE IN
C   INCREASING ORDER AND THAT IF ITYPE = 2 (DERIVATIVES REQUIRED) THE
C   POINTS DO NOT CONFLICT WITH THE BREAK-POINTS.
C
          DO 10 I = 2,NP
            TEMP =  XP(I) - XP(I-1)
            IF( TEMP .LE. TWOU )THEN
                CALL SERROR(' INTERC ROUTINE CALLED WITH INTERP.
     1          POINTS NOT IN STRICTLY INCREASING ORDER I.E. COMPONENT
     2          NO (=I1) WITH VALUE (=R1) IS GREATER THAN COMPONENT
     3          ( =I2) WITH VALUE (=R2).', 1, 2, J, I, 2, XP(J), XP(I))
            END IF
 10      CONTINUE
         IF(ITYPE .GE. 2 .AND. IBK .GT. 2)THEN
            DO 20 I = 1,NP
              DO 20 J = 2, NEL
               TEMP =  DABS(XP(I) - WK(I5-1+J))
               IF( TEMP .LE. TWOU )THEN
                K = I5+J-1
                CALL SERROR(' INTERC ROUTINE CALLED WITH ITYPE = 2 AND
     1          INTERP. POINTS EQUAL TO BREAK-POINTS I.E. COMPONENT
     2          NO (=I1) WITH VALUE (=R1) IS CLOSE TO BREAK POINT
     3          (=I2) WITH VALUE (=R2).', 1, 2, I, J, 2, XP(I),WK(K))
               END IF
 20         CONTINUE
         END IF
C
C    CALL THE INTERPOLATION ROUTINE.
C
          CALL INTRCH( NP, XP, UP, ITYPE, U, NPTS, NPDE, NEL, NPTL,
     1                 WK, WK(I10), WK(I5), IBK, IFLAG)
 70       CONTINUE
          RETURN
          END
C***********************************************************************
C
          SUBROUTINE INTRCH( NP, XP, UP, ITYPE, U, NPTS, NPDE, NEL,
     1        NPTL, OMEGA, COEFF, XBK, IBK, IFLAG)
C
C***********************************************************************
C         PARAMETER LIST
C         **************
C         XP(NP)         THE MESH POINTS AT WHICH INTERPOLATED VALUES
C                        ARE REQUIRED. THESE POINTS SUCH BE IN
C                        INCREASING ORDER.
C         UP(NPDE,NP,ITYPE)  ARRAY THAT HOLDS THE VALUES FOUND BY
C                            INTERPOLATION.
C         IF ITYPE >= 1  UP(J,K,1) HOLDS THE SOLUTION VALUE AT MESH
C                                  POINT XP(K) FOR JTH PDE
C         IF ITYPE >= 2  UP(J,K,2) HOLDS THE SPACE DERIV OF THE SOLUTION
C                                  AT POINT XP(K) FOR JTH PDE.
C
C         U(1..NEQN)     ORIGINAL SOLUTION VECTOR FROM THE ODE CODE.
C
C         NPTS           THE NUMBER OF MESH POINTS USED IN COMPUTING U.
C         NPDE           THE NUMBER OF PDES IN THE PROBLEM.
C         NEL            THE NUMBER OF SPATIAL ELEMENTS IN THE MESH.
C         NPTL           THE NUMBER OF MESH POINTS PER ELEMENT.
C                        THEREFORE NPTS = NEL*(NPTL-1) + 1
C         OMEGA          MATRIX USED IN MAPPING FROM THE SOLUTION ON A
C                        SPATIAL INTERVAL TO ITS CHEBYSHEV COEFFS.
C         COEFFS         WORKSPACE USED TO HOLD THESE COEFFS.
C         XBK(IBK)       ARRAY USED TO HOLD THE BREAKPOINTS BETWEEN THE
C                        SPATIAL ELEMENTS.
C         IFLAG          ERROR FLAG SET TO 0 UNLESS EXTRAPOLATION IS
C                        TRIED AND THEN SET TO 1.
C
C                THE METHOD USED IS DECOMPOSITION OF THE SOLUTION
C         PER ELEMENT INTO CHEBYSHEV COEFFICIENTS. THIS IS DONE BY
C         MATRIX MULTIPLICATION USING THE OMEGA MATRIX .  F.F.T.
C         COULD ALSO BE USED. INTERPOLATION IS USED TO PROVIDE THOSE
C         SOLUTION VALUES IN THE ELEMENT (USING CLENSHAWS ALGORITHM).
C
C***********************************************************************
          INTEGER NP, ITYPE, NPTS, NPDE, NPTL, NEL, IBK, IFLAG, NV, IONE
          INTEGER I, J, K, NM1, IX, IY, IP, II, IP1, IZ, IOVFLO
          DOUBLE PRECISION XP(NP), UP(NPDE,NP,1), TEM2, D1,
     1            U(NPDE,1), COEFF(NPDE,NPTL,2), OMEGA(NPTL,NPTL), CU,
     2            XBK(IBK), BR1, BR2, BR, AL, XCON(2), TWOU, TEM, TEM1
          COMMON /SCONS1/ D1, TWOU, IOVFLO
          SAVE
C
C  TREAT EACH ELEMENT SEPARATELY
C
          CU  =  TWOU
          TEM1= 1 - CU
          TEM = 1 + CU
          IONE = 1
          IP = 0
          NM1 = NPTL-1
          IZ  = 0
          DO 60 I = 1,NEL
             IP1 = I+1
             IF( XBK(I) .GT.(XBK(I+1)*TEM1- CU))THEN
                CALL SERROR(' INTERC ROUTINE BREAKPOINT NUMBER (=I1)
     1          WITH VALUE (=R1) IS TOO CLOSE OR LARGER THAN BREAK
     2          POINT NO (=I2) WITH VALUE (=R2). INCORRECT CALL TO
     3          INTERC ASSUMED OR WORKSPACE CORRUPTED', 1, 2, I,IP1, 2,
     4          XBK(I), XBK(IP1))
                GOTO 70
             END IF
10           IP = IP+1
             IF(IP     .EQ. (NP+1))             GOTO 70
             IF(XP(IP) .LT.(XBK(I)*TEM1- CU) ) GOTO 10
             IF(XP(IP) .GT.(XBK(I+1)*TEM+ CU)) GOTO 50
             IF(XP(IP) .GE. (XBK(I+1) * TEM1 -  CU))THEN
                IF( I .LT. NEL .AND. ITYPE .GE.2)IZ = 1
C                  IZ = 1 MEANS THAT WEIGHTED AVERAGE MUST BE USED
C                  FOR DERIVATIVE VALUES THAT ARE REQUESTED AT XBK(I+1)
             END IF
C***********************************************************************
C          PROCESS A SEQUENCE OF XP(J) VALUES IN ELEMENT I             *
C          IX = START OF CORRECT PART OF SOLUTION VECTOR U             *
C          FORM THE CHEBYSHEV COEFFS IN THE ARRAY COEFF.               *
C***********************************************************************
             IX = NM1*(I-1)
             DO 15 K = 1,NPDE
                DO 15 J = 1,NPTL
                   COEFF(K,J,1) = 0.0D0
                   DO 15 II = 1,NPTL
 15                   COEFF(K,J,1) = COEFF(K,J,1)+OMEGA(J,II)*U(K,IX+II)
C
C            FORM THE CHEBYSHEV COEFFS OF THE SPACE DERIV.
C
             IF(ITYPE .GE. 2)THEN
                DO 17 K = 1,NPDE
                   COEFF(K,NPTL,2) = 0.0D0
                   COEFF(K,NPTL-1,2) = 2.0D0 * NM1 * COEFF(K,NPTL,1)
                   DO 16 J = 2,NM1
                      COEFF(K,NPTL-J,2) = COEFF(K,NPTL-J+2,2) +
     1                                    COEFF(K,NPTL-J+1,1)*2*(NPTL-J)
 16                CONTINUE
                   COEFF(K,1,2) = COEFF(K,1,2) * 0.5D0
 17             CONTINUE
             END IF
             XCON(1) = 2.0D0 / (XBK(I+1) - XBK(I))
             XCON(2) = -0.5D0*XCON(1)*(XBK(I+1) + XBK(I))
             IY = MIN0(2,ITYPE)
20           DO 40 II = 1,IY
             DO 40  K = 1,NPDE
                BR1 = 0.0D0
                BR2 = 0.0D0
C
C   COEFF(K,NPTL) IS THE NPTL-TH  COEFF OF SOLUTION OF PDE K
C
                AL = (XP(IP)*XCON(1)+XCON(2))*2.0D0
                BR = COEFF(K,NPTL,II)
                DO 30 J = 1,NM1
                   BR2 = COEFF(K,NPTL-J,II)+AL*BR-BR1
                   BR1 = BR
                   BR  = BR2
30              CONTINUE
                IF(II .EQ. 1)THEN
                   UP(K,IP,II) = BR - BR1*AL*0.5D0
                ELSE IF(IZ .LT. 2)THEN
                   UP(K,IP,II) = (BR - BR1 * AL * 0.5)*XCON(1)
                ELSE
                   UP(K,IP,II) = 1.D0 /( XBK(I+1) - XBK(I-1))*
     1                          (UP(K,IP,II) * (XBK(I) - XBK(I-1)) +
     2            (BR - BR1 * AL * 0.5)*XCON(1) * (XBK(I+1) - XBK(I)) )
                END IF
40           CONTINUE
           IF(IP.EQ.NP)GOTO 60
           IP = IP+1
           IF( IZ .EQ. 1)THEN
               IZ = 2
               GOTO 50
C                TO CALCULATE THE OTHER ELEMENTS CONTRIBUTION TO DERIV.
           END IF
           IF(IZ .EQ. 2)IZ = 0
           IF(XP(IP) .LE. (XBK(I+1)*TEM1 -  CU))THEN
              GOTO 20
           ELSE IF(XP(IP) .LE. (XBK(I+1)*TEM + CU) )THEN
                IF( I .LT. NEL .AND. ITYPE .GE.2)IZ = 1
C                  IZ = 1 MEANS THAT WEIGHTED AVERAGE MUST BE USED
C                  FOR DERIVATIVE VALUES THAT ARE REQUESTED AT XBK(I+1)
                 GOTO 20
           END IF
50         IP = IP - 2
60       CONTINUE
         RETURN
70       IFLAG = 1
         RETURN
C---------END OF INTRCH-------------------------------------------
C
         END
