      SUBROUTINE DCLQEST(Z,IZ,NX,NV,NY,DEPS,INFO)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DCLQEST                                             c
C   AUTHOR : DAVID DI RUSCIO        ITK/NTH   19. november 1989        c
C   PURPOSE                                                            c
C   DCLQEST (Double prec. Continuous. LQ- ESTimator syntesis)          c
C   DCLQEST solves the Linear Quadratic estimator syntesis problem.    c
C   the algebraic Riccati equation (ARE) is solved directly by Laubs   c
C   Schur method, thereby gaining substanital numerical advantages.    c
C   See Laub (1979). IEEE Trans. on Aut. Cont.,vol AC-24,pp.913-921.   c
C                                                                      c
C   GENERAL REMARKS ABOUT THE ARRAY Z.                                 c
C   Z      DOUBLE PRECISION(*) array. Z is used to hold the system     c
C          matrices A, D, W, V and C (arguments) at input and Z holds  c
C          the results X, K and ER,EI at output, furthermore Z is used c
C          as work space. Information about pointers is described      c
C          below. Z must be dimensioned at least                       c
C          dimZ = 12*NX*NX + NY*NX + 3*NV*NX + 6*NX + NY*NY + NV*NV + 1c
C          The matrices is stacked row to row in Z, ie. in the normal  c
C          way FORTRAN stores matrices.                                c
C          EXAMPLE.                                                    c
C          A (n x m) matrix X is stacked in Z as follow.               c
C          DO 10 J=1,M                                                 c
C             DO 20 I=1,N                                              c
C                IN   = I + (J-1)*N                                    c
C                Z(IN) = X(I,J)                                        c
C    10       CONTINUE                                                 c
C    20    CONTINUE                                                    c
C                                                                      c
C   ON INPUT                                                           c
C   NX     INTEGER state dimension.                                    c
C   NV     INTEGER dimension on system disturbances.                   c
C   NY     INTEGER dimension on system measurements.                   c
C   Z      DOUBLE PRECISION(*) array containing the system matrixes    c
C          A, D, W, V and C. The matrices is unchanged at output.      c
C           The matrices is stored in Z as follow.                     c
C          (pointers)         (matrices)                               c
C          IDA = 1            Z(IDA) = A  -> system matrix             c
C          IDD = IDA + NX*NX  Z(IDD) = D  -> measurements matrix       c
C          IDW = IDD + NY*NX  Z(IDW) = W  -> covariance measurements   c
C          IDV = IDW + NY*NY  Z(IDV) = V  -> covariance disturbances   c
C          IDC = IDV + NV*NV  Z(IDC) = C  -> disturbance matrix        c
C   IZ     INTEGER(*) work vector. IZ must be dimensioned at least     c
C          NX + NX.                                                    c
C   DEPS   DOUBLE precision machine precision. Ie. the smallest        c
C          number such that 1.0D0 + DEPS .GE. 1.0D0. (Typically        c
C          DEPS = 1.0E-16)                                             c
C                                                                      c
C    ON OUTPUT                                                         c
C    Z    DOUBLE PRECISION(*) array containing the LQ-solutions        c
C         X, K, ER and EI. ER and EI is the real and imaginary parts   c
C         of the closed loop eigenvalues, respectively. Ie. the        c
C         eigenvalues of (A-BG) or (A+HR) where H=B*invP*trB.          c
C         The results is stored in Z as follow.                        c
C         (pointers)           (matrices)                              c
C         IDX = IDC + NX*NV    Z(IDX) = X  --> Solution of ARE.        c
C         IDK = IDX + NX*NX    Z(IDK) = K  --> Kalman filter gain.     c
C         IER = IDK + NX*NY    Z(IER) = ER --> real part of EIG(A-KD)  c
C         IEI = IER + NX       Z(IEI) = EI --> imag part of EIG(A-KD)  c
C    INFO INTEGER error signal parameter.                              c
C         INFO = 0  ---> Successfull operation.                        c
C         INFO = 1  ---> Eigenvalue(s) not proper isolated (HQR3).     c
C         INFO = 2  ---> Reordering eigenspectrum failed (EIGORD).     c
C         INFO = 3  ---> Linear system of equation singular, ie.       c
C                        system tr(U11) X = tr(U21) singular (DRICSOL).c
C                        This messages indicate that the canonical     c
C                        system is singular to working precision, and  c
C                        that the system A-KD has eigenvalues on the   c
C                        imaginary axis.                               c
C                                                                      c
C   ROUTINES CALLED: DINV, DMAMUL, DTRMUL, DMATRP,DMACOP, DRICSOL      c
C                                                                      c
C***END PROLOGUE DCLQEST                                               c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER IZ(*),NX,NV,NY,INFO                                        
      DOUBLE PRECISION Z(*),DEPS
C...Local par.
      INTEGER NXX,NVX,NVV,NYX,NYY,NP,IDA,IDD,IDW,IDV,IDC,IDQ
      INTEGER IW1,IW2,IW3,IW4,IW5,IW7,IW8,IDX,IDK,IER,IEI
C... Common block for debug purpose
C...(IPRT=1 will print info during computation to fortran file unit IOUT)
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

C1.. Initialize some dimensions
      NXX = NX*NX
      NVX = NV*NX
      NVV = NV*NV
      NYX = NY*NX
      NYY = NY*NY
      NP  = 4*NXX

C2.. Initialize all pointers for input matrixes and work space
C... Set pointers to system matrixes A, D, W, V and C in array Z(*)
      IDA = 1
      IDD = IDA + NXX
      IDW = IDD + NYX
      IDV = IDW + NYY    
      IDC = IDV + NVV    
C... Pointers for arrays used as work space when computing tr(A),
C    tr(D)*inv(W), H = tr(D)*inv(W)*D and Q = C*V*tr(C)
      IWA = IDC + NVX
      IDQ = IWA + NXX
      IW1 = IDQ + NXX
      IW2 = IW1 + NYX
      IW3 = IW2 + NVX
C... Pointers for F, U (arrays involving the Hamiltonian) DRICSOL
      IW4 = IW3 + NXX
      IW5 = IW4 + NP
C... Pointers for eigenvalue array (ER , EI) in DRICSOL
      IW6 = IW5 + NP
      IW7 = IW6 + NX + NX
C... Work vector in DRICSOL
      IW8 = IW7 + NX + NX

C... DEBUG                              
      IF (IPRT.EQ.1) THEN
       IN = IW8 + NX + NX
       MZ = 12*NX*NX + 2*NY*NX + 2*NV*NX + 6*NX + NY*NY + NV*NV + 1
       WRITE(IOUT,'(1X,A,I4)')'DIMENSION dimZ     =',MZ
       WRITE(IOUT,'(1X,A,I4)')'DIM. IW8 + NX + NX =',IN
       CALL DMAPRT('DDREST A =',Z(IDA),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('DDREST D =',Z(IDD),NY,NX,NY,NX,IOUT)
       CALL DMAPRT('DDREST W =',Z(IDW),NY,NY,NY,NY,IOUT)
       CALL DMAPRT('DDREST V =',Z(IDV),NV,NV,NV,NV,IOUT)
       CALL DMAPRT('DDREST C =',Z(IDC),NX,NV,NX,NV,IOUT)
      ENDIF

C3.. Initialize Z(IW2) = tr(D)*inv(W)  and  H = tr(D)*inv(W)*D
C... Compute inv(W) --> Z(IW4) 
      CALL DINV(NY,Z(IDW),Z(IW4),IZ,NY,NY,INFO)
      IF (INFO.NE.0) THEN
       INFO = 4
       GO TO 1000
      ENDIF
      
C... DEBUG
      IF (IPRT.EQ.1) THEN
       CALL DMAPRT('DDREST inv(W)  =',Z(IW4),NY,NY,NY,NY,IOUT)
      ENDIF

C... Compute tr(D)*inv(W) ---> Z(IW1)
      CALL DTRMUL(Z(IDD),Z(IW4),Z(IW1),NX,NY,NY,NY,NY,NX,2)

C... Compute H = tr(D)*inv(W)*D ---> Z(IW3)
      CALL DMAMUL(Z(IW1),Z(IDD),Z(IW3),NX,NY,NX,NX,NY,NX)

C4.. Compute  Q = C*V*tr(C)
      CALL DTRMUL(Z(IDV),Z(IDC),Z(IW2),NV,NV,NX,NV,NX,NV,1)
      CALL DMAMUL(Z(IDC),Z(IW2),Z(IDQ),NX,NV,NX,NX,NV,NX)

C... Put tr(A) in Z(IWA)
      CALL DMATRP(Z(IDA),Z(IWA),NX,NX,NX,NX)

C... DEBUG
      IF (IPRT.EQ.1) THEN
       CALL DMAPRT('CALL DRICSOL tr(A) =',Z(IWA),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('CALL DRICSOL H     =',Z(IW3),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('CALL DRICSOL Q     =',Z(IDQ),NX,NX,NX,NX,IOUT)
      ENDIF

C4.. Solve Riccati equation by Laubs Schur Method, R in Z(IW3).
      INFO = 0
      CALL DRICSOL(Z(IWA),Z(IW3),Z(IDQ),NX,NX,Z(IW4),Z(IW5),NX+NX,
     +             Z(IW6),Z(IW7),Z(IW8),IZ,DEPS,INFO)

C5.. Compute Kalman filter gain K = X*tr(D)*inv(W) --> Z(IW4)
      CALL DMAMUL(Z(IW3),Z(IW1),Z(IW4),NX,NX,NY,NX,NX,NX)

C6.. Copy results X, K, ER and EI to stack Z(*)
      IDX = IDC + NVX
      IDK = IDX + NXX
      IER = IDK + NYX
      IEI = IER + NX
      CALL DMACOP(Z(IW3),Z(IDX),NX,NX,NX,NX)
      CALL DMACOP(Z(IW4),Z(IDK),NX,NX,NX,NY)
      CALL DMACOP(Z(IW6),Z(IER),NX,NX,NX,1)
      CALL DMACOP(Z(IW7),Z(IEI),NX,NX,NX,1)

C... DEBUG
      IF (IPRT.EQ.1) THEN
       WRITE(IOUT,'(1X,A)')'OUTPUT DDREST WITH :'
       CALL DMAPRT('A =',Z(IDA),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('D =',Z(IDD),NY,NX,NY,NX,IOUT)
       CALL DMAPRT('W =',Z(IDW),NY,NY,NY,NY,IOUT)
       CALL DMAPRT('V =',Z(IDV),NV,NV,NV,NV,IOUT)
       CALL DMAPRT('C =',Z(IDC),NX,NV,NX,NV,IOUT)
       CALL DMAPRT('K =',Z(IDK),NX,NY,NX,NY,IOUT)
       CALL DMAPRT('X =',Z(IDX),NX,NX,NX,NX,IOUT)
       CALL DEIGPRT('EIG(A-KD) =',Z(IER),Z(IEI),NX,IOUT)
       WRITE(IOUT,'(1X,A,I5)')'dimZ (used) =',IEI + NX
       WRITE(IOUT,'(1X,A,G12.4)')
     + 'Condition number of the Hamiltonian matrix =',Z(IW8)
      ENDIF

1000  RETURN
      END
