      SUBROUTINE DDRSOL(Z,IZ,NX,NU,DEPS,INFO)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DDRSOL                                              c
C   AUTHOR : DAVID DI RUSCIO        ITK/NTH   november 1989            c
C   PURPOSE                                                            c
C   DDRSOL solves the Linear Quadratic controller problem. The         c
C   Riccati equation is solved directly by Laubs Schur method,         c
C   thereby gaining substanital numerical advantages.                  c
C                                                                      c
C                                                                      c
C   GENERAL REMARKS ABOUT THE ARRAY Z.                                 c
C   Z      DOUBLE PRECISION(*) array. Z is used to hold the system     c
C          matrices A, B, P and Q (arguments) at input and Z holds the c
C          results R, G and ER,EI at output, furthermore Z is used as  c
C          work space. Information about pointers is described below.  c
C          Z must be dimensioned at least                              c
C          dimZ = 11*NX*NX + NU*NU + 2*NU*NX + 6*NX + 1                c
C          The matrices is stacked row to row in Z, ie. in the normal  c
C          way FORTRAN stores matrices.                                c
C          EXAMPLE.                                                    c
C          A (n x m) matrix X is stacked in Z as follow.               c
C          DO 10 J=1,M                                                 c
C             DO 20 I=1,N                                              c
C                IN   = I + (J-1)*N                                    c
C                Z(IN) = X(I,J)                                        c
C    10       CONTINUE                                                 c
C    20    CONTINUE                                                    c
C                                                                      c
C   ON INPUT                                                           c
C   NX     INTEGER state dimension.                                    c
C   NU     INTEGER control input dimension.                            c
C   Z      DOUBLE PRECISION(*) array containing the system matrixes    c
C          A, B, P and Q. The matrices is unchanged at output.         c
C           The matrices is stored in Z as follow.                     c
C          (pointers)         (matrices)                               c
C          IDA = 1            Z(IDA) = A                               c
C          IDB = IDA + NX*NX  Z(IDB) = B                               c
C          IDP = IDB + NU*NX  Z(IDP) = P                               c
C          IDQ = IDP + NU*NU  Z(IDQ) = Q                               c
C   IZ     INTEGER(*) work vector. IZ must be dimensioned at least     c
C          NX + NX.                                                    c
C   DEPS   DOUBLE precision machine precision. Ie. the smallest        c
C          number such that 1.0D0 + DEPS .GE. 1.0D0. (Typically        c
C          DEPS = 1.0E-16)                                             c
C                                                                      c
C    ON OUTPUT                                                         c
C    Z    DOUBLE PRECISION(*) array containing the LQ-solutions        c
C         R, G, ER and EI. ER and EI is the real and imaginary parts   c
C         of the closed loop eigenvalues, respectively. Ie. the        c
C         eigenvalues of (A-BG) or (A+HR) where H=B*invP*trB.          c
C         The results is stored in Z as follow.                        c
C         (pointers)           (matrices)                              c
C         IDR = IDQ + NX*NX    Z(IDR) = R                              c
C         IDG = IDR + NX*NX    Z(IDG) = G                              c
C         IER = IDG + NU*NX    Z(IER) = ER                             c
C         IEI = IER + NX       Z(IEI) = EI                             c
C    INFO INTEGER error signal parameter.                              c
C         INFO = 0  ---> Successfull operation.                        c
C         INFO = 1  ---> Eigenvalue(s) not proper isolated (HQR3).     c
C         INFO = 2  ---> Reordering eigenspectrum failed (EIGORD).     c
C         INFO = 3  ---> Linear system of equation singular, ie.       c
C                        system tr(U11) R = tr(U21) singular (DRICSOL).c
C                                                                      c
C   ROUTINES CALLED: DINV, DTRMUL, DMAMUL, DMANEG, DMACOP, DRICSOL     c
C***END PROLOGUE DDRSOL                                                c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER IZ(*),NX,NU,INFO                                        
      DOUBLE PRECISION Z(*),DEPS
C...Local par.
      INTEGER NXX,NUX,NUU,NP,IDA,IDB,IDP,IDQ,IW2,IW3,IW4,IW5,IW7,IW8
      INTEGER IDR,IDG,IER,IEI
C.. Common block for debug purposes
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

C1.. Initialize some dimensions
      NXX = NX*NX
      NUX = NU*NX
      NUU = NU*NU
      NP  = 4*NXX

C2.. Initialize all pointers for input matrixes and work space
C... Set pointers to system matrixes A, B, P and Q in array Z(*)
      IDA = 1
      IDB = IDA + NXX
      IDP = IDB + NUX
      IDQ = IDP + NUU    
C... Pointers for arrays used as work space when computing 
C    invP*trB and H = B*invP*trB
      IW2 = IDQ + NXX
      IW3 = IW2 + NUX
C... Pointers for F, U (arrays involving the Hamiltonian) DRICSOL
      IW4 = IW3 + NXX
      IW5 = IW4 + NP
C... Pointers for eigenvalue array (ER , EI) in DRICSOL
      IW6 = IW5 + NP
      IW7 = IW6 + NX + NX
C... Work vector in DRICSOL
      IW8 = IW7 + NX + NX

C... DEBUG                              
      IF (IPRT.EQ.1) THEN
       IN = IW8 + NX + NX
       MZ = 11*NX*NX + NU*NU + 2*NU*NX + 6*NX + 1
       WRITE(IOUT,'(1X,A,I4)')'DIMENSION dimZ     =',MZ
       WRITE(IOUT,'(1X,A,I4)')'DIM. IW8 + NX + NX =',IN
       CALL DMAPRT('DDRSOL A =',Z(IDA),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('DDRSOL B =',Z(IDB),NX,NU,NX,NU,IOUT)
       CALL DMAPRT('DDRSOL P =',Z(IDP),NU,NU,NU,NU,IOUT)
       CALL DMAPRT('DDRSOL Q =',Z(IDQ),NX,NX,NX,NX,IOUT)
      ENDIF

C3.. Initialize Z(IW2) = invP*trB  and  H = B*invP*trB
C... Compute invP --> Z(IW4) 
      CALL DINV(NU,Z(IDP),Z(IW4),IZ,NU,NU,INFO)
      IF (INFO.NE.0) THEN
       INFO = 4
       GO TO 1000
      ENDIF

C... DEBUG
      IF (IPRT.EQ.1) THEN
       CALL DMAPRT('DDRSOL invP   =',Z(IW4),NU,NU,NU,NU,IOUT)
      ENDIF

C... Compute invP*trB ---> Z(IW2)
      CALL DTRMUL(Z(IW4),Z(IDB),Z(IW2),NU,NU,NX,NU,NX,NU,1)

C... Compute H = B*invP*trB ---> Z(IW3)
      CALL DMAMUL(Z(IDB),Z(IW2),Z(IW3),NX,NU,NX,NX,NU,NX)

C... DEBUG
      IF (IPRT.EQ.1) THEN
       CALL DMAPRT('CALL RICS  A =',Z(IDA),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('CALL RICS  H =',Z(IW3),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('CALL RICS  Q =',Z(IDQ),NX,NX,NX,NX,IOUT)
      ENDIF

C4.. Solve Riccati equation by Laubs Schur Method, R in Z(IW3).
      INFO = 0
      CALL DRICSOL(Z(IDA),Z(IW3),Z(IDQ),NX,NX,Z(IW4),Z(IW5),NX+NX,
     +             Z(IW6),Z(IW7),Z(IW8),IZ,DEPS,INFO)

C5.. Compute feedback matrix G = -invP*trB*R --> -Z(IW2)*Z(IW3)
      CALL DMAMUL(Z(IW2),Z(IW3),Z(IW4),NU,NX,NX,NU,NX,NX)
      CALL DMANEG(Z(IW4),NX,NU,NX)

C6.. Copy results R, G, ER and EI to stack Z(*)
      IDR = IDQ + NXX
      IDG = IDR + NXX
      IER = IDG + NUX
      IEI = IER + NX
      CALL DMACOP(Z(IW3),Z(IDR),NX,NX,NX,NX)
      CALL DMACOP(Z(IW4),Z(IDG),NX,NU,NU,NX)
      CALL DMACOP(Z(IW6),Z(IER),NX,1,NX,1)
      CALL DMACOP(Z(IW7),Z(IEI),NX,1,NX,1)

C... DEBUG
      IF (IPRT.EQ.1) THEN
       CALL DMAPRT('R =',Z(IDR),NX,NX,NX,NX,IOUT)
       CALL DMAPRT('G =',Z(IDG),NU,NX,NU,NX,IOUT)
      ENDIF

1000  RETURN
      END
